/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2007                               **
**                                                                        **
**                      MODULE ARP Protocol Service                       **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2007.03.22                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/


#include "arpservice.h"
#include "types.h"
#include "ipservice.h"
#include "frameutil.h"


#define StandardLifeTime         1280

#define ARP_request              1
#define ARP_response             2
#define RARP_request             3
#define RARP_response            4

typedef struct {
                 USHORT               HardwareType ;                           /* =  2 */
                 USHORT               ProtocolType ;                           /* =  2 */
                 USHORT               AddressLength ;                          /* =  2 */
                 USHORT               Operation ;                              /* =  2 */
                 UCHAR                SenderMACAddress [ EthMACAddressSize ] ; /* =  6 */
                 ULONG                SenderIPAddress ;                        /* =  4 */
                 UCHAR                TargetMACAddress [ EthMACAddressSize ] ; /* =  6 */
                 ULONG                TargetIPAddress ;                        /* =  4 */
               } ARPHeaderRecT ;                                               /* = 28 */
#define HardwareTypeOffset            0
#define ProtocolTypeOffset            HardwareTypeOffset + 2
#define AddressLengthOffset           ProtocolTypeOffset + 2
#define OperationOffset               AddressLengthOffset + 2
#define SenderMACAddressOffset        OperationOffset + 2
#define SenderIPAddressOffset         SenderMACAddressOffset + EthMACAddressSize
#define TargetMACAddressOffset        SenderIPAddressOffset + 4
#define TargerIPAddressOffset         TargetMACAddressOffset + EthMACAddressSize
#define ARPHeaderRecTSize             28



static UCHAR Broadcast [ 6 ] = { 0xFF , 0xFF , 0xFF , 0xFF , 0xFF , 0xFF } ;




/*******************************************************************************************
********************************************************************************************
**                                                                                        **
**                                       ARP SERVICR                                      **
**                                                                                        **
********************************************************************************************
*******************************************************************************************/


PtrARPTableElementRecT LocateMACForIP ( PtrIPLayerInstanceRecT IPLayerInstance ,
                                        ULONG                  IPAddress )
{
  ULONG Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < ARPTableSize ; Loop ++ )
  {
    if ( IPLayerInstance -> ARPTable [ Loop ] . ValidElement )
    {
      if ( IPLayerInstance -> ARPTable [ Loop ] . IPAddress == IPAddress )
      {
        IPLayerInstance -> ARPTable [ Loop ] . LifeTime = StandardLifeTime ;
        return ( & ( IPLayerInstance -> ARPTable [ Loop ] ) ) ;
      } /* if */ ;
    } /* if */ ;
  } /* for */ ;
  return NIL ;
} /* LocateMACForIP */


static void AddARPTabElement ( PtrIPLayerInstanceRecT IPLayerInstance ,
                               UCHAR *                MACAddress ,
                               ULONG                  IPAddress )
{
  ULONG Loop ;
  ULONG MinPos ;
  UCHAR * OCh ;
  UCHAR * ICh ;
  PtrARPTableElementRecT TabElement ;
  USHORT MinValue ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < ARPTableSize ; Loop ++ )
  {
    if ( IPLayerInstance -> ARPTable [ Loop ] . ValidElement )
    {
      if ( IPLayerInstance -> ARPTable [ Loop ] . IPAddress == IPAddress )
      {
        return ;
      } /* if */ ;
    } /* if */ ;
  } /* for */ ;
  TabElement = NIL ;
  for ( Loop = 0 ; Loop < ARPTableSize ; Loop ++ )
  {
    if ( ! IPLayerInstance -> ARPTable [ Loop ] . ValidElement )
    {
      TabElement = & ( IPLayerInstance -> ARPTable [ Loop ] ) ;
      break ;
    } /* if */ ;
  } /* for */ ;
  if ( ! TabElement )
  {
    MinValue = IPLayerInstance -> ARPTable [ 0 ] . LifeTime ;
    MinPos = 0 ;
    for ( Loop = 1 ; Loop < ARPTableSize ; Loop ++ )
    {
      if ( IPLayerInstance -> ARPTable [ Loop ] . LifeTime < MinValue )
      {
        MinValue = IPLayerInstance -> ARPTable [ Loop ] . LifeTime ;
        MinPos = Loop ;
      } /* if */ ;
    } /* for */ ;
    IPLayerInstance -> ARPTable [ MinPos ] . ValidElement = 0 ;
    TabElement = & ( IPLayerInstance -> ARPTable [ MinPos ] ) ;
  } /* if */ ;
  OCh = TabElement -> MACAddress ;
  ICh = MACAddress ;
  for ( Loop = 0 ; Loop < EthMACAddressSize ; Loop ++ )
    * OCh ++ = * ICh++ ;
  TabElement -> ValidElement = 1 ;
  TabElement -> LifeTime = StandardLifeTime ;
  TabElement -> IPAddress = IPAddress ;
} /* AddARPTabElement */


static void SendARPResponse ( PtrIPLayerInstanceRecT IPLayerInstance ,
                              ARPHeaderRecT *        ARPPacked )
{
  EMACInstanceRecT * InterfInstance ;
  PtrOSFrameRecT OutFrame ;
  /*-------------------------------------------------------------------------*/
  InterfInstance = IPLayerInstance -> InterfaceInstance ;
  OutFrame = InterfInstance -> InterfaceAllocFrameMemory ( sizeof ( OSFrameRecT ) ) ;
  if ( OutFrame )
  {
    OutFrame -> Index = 0 ;
    StartEthFrame ( OutFrame , ARPPacked -> SenderMACAddress , InterfInstance -> InterfaceMACAddress ,
                    FRAME_ARP ) ;
    AddUSHORTSwapField ( OutFrame , EthHardwType ) ;
    AddUSHORTSwapField ( OutFrame , FRAME_IP ) ;
    AddUSHORTSwapField ( OutFrame , IP_HLen_PLen ) ;
    AddUSHORTSwapField ( OutFrame , ARP_response ) ;
    AddByteArrayField ( OutFrame , InterfInstance -> InterfaceMACAddress , EthMACAddressSize ) ;
    AddULONGSwapField ( OutFrame , IPLayerInstance -> MyLocalIPAddress ) ;
    AddByteArrayField ( OutFrame , ARPPacked -> SenderMACAddress , EthMACAddressSize ) ;
    AddULONGSwapField ( OutFrame , ARPPacked -> SenderIPAddress ) ;
    InterfInstance -> InterfaceSendFrame ( OutFrame ) ;
    InterfInstance -> InterfaceDeallocFrameMemory ( OutFrame ) ;
  } /* if */ ;
} /* SendARPResponse */


static void RecvARPResponse ( PtrIPLayerInstanceRecT IPLayerInstance ,
                              ARPHeaderRecT *        ARPPacked )
{
  /*-------------------------------------------------------------------------*/
  AddARPTabElement ( IPLayerInstance , ARPPacked -> SenderMACAddress ,
                     ARPPacked -> SenderIPAddress ) ;
} /* RecvARPResponse */


void PrepareARPRequest ( PtrIPLayerInstanceRecT IPLayerInstance ,
                         ULONG                  DestIPAddress )
{
  EMACInstanceRecT * InterfInstance ;
  PtrOSFrameRecT OutFrame ;
  UCHAR DestMACAddress [ EthMACAddressSize ] ;
  ULONG Loop ;
  /*-------------------------------------------------------------------------*/
  InterfInstance = IPLayerInstance -> InterfaceInstance ;
  OutFrame = InterfInstance -> InterfaceAllocFrameMemory ( sizeof ( OSFrameRecT ) ) ;
  if ( OutFrame )
  {
    OutFrame -> Index = 0 ;
    for ( Loop = 0 ; Loop < EthMACAddressSize ; Loop ++ )
      DestMACAddress [ Loop ] = 0 ;
    StartEthFrame ( OutFrame , Broadcast , InterfInstance -> InterfaceMACAddress ,
                    FRAME_ARP ) ;
    AddUSHORTSwapField ( OutFrame , EthHardwType ) ;
    AddUSHORTSwapField ( OutFrame , FRAME_IP ) ;
    AddUSHORTSwapField ( OutFrame , IP_HLen_PLen ) ;
    AddUSHORTSwapField ( OutFrame , ARP_request ) ;
    AddByteArrayField ( OutFrame , InterfInstance -> InterfaceMACAddress , EthMACAddressSize ) ;
    AddULONGSwapField ( OutFrame , IPLayerInstance -> MyLocalIPAddress ) ;
    AddByteArrayField ( OutFrame , DestMACAddress , EthMACAddressSize ) ;
    AddULONGSwapField ( OutFrame , DestIPAddress ) ;
    InterfInstance -> InterfaceSendFrame ( OutFrame ) ;
    InterfInstance -> InterfaceDeallocFrameMemory ( OutFrame ) ;
  } /* if */ ;
} /* PrepareARPRequest */


void ProcessARPPacked ( PtrIPLayerInstanceRecT IPLayerInstance ,
                        PtrOSFrameRecT         Frame )
{
  ARPHeaderRecT ARPPacked ;
  /*-------------------------------------------------------------------------*/
  ARPPacked . HardwareType = GetUSHORTSwapField ( Frame , EtherPackedHeaderRecTSize + HardwareTypeOffset ) ;
  ARPPacked . ProtocolType = GetUSHORTSwapField ( Frame , EtherPackedHeaderRecTSize + ProtocolTypeOffset ) ;
  ARPPacked . AddressLength = GetUSHORTSwapField ( Frame , EtherPackedHeaderRecTSize + AddressLengthOffset ) ;
  ARPPacked . Operation = GetUSHORTSwapField ( Frame , EtherPackedHeaderRecTSize + OperationOffset ) ;
  GetByteArrayField ( ARPPacked . SenderMACAddress , Frame , EtherPackedHeaderRecTSize + SenderMACAddressOffset ,
                      EthMACAddressSize ) ;
  ARPPacked . SenderIPAddress = GetULONGSwapField ( Frame , EtherPackedHeaderRecTSize + SenderIPAddressOffset ) ;
  GetByteArrayField ( ARPPacked . TargetMACAddress , Frame , EtherPackedHeaderRecTSize + TargetMACAddressOffset ,
                      EthMACAddressSize ) ;
  ARPPacked . TargetIPAddress = GetULONGSwapField ( Frame , EtherPackedHeaderRecTSize + TargerIPAddressOffset ) ;
  if ( ARPPacked . HardwareType == EthHardwType ) 
  {
    if ( ARPPacked . ProtocolType == FRAME_IP )
    {
      if ( ARPPacked . AddressLength == IP_HLen_PLen )
      {
        switch ( ARPPacked . Operation )
        {
          case ARP_request              :
            if ( ARPPacked . TargetIPAddress == IPLayerInstance -> MyLocalIPAddress )
            {
              SendARPResponse ( IPLayerInstance , & ARPPacked ) ;
            } /* if */ ;
            break ;
          case ARP_response             :
            RecvARPResponse ( IPLayerInstance , & ARPPacked ) ;
            break ;
          case RARP_request             :
            break ;
          case RARP_response            :
            break ;
          default                       :
            break ;
        } /* switch */ ;
      } /* if */ ;
    } /* if */ ;
  } /* if */ ;
} /* ProcessARPPacked */


void DiagARPTable ( PtrIPLayerInstanceRecT IPLayerInstance )
{
  ULONG Loop ;
  USHORT LifeT ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < ARPTableSize ; Loop ++ )
  {
    if ( IPLayerInstance -> ARPTable [ Loop ] . ValidElement )
    {
      LifeT = IPLayerInstance -> ARPTable [ Loop ] . LifeTime -- ;
      if ( ! LifeT )
      {
        IPLayerInstance -> ARPTable [ Loop ] . ValidElement = 0 ;
      } /* if */ ;
    } /* if */ ;
  } /* for */ ;
} /* DiagARPTable */


void DelARPTableElement ( PtrIPLayerInstanceRecT IPLayerInstance ,
                          ULONG                  IPAddress )
{
  ULONG Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < ARPTableSize ; Loop ++ )
  {
    if ( IPLayerInstance -> ARPTable [ Loop ] . ValidElement )
    {
      if ( IPLayerInstance -> ARPTable [ Loop ] . IPAddress == IPAddress )
      {
        IPLayerInstance -> ARPTable [ Loop ] . ValidElement = 0 ;
        return ;
      } /* if */ ;
    } /* if */ ;
  } /* for */ ;
} /* DelARPTableElement */


void DelARPTable ( PtrIPLayerInstanceRecT IPLayerInstance )
{
  ULONG Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < ARPTableSize ; Loop ++ )
  {
    IPLayerInstance -> ARPTable [ Loop ] . ValidElement = 0 ;
  } /* for */ ;
} /* DelARPTableElement */

