/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                              BIALYSTOK                                 **
**                                                                        **
**                          MODULE DataService                            **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2018.06.12                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include "dataservice.h"
#include "types.h"
#include "uart1.h"
#include "strings.h"
#include "systime.h"
#include "leddisplay.h"

#define CommandBufferSize             32

DataValueType Analog1Value ;            // temperatura wody w piecu -> TMPS01 -> UCHAR Device11ID [ ]    = "TMPS01" ;
DataValueType Analog2Value ;            // temperatura wewnetrzna   -> TMPS02 -> UCHAR Device12ID [ ]    = "TMPS02" ;
DataValueType Analog3Value ;            // temperatura spalin       -> TMPS03 -> UCHAR Device13ID [ ]    = "TMPS03" ;
DataValueType Analog4Value ;            // wilgotnosc wzgledna      -> TMPS04 -> UCHAR Device14ID [ ]    = "TMPS04" ;
DataValueType Analog5Value ;            // temperatura zewnetrzna   -> TMPS05 -> UCHAR Device15ID [ ]    = "TMPS05" ;
DataValueType Analog6Value ;            // cisnienie atmosferyczne  -> TMPS06 -> UCHAR Device16ID [ ]    = "TMPS06" ;
UCHAR On0Status ;
UCHAR On1Status ;
UCHAR On2Status ;
UCHAR On3Status ;
static USHORT CommandIndex ;
static UCHAR CommandBuffer [ CommandBufferSize + 1 ] ;

const static UCHAR MyDeviceID [ ]    = "WEBS01" ;

const static UCHAR OnState [ ]       = "CHISON" ;
const static UCHAR OffState [ ]      = "CHISOF" ;

const static UCHAR Device00ID [ ]    = "COSEN1" ;

const static UCHAR Device01ID [ ]    = "PWC001" ;
const static UCHAR Device02ID [ ]    = "PWC002" ;
const static UCHAR Device03ID [ ]    = "MCTRL01" ;

const static UCHAR Device11ID [ ]    = "TMPS01" ;
const static UCHAR Device12ID [ ]    = "TMPS02" ;
const static UCHAR Device13ID [ ]    = "TMPS03" ;
const static UCHAR Device14ID [ ]    = "TMPS04" ;
const static UCHAR Device15ID [ ]    = "TMPS05" ;
const static UCHAR Device16ID [ ]    = "TMPS06" ;

const static UCHAR ChOnCommand [ ]   = "CHON" ;
const static UCHAR ChOffCommand [ ]  = "CHOFF" ;


static void DataErrorLEDOff ( void * CallParam )
{
  /*-------------------------------------------------------------------------*/
  LEDOff ( DataErrorLED ) ;
} /* DataErrorLEDOff */


static void DataError ( void )
{
  /*-------------------------------------------------------------------------*/
  AttachTimer ( ( ULONG ) 200 , DataErrorLEDOff , ( void * ) 0 ) ;
  LEDOn ( DataErrorLED ) ;
} /* DataError */ ;


static void UART1SendConstString ( const UCHAR * String )
{
  const UCHAR * ChPtr ;
  UCHAR Ch ;
  /*-------------------------------------------------------------------------*/
  ChPtr = String ;
  for ( ; ; )
  {
    Ch = * ChPtr ++ ;
    if ( ! Ch )
      break ;
    UART1Send ( Ch ) ;
  } /* for */ ;
} /* UART1SendConstString */


static USHORT CompStr ( UCHAR *       String ,
                        const UCHAR * Pattern )
{
  UCHAR Ch1 ;
  UCHAR Ch2 ;
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < CommandIndex ; Loop ++ )
  {
    Ch1 = String [ Loop ] ;
    Ch2 = Pattern [ Loop ] ;
    if ( Ch1 != Ch2 )
      return 0 ;
    if ( ! Ch1 )
      return 1 ;
  } /* for */ ;
  return 0 ;
} /* CompStr */


static UCHAR SetNewState ( UCHAR * Command )
{
  /*-------------------------------------------------------------------------*/
  if ( CompStr ( Command , OnState ) )
    return 1 ;
  if ( CompStr ( Command , OffState ) )
    return 0 ;
  return 0 ;
} /* SetNewState */


static void SetValue ( UCHAR * OutValue ,
                       UCHAR * Command )
{
  UCHAR * ChPtr ;
  UCHAR Ch ;
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  ChPtr = Command ;
  for ( ; ; )
  {
    Ch = * ChPtr ++ ;
    if ( ! Ch )
      return ;
    if ( Ch == '.' )
      break ;
  } /* for */ ;
  for ( ; ; )
  {
    Ch = * ChPtr ++ ;
    if ( ! Ch )
      return ;
    if ( Ch == '.' )
      break ;
  } /* for */ ;
  for ( Loop = 0 ; Loop < sizeof ( DataValueType ) ; Loop ++ )
  {
    Ch = * ChPtr ++ ;
    if ( ( ! Ch ) || ( Ch == '.' ) )
    {
      * OutValue ++ = 0 ;
      return ;
    } /* if */ ;
    * OutValue ++ = Ch ;
  } /* for */ ;
} /* SetValue */


static void ExecuteSerialCommand ( void )
{
//WEBS01:TMPS01:RPV.T01.23,5
  USHORT Index ;
  UCHAR Ch ;
  UCHAR * ChPtr ;
  UCHAR DeviceID [ 8 ] ;
  UCHAR Command [ 32 ] ;
  /*-------------------------------------------------------------------------*/
  Index = 0 ;
  ChPtr = CommandBuffer ;
  memclear ( DeviceID , sizeof ( DeviceID ) ) ;
  memclear ( Command , sizeof ( Command ) ) ;
  for ( ; ; )
  {
    Ch = * ChPtr ++ ;
    if ( ! Ch )
    {
      DataError ( ) ;
      return ;
    } /* if */ ;
    if ( Ch == ':' )
      break ;
  } /* for */ ;
  for ( ; ; )
  {
    Ch = * ChPtr ++ ;
    if ( ! Ch )
    {
      DataError ( ) ;
      return ;
    } /* if */ ;
    if ( Ch == ':' )
      break ;
    if ( Index < sizeof ( DeviceID ) )
      DeviceID [ Index ] = Ch ;
    Index ++ ;
  } /* for */ ;
  Index = 0 ;
  for ( ; ; )
  {
    Ch = * ChPtr ++ ;
    if ( ! Ch )
      break ;
    if ( Index < sizeof ( Command ) )
      Command [ Index ] = Ch ;
    Index ++ ;
  } /* for */ ;
  if ( CompStr ( DeviceID , Device00ID ) )
  {
    On0Status = SetNewState ( Command ) ;
    return ;
  } /* if */ ;
  if ( CompStr ( DeviceID , Device01ID ) )
  {
    On1Status = SetNewState ( Command ) ;
    return ;
  } /* if */ ;
  if ( CompStr ( DeviceID , Device03ID ) )
  {
    On3Status = SetNewState ( Command ) ;
    return ;
  } /* if */ ;
  if ( CompStr ( DeviceID , Device02ID ) )
  {
    On2Status = SetNewState ( Command ) ;
    return ;
  } /* if */ ;
  if ( CompStr ( DeviceID , Device11ID ) )
  {
    SetValue ( Analog1Value , Command ) ;
    return ;
  } /* if */ ;
  if ( CompStr ( DeviceID , Device12ID ) )
  {
    SetValue ( Analog2Value , Command ) ;
    return ;
  } /* if */ ;
  if ( CompStr ( DeviceID , Device13ID ) )
  {
    SetValue ( Analog3Value , Command ) ;
    return ;
  } /* if */ ;
  if ( CompStr ( DeviceID , Device14ID ) )
  {
    SetValue ( Analog4Value , Command ) ;
    return ;
  } /* if */ ;
  if ( CompStr ( DeviceID , Device15ID ) )
  {
    SetValue ( Analog5Value , Command ) ;
    return ;
  } /* if */ ;
  if ( CompStr ( DeviceID , Device16ID ) )
  {
    SetValue ( Analog6Value , Command ) ;
    return ;
  } /* if */ ;
  DataError ( ) ;
} /* ExecuteSerialCommand */


void SerialPool ( void )
{
  UCHAR Ch ;
  /*-------------------------------------------------------------------------*/
  if ( UART1DataPresent ( ) )
  {
    Ch = UART1GetData ( ) ;
    if ( Ch == 0x0D )
    {
      ExecuteSerialCommand ( ) ;
      CommandIndex = 0 ;
    } /* if ... */
    else
    {
      if ( CommandIndex < CommandBufferSize )
      {
        CommandBuffer [ CommandIndex ] = Ch ;
        CommandBuffer [ CommandIndex + 1 ] = 0 ;
        CommandIndex ++ ;
      } /* if */ ;
    } /* if ... else */ ;
  } /* if */ ;
} /* SerialPool */


void Device01On ( void )
{
  /*-------------------------------------------------------------------------*/
  UART1SendConstString ( Device01ID ) ;
  UART1Send ( ':' ) ;
  UART1SendConstString ( MyDeviceID ) ;
  UART1Send ( ':' ) ;
  UART1SendConstString ( ChOnCommand ) ;
  UART1Send ( 0x0D ) ;
} /* Device01On */


void Device01Off ( void )
{
  /*-------------------------------------------------------------------------*/
  UART1SendConstString ( Device01ID ) ;
  UART1Send ( ':' ) ;
  UART1SendConstString ( MyDeviceID ) ;
  UART1Send ( ':' ) ;
  UART1SendConstString ( ChOffCommand ) ;
  UART1Send ( 0x0D ) ;
} /* Device01Off */


void Device02On ( void )
{
  /*-------------------------------------------------------------------------*/
  UART1SendConstString ( Device02ID ) ;
  UART1Send ( ':' ) ;
  UART1SendConstString ( MyDeviceID ) ;
  UART1Send ( ':' ) ;
  UART1SendConstString ( ChOnCommand ) ;
  UART1Send ( 0x0D ) ;
} /* Device01On */


void Device02Off ( void )
{
  /*-------------------------------------------------------------------------*/
  UART1SendConstString ( Device02ID ) ;
  UART1Send ( ':' ) ;
  UART1SendConstString ( MyDeviceID ) ;
  UART1Send ( ':' ) ;
  UART1SendConstString ( ChOffCommand ) ;
  UART1Send ( 0x0D ) ;
} /* Device01Off */


static void InitData ( UCHAR * OutData ,
                       UCHAR * InData )
{
  UCHAR Ch ;
  /*-------------------------------------------------------------------------*/
  for ( ; ; )
  {
    Ch = * InData ++ ;
    * OutData ++ = Ch ;
    if ( ! Ch )
    return ;
  } /* for */ ;
} /* InitData */


void InitDataService ( void )
{
  /*-------------------------------------------------------------------------*/
  InitData ( Analog1Value , ( UCHAR * ) "??,?" ) ;
  InitData ( Analog2Value , ( UCHAR * ) "??,?" ) ;
  InitData ( Analog3Value , ( UCHAR * ) "??,?" ) ;
  InitData ( Analog4Value , ( UCHAR * ) "??,?" ) ;
  InitData ( Analog5Value , ( UCHAR * ) "??,?" ) ;
  InitData ( Analog6Value , ( UCHAR * ) "??,?" ) ;
  On0Status = 0 ;
  On1Status = 0 ;
  On2Status = 0 ;
  On3Status = 0 ;
  CommandIndex = 0 ;
} /* InitDataService */

