/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2007                               **
**                                                                        **
**                          MODULE EMACDriver                             **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2007.03.22                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include "emacdriver.h"
#include <LPC23xx.H>
#include "types.h"

/*
**********************************************************************
**********************************************************************
**                                                                  **
**      Modul utworzony w oparciu o dokumentacje firmowa dla        **
**        mikrokontrolera LPC2378 pobranego ze strony www.nxp.com   **
**                                                                  **
**      kod dokumentu: UM10211                                      **
**      tytul dok.   : LPC2364/6/8/78 User manual                   **
**      wersja dok.  : Rev. 01  6 October 2006                     **
**                                                                  **
**********************************************************************
**********************************************************************
**                                                                  **
**      Modul utworzony w oparciu o dokumentacje firmowa dla        **
**        ukladu DP83848 pobranego ze strony www.national.com       **
**                                                                  **
**      tytul dok.   : DP83848C PHYTER - Commercial Temperature    **
**                     Single Port 10/100 Mb/s Ethernet Physical    **
**                     Layer Transceiver                            **
**      wersja dok.  : Febuary 2007                                 **
**                                                                  **
**********************************************************************
**********************************************************************

*/

#define OLD_EMAC_MODULE_ID  0x39022000  /* Rev. ID for first rev '-'         */

//#define RxFragmentsPcs       2
//#define TxFragmentsPcs       3
#define RxFragmentsPcs       4
#define TxFragmentsPcs       6

#define PCENET              0x40000000  /* PCENET Ethernet block power/clock control bit. */

#define EthernetRAMBase     0x7FE00000

                                        /* Command Register                               */
#define CR_RX_EN            0x00000001  /* Enable Receive: 1 << 0                         */
#define CR_TX_EN            0x00000002  /* Enable Transmit: 1 << 1                        */
#define CR_REG_RES          0x00000008  /* Reset Host Registers: 1 << 3                   */
#define CR_TX_RES           0x00000010  /* Reset Transmit Datapath: 1 << 4                */
#define CR_RX_RES           0x00000020  /* Reset Receive Datapath: 1 << 5                 */
#define CR_PASS_RUNT_FRM    0x00000040  /* Pass Runt Frames: 1 << 6                       */
#define CR_PASS_RX_FILT     0x00000080  /* Pass RX Filter: 1 << 7                         */
#define CR_TX_FLOW_CTRL     0x00000100  /* TX Flow Control: 1 << 8                        */
#define CR_RMII             0x00000200  /* Reduced MII Interface: 1 << 9                  */
#define CR_FULL_DUP         0x00000400  /* Full Duplex: 1 << 10                           */

                                        /* MAC Configuration Register 1                   */
#define MAC1_REC_EN         0x00000001  /* Receive Enable: 1 << 0                         */
#define MAC1_PASS_ALL       0x00000002  /* Pass All Receive Frames: 1 << 1                */
#define MAC1_RX_FLOWC       0x00000004  /* RX Flow Control: 1 << 2                        */
#define MAC1_TX_FLOWC       0x00000008  /* TX Flow Control: 1 << 3                        */
#define MAC1_LOOPB          0x00000010  /* Loop Back Mode: 1 << 4                         */
#define MAC1_RES_TX         0x00000100  /* Reset TX Logic: 1 << 8                         */
#define MAC1_RES_MCS_TX     0x00000200  /* Reset MAC TX Control Sublayer: 1 << 9          */
#define MAC1_RES_RX         0x00000400  /* Reset RX Logic: 1 << 10                        */
#define MAC1_RES_MCS_RX     0x00000800  /* Reset MAC RX Control Sublayer: 1 << 11         */
#define MAC1_SIM_RES        0x00004000  /* Simulation Reset: 1 << 14                      */
#define MAC1_SOFT_RES       0x00008000  /* Soft Reset MAC: 1 << 15                        */

                                        /* MAC Configuration Register 2                   */
#define MAC2_FULL_DUP       0x00000001  /* Full Duplex Mode: 1 << 0                       */
#define MAC2_FRM_LEN_CHK    0x00000002  /* Frame Length Checking: 1 << 1                  */
#define MAC2_HUGE_FRM_EN    0x00000004  /* Huge Frame Enable: 1 << 2                      */
#define MAC2_DLY_CRC        0x00000008  /* Delayed CRC Mode: 1 << 3                       */
#define MAC2_CRC_EN         0x00000010  /* Append CRC to every Frame: 1 << 4              */
#define MAC2_PAD_EN         0x00000020  /* Pad all Short Frames: 1 << 5                   */
#define MAC2_VLAN_PAD_EN    0x00000040  /* VLAN Pad Enable: 1 << 6                        */
#define MAC2_ADET_PAD_EN    0x00000080  /* Auto Detect Pad Enable: 1 << 7                 */
#define MAC2_PPREAM_ENF     0x00000100  /* Pure Preamble Enforcement: 1 << 8              */
#define MAC2_LPREAM_ENF     0x00000200  /* Long Preamble Enforcement: 1 << 9              */
#define MAC2_NO_BACKOFF     0x00001000  /* No Backoff Algorithm: 1 << 12                  */
#define MAC2_BACK_PRESSURE  0x00002000  /* Backoff Presurre / No Backoff: 1 << 13         */
#define MAC2_EXCESS_DEF     0x00004000  /* Excess Defer: 1 << 14                          */

                                        /* MII Management Indicators Register             */
#define MIND_BUSY           0x00000001  /* MII is Busy: 1 << 0                            */
#define MIND_SCAN           0x00000002  /* MII Scanning in Progress: 1 << 1               */
#define MIND_NOT_VAL        0x00000004  /* MII Read Data not valid: 1 << 2                */
#define MIND_MII_LINK_FAIL  0x00000008  /* MII Link Failed: 1 << 3                        */

                                        /* MII Management Command Register                */
#define MCMD_READ           0x00000001  /* MII Read: 1 << 0                               */
#define MCMD_SCAN           0x00000002  /* MII Scan continuously: 1 << 1                  */

#define CLRT_DEF            0x0000370F  /* Collision Window/Retry Register: default value */

#define IPGR_DEF            0x00000012  /* Non Back-to-Back Inter-Packet-Gap Register:    */
                                        /*                             Recommended value  */

#define SUPP_SPEED          0x00000100  /* Reduced MII Logic Current Speed                */
#define SUPP_RES_RMII       0x00000800  /* Reset Reduced MII Logic                        */

                                        /* Receive Filter Control Register                */
#define RFC_UCAST_EN        0x00000001  /* Accept Unicast Frames Enable: 1 << 0           */
#define RFC_BCAST_EN        0x00000002  /* Accept Broadcast Frames Enable: 1 << 1         */
#define RFC_MCAST_EN        0x00000004  /* Accept Multicast Frames Enable: 1 << 2         */
#define RFC_UCAST_HASH_EN   0x00000008  /* Accept Unicast Hash Filter Frames: 1 << 3      */
#define RFC_MCAST_HASH_EN   0x00000010  /* Accept Multicast Hash Filter Frame: 1 << 4     */
#define RFC_PERFECT_EN      0x00000020  /* Accept Perfect Match Enable: 1 << 5            */
#define RFC_MAGP_WOL_EN     0x00001000  /* Magic Packet Filter WoL Enable: 1 << 12        */
#define RFC_PFILT_WOL_EN    0x00002000  /* Perfect Filter WoL Enable: 1 << 13             */

                                        /* Interrupt Status/Enable/Clear/Set Registers    */
#define INT_RX_OVERRUN      0x00000001  /* Overrun Error in RX Queue: 1 << 0              */
#define INT_RX_ERR          0x00000002  /* Receive Error: 1 << 1                          */
#define INT_RX_FIN          0x00000004  /* RX Finished Process Descriptors: 1 << 2        */
#define INT_RX_DONE         0x00000008  /* Receive Done: 1 << 3                           */
#define INT_TX_UNDERRUN     0x00000010  /* Transmit Underrun: 1 << 4                      */
#define INT_TX_ERR          0x00000020  /* Transmit Error: 1 << 5                         */
#define INT_TX_FIN          0x00000040  /* TX Finished Process Descriptors: 1 << 6        */
#define INT_TX_DONE         0x00000080  /* Transmit Done: 1 << 7                          */
#define INT_SOFT_INT        0x00001000  /* Software Triggered Interrupt: 1 << 12          */
#define INT_WAKEUP          0x00002000  /* Wakeup Event Interrupt: 1 << 13                */

                                        /* RX Descriptor Control Word                     */
#define RCTRL_SIZE          0x000007FF  /* Buffer size mask                               */
#define RCTRL_INT           0x80000000  /* Generate RxDone Interrupt: 1 << 31             */

                                        /* TX Descriptor Control Word                     */
#define TCTRL_SIZE          0x000007FF  /* Size of data buffer in bytes                   */
#define TCTRL_OVERRIDE      0x04000000  /* Override Default MAC Registers: 1 << 26        */
#define TCTRL_HUGE          0x08000000  /* Enable Huge Frame: 1 << 27                     */
#define TCTRL_PAD           0x10000000  /* Pad short Frames to 64 bytes: 1 << 28          */
#define TCTRL_CRC           0x20000000  /* Append a hardware CRC to Frame: 1 << 29        */
#define TCTRL_LAST          0x40000000  /* Last Descriptor for TX Frame: 1 << 30          */
#define TCTRL_INT           0x80000000  /* Generate TxDone Interrupt: 1 << 31             */

                                        /* Back-to-Back Inter-Packet-Gap Register         */
#define IPGT_FULL_DUP       0x00000015  /* Recommended value for Full Duplex              */
#define IPGT_HALF_DUP       0x00000012  /* Recommended value for Half Duplex              */

                                        /* Test Register                                  */
#define TEST_SHCUT_PQUANTA  0x00000001  /* Shortcut Pause Quanta                          */
#define TEST_TST_PAUSE      0x00000002  /* Test Pause                                     */
#define TEST_TST_BACKP      0x00000004  /* Test Back Pressure                             */

                                        /* MII Management Configuration Register          */
#define MCFG_SCAN_INC       0x00000001  /* Scan Increment PHY Address: 1 << 0             */
#define MCFG_SUPP_PREAM     0x00000002  /* Suppress Preamble: 1 << 1                      */
#define MCFG_CLK_SEL        0x0000001C  /* Clock Select Mask                              */
#define MCFG_RES_MII        0x00008000  /* Reset MII Management Hardware                  */

#define MII_WR_TOUT         0x00050000  /* MII Write timeout count                        */
#define MII_RD_TOUT         0x00050000  /* MII Read timeout count                         */

                                        /* MII Management Address Register                */
#define MADR_REG_ADR        0x0000001F  /* MII Register Address Mask                      */
#define MADR_PHY_ADR        0x00001F00  /* PHY Address Mask                               */

                                        /* Status Register                                */
#define SR_RX_EN            0x00000001  /* Enable Receive                                 */
#define SR_TX_EN            0x00000002  /* Enable Transmit                                */

                                        /* Transmit Status Vector 0 Register              */
#define TSV0_CRC_ERR        0x00000001  /* CRC error                                      */
#define TSV0_LEN_CHKERR     0x00000002  /* Length Check Error                             */
#define TSV0_LEN_OUTRNG     0x00000004  /* Length Out of Range                            */
#define TSV0_DONE           0x00000008  /* Tramsmission Completed                         */
#define TSV0_MCAST          0x00000010  /* Multicast Destination                          */
#define TSV0_BCAST          0x00000020  /* Broadcast Destination                          */
#define TSV0_PKT_DEFER      0x00000040  /* Packet Deferred                                */
#define TSV0_EXC_DEFER      0x00000080  /* Excessive Packet Deferral                      */
#define TSV0_EXC_COLL       0x00000100  /* Excessive Collision                            */
#define TSV0_LATE_COLL      0x00000200  /* Late Collision Occured                         */
#define TSV0_GIANT          0x00000400  /* Giant Frame                                    */
#define TSV0_UNDERRUN       0x00000800  /* Buffer Underrun                                */
#define TSV0_BYTES          0x0FFFF000  /* Total Bytes Transferred                        */
#define TSV0_CTRL_FRAME     0x10000000  /* Control Frame                                  */
#define TSV0_PAUSE          0x20000000  /* Pause Frame                                    */
#define TSV0_BACK_PRESS     0x40000000  /* Backpressure Method Applied                    */
#define TSV0_VLAN           0x80000000  /* VLAN Frame                                     */

                                        /* Transmit Status Vector 1 Register              */
#define TSV1_BYTE_CNT       0x0000FFFF  /* Transmit Byte Count                            */
#define TSV1_COLL_CNT       0x000F0000  /* Transmit Collision Count                       */

                                        /* Receive Status Vector Register                 */
#define RSV_BYTE_CNT        0x0000FFFF  /* Receive Byte Count                             */
#define RSV_PKT_IGNORED     0x00010000  /* Packet Previously Ignored                      */
#define RSV_RXDV_SEEN       0x00020000  /* RXDV Event Previously Seen                     */
#define RSV_CARR_SEEN       0x00040000  /* Carrier Event Previously Seen                  */
#define RSV_REC_CODEV       0x00080000  /* Receive Code Violation                         */
#define RSV_CRC_ERR         0x00100000  /* CRC Error                                      */
#define RSV_LEN_CHKERR      0x00200000  /* Length Check Error                             */
#define RSV_LEN_OUTRNG      0x00400000  /* Length Out of Range                            */
#define RSV_REC_OK          0x00800000  /* Frame Received OK                              */
#define RSV_MCAST           0x01000000  /* Multicast Frame                                */
#define RSV_BCAST           0x02000000  /* Broadcast Frame                                */
#define RSV_DRIB_NIBB       0x04000000  /* Dribble Nibble                                 */
#define RSV_CTRL_FRAME      0x08000000  /* Control Frame                                  */
#define RSV_PAUSE           0x10000000  /* Pause Frame                                    */
#define RSV_UNSUPP_OPC      0x20000000  /* Unsupported Opcode                             */
#define RSV_VLAN            0x40000000  /* VLAN Frame                                     */

                                        /* Flow Control Counter Register                  */
#define FCC_MIRR_CNT        0x0000FFFF  /* Mirror Counter                                 */
#define FCC_PAUSE_TIM       0xFFFF0000  /* Pause Timer                                    */

                                        /* Flow Control Status Register                   */
#define FCS_MIRR_CNT        0x0000FFFF  /* Mirror Counter Current                         */

                                        /* Receive Filter WoL Status/Clear Registers      */
#define WOL_UCAST           0x00000001  /* Unicast Frame caused WoL                       */
#define WOL_BCAST           0x00000002  /* Broadcast Frame caused WoL                     */
#define WOL_MCAST           0x00000004  /* Multicast Frame caused WoL                     */
#define WOL_UCAST_HASH      0x00000008  /* Unicast Hash Filter Frame WoL                  */
#define WOL_MCAST_HASH      0x00000010  /* Multicast Hash Filter Frame WoL                */
#define WOL_PERFECT         0x00000020  /* Perfect Filter WoL                             */
#define WOL_RX_FILTER       0x00000080  /* RX Filter caused WoL                           */
#define WOL_MAG_PACKET      0x00000100  /* Magic Packet Filter caused WoL                 */

                                        /* Power Down Register                            */
#define PD_POWER_DOWN       0x80000000  /* Power Down MAC                                 */

                                        /* RX Status Hash CRC Word                        */
#define RHASH_SA            0x000001FF  /* Hash CRC for Source Address                    */
#define RHASH_DA            0x001FF000  /* Hash CRC for Destination Address               */

                                        /* RX Status Information Word                     */
#define RINFO_SIZE          0x000007FF  /* Data size in bytes                             */
#define RINFO_CTRL_FRAME    0x00040000  /* Control Frame                                  */
#define RINFO_VLAN          0x00080000  /* VLAN Frame                                     */
#define RINFO_FAIL_FILT     0x00100000  /* RX Filter Failed                               */
#define RINFO_MCAST         0x00200000  /* Multicast Frame                                */
#define RINFO_BCAST         0x00400000  /* Broadcast Frame                                */
#define RINFO_CRC_ERR       0x00800000  /* CRC Error in Frame                             */
#define RINFO_SYM_ERR       0x01000000  /* Symbol Error from PHY                          */
#define RINFO_LEN_ERR       0x02000000  /* Length Error                                   */
#define RINFO_RANGE_ERR     0x04000000  /* Range Error (exceeded max. size)               */
#define RINFO_ALIGN_ERR     0x08000000  /* Alignment Error                                */
#define RINFO_OVERRUN       0x10000000  /* Receive overrun                                */
#define RINFO_NO_DESCR      0x20000000  /* No new Descriptor available                    */
#define RINFO_LAST_FLAG     0x40000000  /* Last Fragment in Frame                         */
#define RINFO_ERR           0x80000000  /* Error Occured (OR of all errors)               */
//#define RINFO_ERR_MASK     ( RINFO_FAIL_FILT | RINFO_CRC_ERR   | RINFO_SYM_ERR | RINFO_LEN_ERR | RINFO_ALIGN_ERR | RINFO_OVERRUN )

                                        /* TX Status Information Word                     */
#define TINFO_COL_CNT       0x01E00000  /* Collision Count                                */
#define TINFO_DEFER         0x02000000  /* Packet Deferred (not an error)                 */
#define TINFO_EXCESS_DEF    0x04000000  /* Excessive Deferral                             */
#define TINFO_EXCESS_COL    0x08000000  /* Excessive Collision                            */
#define TINFO_LATE_COL      0x10000000  /* Late Collision Occured                         */
#define TINFO_UNDERRUN      0x20000000  /* Transmit Underrun                              */
#define TINFO_NO_DESCR      0x40000000  /* No new Descriptor available                    */
#define TINFO_ERR           0x80000000  /* Error Occured (OR of all errors)               */


#define nop() __asm ("nop")

/*
  ****************************************************************************
  **                        DP83848C PHY Registers                          **
  ****************************************************************************

  +-----+----+---------------------------------------------------------------------+
  | Adr |Mode|                     Description                                     |
  +-----+----+---------------------------------------------------------------------+
  | 00h | RW | BMCR Basic Mode Control Register                                    |
  +-----+----+---------------------------------------------------------------------+
  | 01h | RO | BMSR Basic Mode Status Register                                     |
  +-----+----+---------------------------------------------------------------------+
  | 02h | RO | PHYIDR1 PHY Identifier Register #1                                  |
  +-----+----+---------------------------------------------------------------------+
  | 03h | RO | PHYIDR2 PHY Identifier Register #2                                  |
  +-----+----+---------------------------------------------------------------------+
  | 04h | RW | ANAR Auto-Negotiation Advertisement Register                        |
  +-----+----+---------------------------------------------------------------------+
  | 05h | RW | ANLPAR Auto-Negotiation Link Partner Ability Register (Base Page)   |
  +-----+----+---------------------------------------------------------------------+
  | 05h | RW | ANLPARNP Auto-Negotiation Link Partner Ability Register (Next Page) |
  +-----+----+---------------------------------------------------------------------+
  | 06h | RW | ANER Auto-Negotiation Expansion Register                            |
  +-----+----+---------------------------------------------------------------------+
  | 07h | RW | ANNPTR Auto-Negotiation Next Page TX                                |
  +-----+----+---------------------------------------------------------------------+
  | 08h - 0Fh|   RESERVED RESERVED                                                 |
  +-----+----+---------------------------------------------------------------------+
  | 10h | RO | PHYSTS PHY Status Register                                          |
  +-----+----+---------------------------------------------------------------------+
  | 11h | RW | MICR MII Interrupt Control Register                                 |
  +-----+----+---------------------------------------------------------------------+
  | 12h | RO | MISR MII Interrupt Status Register                                  |
  +-----+----+---------------------------------------------------------------------+
  | 13h |    | RESERVED RESERVED                                                   |
  +-----+----+---------------------------------------------------------------------+
  | 14h | RO | FCSCR False Carrier Sense Counter Register                          |
  +-----+----+---------------------------------------------------------------------+
  | 15h | RO | RECR Receive Error Counter Register                                 |
  +-----+----+---------------------------------------------------------------------+
  | 16h | RW | PCSR PCS Sub-Layer Configuration and Status Register                |
  +-----+----+---------------------------------------------------------------------+
  | 17h | RW | RBR RMII and Bypass Register                                        |
  +-----+----+---------------------------------------------------------------------+
  | 18h | RW | LEDCR LED Direct Control Register                                   |
  +-----+----+---------------------------------------------------------------------+
  | 19h | RW | PHYCR PHY Control Register                                          |
  +-----+----+---------------------------------------------------------------------+
  | 1Ah | RW | 10BTSCR 10Base-T Status/Control Register                            |
  +-----+----+---------------------------------------------------------------------+
  | 1Bh | RW | CDCTRL1 CD Test Control Register and BIST Extensions Register       |
  +-----+----+---------------------------------------------------------------------+
  | 1Ch |    | RESERVED RESERVED                                                   |
  +-----+----+---------------------------------------------------------------------+
  | 1Dh | RW | EDCR Energy Detect Control Register                                 |
  +-----+----+---------------------------------------------------------------------+
  | 1Eh - 1Fh|  RESERVED RESERVED                                                  |
  +-----+----+---------------------------------------------------------------------+
*/


#define DP83848_BMCR        0x00        /* Basic Mode Control Register       */
#define DP83848_BMSR        0x01        /* Basic Mode Status Register        */
#define DP83848_IDR1        0x02        /* PHY Identifier 1                  */
#define DP83848_IDR2        0x03        /* PHY Identifier 2                  */
#define DP83848_ANAR        0x04        /* Auto-Negotiation Advertisement    */
#define DP83848_ANLPAR      0x05        /* Auto-Neg. Link Partner Abitily    */
#define DP83848_ANER        0x06        /* Auto-Neg. Expansion Register      */
#define DP83848_ANNPTR      0x07        /* Auto-Neg. Next Page TX            */
#define DP83848_STS         0x10        /* Status Register                   */
#define DP83848_MICR        0x11        /* MII Interrupt Control Register    */
#define DP83848_MISR        0x12        /* MII Interrupt Status Register     */
#define DP83848_FCSCR       0x14        /* False Carrier Sense Counter       */
#define DP83848_RECR        0x15        /* Receive Error Counter             */
#define DP83848_PCSR        0x16        /* PCS Sublayer Config. and Status   */
#define DP83848_RBR         0x17        /* RMII and Bypass Register          */
#define DP83848_LEDCR       0x18        /* LED Direct Control Register       */
#define DP83848_PHYCR       0x19        /* PHY Control Register              */
#define DP83848_10BTSCR     0x1A        /* 10Base-T Status/Control Register  */
#define DP83848_CDCTRL1     0x1B        /* CD Test Control and BIST Extens.  */
#define DP83848_EDCR        0x1D        /* Energy Detect Control Register    */

/*

Basic Mode Control Register (BMCR), address 0x00
  +-----+-------------+---------------------------------------------------------------+
  | Bit | Bit Name    |                     Description                               |
  +-----+-------------+---------------------------------------------------------------+
  |  15 | Reset       | Reset: 1 = Initiate software Reset / Reset in Process.        |
  |     |             | 0 = Normal operation.                                         |
  +-----+-------------+---------------------------------------------------------------+
  |  14 | Loopback    | Loopback: 1 = Loopback enabled.                               |
  |     |             | 0 = Normal operation.                                         |
  +-----+-------------+---------------------------------------------------------------+
  |  13 | Speed       | Speed Select: When auto-negotiation is disabled writing to    |
  |     | Selection   | this bit allows the port speed to be selected.                |
  |     |             | 1 = 100 Mb/s. 0 = 10 Mb/s.                                    |
  +-----+-------------+---------------------------------------------------------------+
  |  12 | Auto        | Auto-Negotiation Enable:                                      |
  |     | Negotiation | Strap controls initial value at reset.                        |
  |     | Enable      | 1 = Auto-Negotiation Enabled; 0 = Auto-Negotiation Disabled   |
  +-----+-------------+---------------------------------------------------------------+
  |  11 | Power Down  | Power Down: 1 = Power down. 0 = Normal operation.             |
  +-----+-------------+---------------------------------------------------------------+
  |  10 | Isolate     | Isolate: 1 = Isolates the Port from the MII with the exception|
  |     |             | of the serial management. 0 = Normal operation.               |
  +-----+-------------+---------------------------------------------------------------+
  |   9 | Restart     | Restart Auto-Negotiation:                                     |
  |     | Auto        | 1 = Restart Auto-Negotiation. Re-initiates the                |
  |     | Negotiation | Auto-Negotiation process. 0 = Normal operation.               |
  +-----+-------------+---------------------------------------------------------------+
  |   8 | Duplex Mode | Duplex Mode:                                                  |
  |     |             | 1 = Full Duplex operation.0 = Half Duplex operation.          |
  +-----+-------------+---------------------------------------------------------------+
  |   7 | Collision   | Collision Test:                                               |
  |     | Test        | 1 = Collision test enabled. 0 = Normal operation.             |
  +-----+-------------+---------------------------------------------------------------+


Basic Mode Status Register (BMSR), address 0x01
  +-----+-------------+---------------------------------------------------------------+
  | Bit | Bit Name    |                     Description                               |
  +-----+-------------+---------------------------------------------------------------+
  |  15 | 100BASE-T4  | 100BASE-T4 Capable: 0 = Device not able to perform 100BASE-T4 |
  |     |             | mode.                                                         |
  +-----+-------------+---------------------------------------------------------------+
  |  14 | 100BASE-TX  | 100BASE-TX Full Duplex Capable:                               |
  |     | Full Duplex | 1 = Device able to perform 100BASE-TX in full duplex mode.    |
  +-----+-------------+---------------------------------------------------------------+
  |  13 | 100BASE-TX  | 100BASE-TX Half Duplex Capable:                               |
  |     | Half Duplex | 1 = Device able to perform 100BASE-TX in half duplex mode.    |
  +-----+-------------+---------------------------------------------------------------+
  |  12 |  10BASE-T   | 10BASE-T Full Duplex Capable:                                 |
  |     | Full Duplex | 1 = Device able to perform 10BASE-T in full duplex mode.      |
  +-----+-------------+---------------------------------------------------------------+
  |  11 | 10BASE-T    | 10BASE-T Half Duplex Capable:                                 |
  |     | Half Duplex | 1 = Device able to perform 10BASE-T in half duplex mode.      |
  +-----+-------------+---------------------------------------------------------------+
  |   6 | MF Preamble | Preamble suppression Capable:                                 |
  |     | Suppression | 1 = Device able to perform management transaction with        |
  |     |             | preamble suppressed, 32-bits of preamble needed only once     |
  |     |             | after reset, invalid opcode or invalid turnaround.            |
  |     |             | 0 = Normal management operation.                              |
  +-----+-------------+---------------------------------------------------------------+
  |   5 | Auto        | Auto-Negotiation Complete:                                    |
  |     | Negotiation | 1 = Auto-Negotiation process complete.                        |
  |     | Complete    | 0 = Auto-Negotiation process not complete.                    |
  +-----+-------------+---------------------------------------------------------------+
  |   4 | Remote      | Remote Fault:                                                 |
  |     | Fault       | 1 = Remote Fault condition detected (cleared on read or by    |
  |     |             | reset). Fault criteria: Far End Fault Indication or           |
  |     |             | notification from Link Partner of Remote Fault.               |
  |     |             | 0 = No remote fault condition detected.                       |
  +-----+-------------+---------------------------------------------------------------+
  |   3 | Auto        | Auto Negotiation Ability:                                     |
  |     | Negotiation | 1 = Device is able to perform Auto-Negotiation.               |
  |     | Ability     | 0 = Device is not able to perform Auto-Negotiation.           |
  +-----+-------------+---------------------------------------------------------------+
  |   2 |Link Status  | Link Status:                                                  |
  |     |             | 1 = Valid link established                                    |
  |     |             | 0 = Link not established.                                     |
  +-----+-------------+---------------------------------------------------------------+
  |   1 |Jabber Detect| Jabber Detect:                                                |
  |     |             | 1 = Jabber condition detected. 0 = No Jabber.                 |
  +-----+-------------+---------------------------------------------------------------+
  |   0 | Extended    | Extended Capability:                                          |
  |     | Capability  | 1 = Extended register capabilities.                           |
  |     |             | 0 = Basic register set capabilities only.                     |
  +-----+-------------+---------------------------------------------------------------+


PHY Identifier Register #1 (PHYIDR1), address 0x02
  +-----+-------------+---------------------------------------------------------------+
  | Bit | Bit Name    |                     Description                               |
  +-----+-------------+---------------------------------------------------------------+
  | 15:0|  OUI_MSB    | Default value: 0010 0000 0000 0000     [2000 hex]             |
  +-----+-------------+---------------------------------------------------------------+


PHY Identifier Register #2 (PHYIDR2), address 0x03
  +-----+-------------+---------------------------------------------------------------+
  | Bit | Bit Name    |                     Description                               |
  +-----+-------------+---------------------------------------------------------------+
  |15:10|  OUI_LSB    | Default value: 0101 11                                        |
  +-----+-------------+---------------------------------------------------------------+
  | 9:4 | VNDR_MDL    | 00 1001                                                       |
  +-----+-------------+---------------------------------------------------------------+
  | 3:0 | MDL_REV     | 0000                                                          |
  +-----+-------------+---------------------------------------------------------------+
  |     |             | finalnie: 0101 1100 1001 0000   [5C90 hex]                    |
  +-----+-------------+---------------------------------------------------------------+


Auto-Negotiation Link Partner Ability Register (ANLPAR - base page), address 0x05
  +-----+-------------+---------------------------------------------------------------+
  | Bit | Bit Name    |                     Description                               |
  +-----+-------------+---------------------------------------------------------------+
  | 15  | NP          | Next Page Indication:                                         |
  |     |             | 0 = Link Partner does not desire Next Page Transfer.          |
  |     |             | 1 = Link Partner desires Next Page Transfer.                  |
  +-----+-------------+---------------------------------------------------------------+
  | 14  | ACK         | Acknowledge:                                                  |
  |     |             | 1 = Link Partner acknowledges reception of the ability data   |
  |     |             | word.                                                         |
  |     |             | 0 = Not acknowledged.                                         |
  +-----+-------------+---------------------------------------------------------------+
  | 13  | RF          | Remote Fault:                                                 |
  |     |             | 1 = Remote Fault indicated by Link Partner.                   |
  |     |             | 0 = No Remote Fault indicated by Link Partner.                |
  +-----+-------------+---------------------------------------------------------------+
  | 11  | ASM_DIR     | ASYMMETRIC PAUSE:                                             |
  |     |             | 1 = Asymmetric pause is supported by the Link Partner.        |
  |     |             | 0 = Asymmetric pause is not supported by the Link Partner.    |
  +-----+-------------+---------------------------------------------------------------+
  | 10  | PAUSE       | PAUSE:                                                        |
  |     |             | 1 = Pause function is supported by the Link Partner.          |
  |     |             | 0 = Pause function is not supported by the Link Partner.      |
  +-----+-------------+---------------------------------------------------------------+
  |  9  | T4          | 100BASE-T4 Support:                                           |
  |     |             | 1 = 100BASE-T4 is supported by the Link Partner.              |
  |     |             | 0 = 100BASE-T4 not supported by the Link Partner.             |
  +-----+-------------+---------------------------------------------------------------+
  |  8  | TX_FD       | 100BASE-TX Full Duplex Support:                               |
  |     |             | 1 = 100BASE-TX Full Duplex is supported by the Link Partner.  |
  |     |             | 0 = 100BASE-TX Full Duplex not supported by the Link Partner. |
  +-----+-------------+---------------------------------------------------------------+
  |  7  | TX          | 100BASE-TX Support:                                           |
  |     |             | 1 = 100BASE-TX is supported by the Link Partner.              |
  |     |             | 0 = 100BASE-TX not supported by the Link Partner.             |
  +-----+-------------+---------------------------------------------------------------+
  |  6  | 10_FD       | 10BASE-T Full Duplex Support:                                 |
  |     |             | 1 = 10BASE-T Full Duplex is supported by the Link Partner.    |
  |     |             | 0 = 10BASE-T Full Duplex not supported by the Link Partner.   |
  +-----+-------------+---------------------------------------------------------------+
  |  5  | 10          | 10BASE-T Support:                                             |
  |     |             | 1 = 10BASE-T is supported by the Link Partner.                |
  |     |             | 0 = 10BASE-T not supported by the Link Partner.               |
  +-----+-------------+---------------------------------------------------------------+
  | 4:0 | Selector    | Protocol Selection Bits:                                      |
  |     |             | Link Partners binary encoded protocol selector.              |
  +-----+-------------+---------------------------------------------------------------+


Auto-Negotiation Link Partner Ability Register (ANLPAR - next page), address 0x05
  +-----+-------------+---------------------------------------------------------------+
  | Bit | Bit Name    |                     Description                               |
  +-----+-------------+---------------------------------------------------------------+
  | 15  | NP          | Next Page Indication:                                         |
  |     |             | 1 = Link Partner desires Next Page Transfer.                  |
  |     |             | 0 = Link Partner does not desire Next Page Transfer.          |
  +-----+-------------+---------------------------------------------------------------+
  | 14  | ACK         | Acknowledge:                                                  |
  |     |             | 1 = Link Partner acknowledges reception of the ability data   |
  |     |             | word.                                                         |
  |     |             | 0 = Not acknowledged.                                         |
  +-----+-------------+---------------------------------------------------------------+
  | 13  | MP          | Message Page:                                                 |
  |     |             | 1 = Message Page.                                             |
  |     |             | 0 = Unformatted Page.                                         |
  +-----+-------------+---------------------------------------------------------------+
  | 12  | ACK2        | Acknowledge 2:                                                |
  |     |             | 1 = Link Partner does have the ability to comply to next page |
  |     |             | message.                                                      |
  |     |             | 0 = Link Partner does not have the ability to comply to next  |
  |     |             | page message.                                                 |
  +-----+-------------+---------------------------------------------------------------+
  | 11  | Toggle      | Toggle:                                                       |
  |     |             | 1 = Previous value of the transmitted Link Code word          |
  |     |             | equalled 0.                                                   |
  |     |             | 0 = Previous value of the transmitted Link Code word          |
  |     |             | equalled 1.                                                   |
  +-----+-------------+---------------------------------------------------------------+
  | 10:0| CODE        | Code:                                                         |
  |     |             | This field represents the code field of the next page         |
  |     |             | transmission.                                                 |
  +-----+-------------+---------------------------------------------------------------+


Auto-Negotiate Expansion Register (ANER), address 0x06
  +-----+-------------+---------------------------------------------------------------+
  | Bit | Bit Name    |                     Description                               |
  +-----+-------------+---------------------------------------------------------------+
  |  4  | PDF         | Parallel Detection Fault:                                     |
  |     |             | 1 = A fault has been detected via the Parallel Detection      |
  |     |             | function.                                                     |
  |     |             | 0 = A fault has not been detected.                            |
  +-----+-------------+---------------------------------------------------------------+
  |  3  | LP_NP_ABLE  | Link Partner Next Page Able:                                  |
  |     |             | 1 = Link Partner does support Next Page.                      |
  |     |             | 0 = Link Partner does not support Next Page.                  |
  +-----+-------------+---------------------------------------------------------------+
  |  2  |NP_ABLE      | Next Page Able:                                               |
  |     |             | 1 = Indicates local device is able to send additional         |
  |     |             | Next Pages.                                                 |
  +-----+-------------+---------------------------------------------------------------+
  |  1  | PAGE_RX     | Link Code Word Page Received:                                 |
  |     |             | 1 = Link Code Word has been received, cleared on a read.      |
  |     |             | 0 = Link Code Word has not been received.                     |
  +-----+-------------+---------------------------------------------------------------+


Auto-Negotiation Next Page Transmit Register (ANNPTR), address 0x07
  +-----+-------------+---------------------------------------------------------------+
  | Bit | Bit Name    |                     Description                               |
  +-----+-------------+---------------------------------------------------------------+
  | 15  | NP          | Next Page Indication:                                         |
  |     |             | 0 = No other Next Page Transfer desired.                      |
  |     |             | 1 = Another Next Page desired.                                |
  +-----+-------------+---------------------------------------------------------------+
  | 13  | MP          | Message Page:                                                 |
  |     |             | 1 = Message Page.                                             |
  |     |             | 0 = Unformatted Page.                                         |
  +-----+-------------+---------------------------------------------------------------+
  | 12  | ACK2        | Acknowledge2:                                                 |
  |     |             | 1 = Will comply with message.                                 |
  |     |             | 0 = Cannot comply with message.                               |
  +-----+-------------+---------------------------------------------------------------+
  | 11  | TOG_TX      | Toggle:                                                       |
  |     |             | 1 = Value of toggle bit in previously transmitted Link Code   |
  |     |             | Word was 0.                                                   |
  |     |             | 0 = Value of toggle bit in previously transmitted Link Code   |
  |     |             | Word was 1.                                                   |
  +-----+-------------+---------------------------------------------------------------+
  | 10:0| CODE        | This field represents the code field of the next page         |
  |     |             | transmission. If the MP bit is set (bit 13 of this register), |
  |     |             | then the code shall be interpreted as a "Message Page, as    |
  |     |             | defined in annex 28C of IEEE 802.3u. Otherwise, the code shall|
  |     |             | be interpreted as an "Unformatted Page, and the              |
  |     |             | interpretation is application specific. The default value of  |
  |     |             | the CODE represents a Null Page as defined in Annex 28C of    |
  |     |             | IEEE 802.3u.                                                  |
  +-----+-------------+---------------------------------------------------------------+


PHY Status Register (PHYSTS), address 0x10
  +-----+-------------+---------------------------------------------------------------+
  | Bit | Bit Name    |                     Description                               |
  +-----+-------------+---------------------------------------------------------------+
  |  14 | MDI-X mode  | MDI-X mode as reported by the Auto-Negotiation logic:         |
  |     |             | 1 = MDI pairs swapped (Receive on TPTD pair, Transmit on      |
  |     |             | TPRD pair) 0 = MDI pairs normal                               |
  +-----+-------------+---------------------------------------------------------------+
  |  13 |Receive Error| Receive Error Latch:                                          |
  |     | Latch       | 1 = Receive error event has occurred since last read of       |
  |     |             | RXERCNT, 0 = No receive error event has occurred.             |
  +-----+-------------+---------------------------------------------------------------+
  |  12 | Polarity    | Polarity Status:                                              |
  |     | Status      | 1 = Inverted Polarity detected.                               |
  |     |             | 0 = Correct Polarity detected.                                |
  +-----+-------------+---------------------------------------------------------------+
  |  11 |False Carrier| False Carrier Sense Latch:                                    |
  |     |Sense Latch  | 1 = False Carrier event has occurred since last read of FCSCR |
  |     |             | 0 = No False Carrier event has occurred.                      |
  +-----+-------------+---------------------------------------------------------------+
  |  10 |Signal Detect| 100Base-TX unconditional Signal Detect from PMD.              |
  +-----+-------------+---------------------------------------------------------------+
  |   9 | Descrambler | 100Base-TX Descrambler Lock from PMD.                         |
  |     | Lock        |                                                               |
  +-----+-------------+---------------------------------------------------------------+
  |   8 |Page Received| Link Code Word Page Received:                                 |
  |     |             | 1 = A new Link Code Word Page has been received. Cleared on   |
  |     |             | read of the ANER (address 0x06, bit 1).                       |
  |     |             | 0 = Link Code Word Page has not been received.                |
  +-----+-------------+---------------------------------------------------------------+
  |   7 |MII Interrupt| MII Interrupt Pending:                                        |
  |     |             | 1 = Indicates that an internal interrupt is pending.          |
  |     |             | Interrupt source can be determined by reading the MISR        |
  |     |             | Register (0x12h). Reading the MISR will clear the Interrupt.  |
  |     |             | 0= No interrupt pending.                                      |
  +-----+-------------+---------------------------------------------------------------+
  |   6 |Remote Fault | Remote Fault:                                                 |
  |     |             | 1 = Remote Fault condition detected (cleared on read of BMSR  |
  |     |             | register or by reset). Fault criteria: notification from Link |
  |     |             | Partner of Remote Fault via Auto-Negotiation.                 |
  |     |             | 0 = No remote fault condition detected.                       |
  +-----+-------------+---------------------------------------------------------------+
  |   5 |Jabber Detect| Jabber Detect: This bit only has meaning in 10 Mb/s mode      |
  |     |             | 1 = Jabber condition detected. 0 = No Jabber.                 |
  +-----+-------------+---------------------------------------------------------------+
  |   4 | Auto-Neg    | Auto-Negotiation Complete:                                    |
  |     | Complete    | 1 = Auto-Negotiation complete.                                |
  |     |             | 0 = Auto-Negotiation not complete.                            |
  +-----+-------------+---------------------------------------------------------------+
  |   3 | Loopback    | Loopback:                                                     |
  |     | Status      | 1 = Loopback enabled. 0 = Normal operation.                   |
  +-----+-------------+---------------------------------------------------------------+
  |   2 | Duplex      | Duplex:                                                       |
  |     | Status      | 1 = Full duplex mode. 0 = Half duplex mode.                   |
  +-----+-------------+---------------------------------------------------------------+
  |   1 |Speed Status | Speed10:                                                      |
  |     |             | 1 = 10 Mb/s mode. 0 = 100 Mb/s mode.                          |
  +-----+-------------+---------------------------------------------------------------+
  |   0 |Link Status  | Link Status:                                                  |
  |     |             | 1 = Valid link established  0 = Link not established.         |
  +-----+-------------+---------------------------------------------------------------+


MII Interrupt Control Register (MICR), address 0x11
  +-----+-------------+---------------------------------------------------------------+
  | Bit | Bit Name    |                     Description                               |
  +-----+-------------+---------------------------------------------------------------+
  |  2  | TINT        | Test Interrupt:                                               |
  |     |             | Forces the PHY to generate an interrupt to facilitate         |
  |     |             | interrupt testing. Interrupts will continue to be generated   |
  |     |             | as long as this bit remains set.                              |
  |     |             | 1 = Generate an interrupt                                     |
  |     |             | 0 = Do not generate interrupt                                 |
  +-----+-------------+---------------------------------------------------------------+
  |  1  | INTEN       | Interrupt Enable:                                             |
  |     |             | Enable interrupt dependent on the event enables in the MISR   |
  |     |             | register.                                                     |
  |     |             | 1 = Enable event based interrupts                             |
  |     |             | 0 = Disable event based interrupts                            |
  +-----+-------------+---------------------------------------------------------------+
  |  0  | INT_OE      | Interrupt Output Enable:                                      |
  |     |             | Enable interrupt events to signal via the PWR_DOWN/INT pin by |
  |     |             | configuring the PWR_DOWN/INT pin as an output.                |
  |     |             | 1 = PWR_DOWN/INT is an Interrupt Output                       |
  |     |             | 0 = PWR_DOWN/INT is a Power Down Input                        |
  +-----+-------------+---------------------------------------------------------------+


MII Interrupt Status and Misc. Control Register (MISR), address 0x12
  +-----+-------------+---------------------------------------------------------------+
  | Bit | Bit Name    |                     Description                               |
  +-----+-------------+---------------------------------------------------------------+
  | 14  | ED_INT      | Energy Detect interrupt:                                      |
  |     |             | 1 = Energy detect interrupt is pending and is cleared by the  |
  |     |             | current read.                                                 |
  |     |             | 0 = No energy detect interrupt pending.                       |
  +-----+-------------+---------------------------------------------------------------+
  | 13  | LINK_INT    | Change of Link Status interrupt:                              |
  |     |             | 1 = Change of link status interrupt is pending and is         |
  |     |             | cleared by the current read.                                  |
  |     |             | 0 = No change of link status interrupt pending.               |
  +-----+-------------+---------------------------------------------------------------+
  | 12  | SPD_INT     | Change of speed status interrupt:                             |
  |     |             | 1 = Speed status change interrupt is pending and is cleared by|
  |     |             | the current read.                                             |
  |     |             | 0 = No speed status change interrupt pending.                 |
  +-----+-------------+---------------------------------------------------------------+
  | 11  | DUP_INT     | Change of duplex status interrupt:                            |
  |     |             | 1 = Duplex status change interrupt is pending and is cleared  |
  |     |             | by the current read.                                          |
  |     |             | 0 = No duplex status change interrupt pending.                |
  +-----+-------------+---------------------------------------------------------------+
  | 10  | ANC_INT     | Auto-Negotiation Complete interrupt:                          |
  |     |             | 1 = Auto-negotiation complete interrupt is pending and is     |
  |     |             | cleared by the current read.                                  |
  |     |             | 0 = No Auto-negotiation complete interrupt pending.           |
  +-----+-------------+---------------------------------------------------------------+
  |  9  | FHF_INT     | False Carrier Counter half-full interrupt:                    |
  |     |             | 1 = False carrier counter half-full interrupt is pending and  |
  |     |             | is cleared by the current read.                               |
  |     |             | 0 = No false carrier counter half-full interrupt pending.     |
  +-----+-------------+---------------------------------------------------------------+
  |  8  | RHF_INT     | Receive Error Counter half-full interrupt:                    |
  |     |             | 1 = Receive error counter half-full interrupt is pending and  |
  |     |             | is cleared by the current read.                               |
  |     |             | 0 = No receive error carrier counter half-full interrupt      |
  |     |             | pending.                                                      |
  +-----+-------------+---------------------------------------------------------------+
  |  6  | ED_INT_EN   | Enable Interrupt on energy detect event                       |
  +-----+-------------+---------------------------------------------------------------+
  |  5  | LINK_INT_EN | Enable Interrupt on change of link status                     |
  +-----+-------------+---------------------------------------------------------------+
  |  4  | SPD_INT_EN  | Enable Interrupt on change of speed status                    |
  +-----+-------------+---------------------------------------------------------------+
  |  3  | DUP_INT_EN  | Enable Interrupt on change of duplex status                   |
  +-----+-------------+---------------------------------------------------------------+
  |  2  | ANC_INT_EN  | Enable Interrupt on Auto-negotiation complete event           |
  +-----+-------------+---------------------------------------------------------------+
  |  1  | FHF_INT_EN  | Enable Interrupt on False Carrier Counter Register half-full  |
  |     |             | event                                                         |
  +-----+-------------+---------------------------------------------------------------+
  |  0  | RHF_INT_EN  | Enable Interrupt on Receive Error Counter Register half-full  |
  |     |             | event                                                         |
  +-----+-------------+---------------------------------------------------------------+


*/

#define DP83848_BMCR_Reset     0x8000
#define DP83848_FULLD_100M     0x2100      /* Full Duplex 100Mbit                    */
#define DP83848_HALFD_100M     0x2000      /* Half Duplex 100Mbit                    */
#define DP83848_FULLD_10M      0x0100      /* Full Duplex 10Mbit                     */
#define DP83848_HALFD_10M      0x0000      /* Half Duplex 10MBit                     */
#define DP83848_AUTO_NEG       0x3000      /* Select Auto Negotiation                */
#define DP83848_Auto_Neg_Compl 0x0020      /* BSR: Auto-Negotiation process complete */
#define DP83848_Duplex_Status  0x0004      /* BSR: Full duplex mode                  */
#define DP83848_Speed_Stat_10M 0x0002      /* BSR: Speed 10M                         */

#define DP83848C_DEF_ADR       0x0100      /* Default PHY device address             */
#define DP83848C_ID1           0x2000      /* PHY Identifier 1                       */
#define DP83848C_ID2           0x5C90      /* PHY Identifier 2                       */

/*
  ****************************************************************************
  **                             LPC2378 Registers                          **
  ****************************************************************************

PINSEL2 - Pin function select register 2 (address 0xE002 C008)
   +--------+------------+------------+------------+------------+
   |  bits  |   = 0 0    |    = 0 1   |   = 1 0    |    = 1 1   |
   +--------+------------+------------+------------+------------+
   |  1:0   |  P1.0 GPIO |  ENET_TXD0 |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  3:2   |  P1.1 GPIO |  ENET_TXD1 |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  5:4   |  Reserved  |  Reserved  |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  7:6   |  Reserved  |  Reserved  |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  9:8   |  P1.4 GPIO | ENET_TX_EN |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  11:10 |  Reserved  |  Reserved  |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  13:12 |  Reserved  |  Reserved  |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  15:14 |  Reserved  |  Reserved  |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  17:16 |  P1.8 GPIO |  ENET_CRS  |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  19:18 |  P1.9 GPIO |  ENET_RXD0 |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  21:20 | P1.10 GPIO |  ENET_RXD1 |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  23:22 |  Reserved  |  Reserved  |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  25:24 |  Reserved  |  Reserved  |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  27:26 |  Reserved  |  Reserved  |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  29:28 | P1.14 GPIO | ENET_RX_ER |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  31:30 | P1.15 GPIO |ENET_REF_CLK|  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
ERRATA
    Setting up the Ethernet interface in RMII mode

    The LPC2378 has an Ethernet interface, which can be interfaced
    with an off-chip PHY using the RMII interface only.
    The MII interface is not supported on this device.

    The default configuration of the device does not enable the
    RMII interface.

    To use the Ethernet interface in RMII mode write a 1 to bit 12
    in PINSEL2 register (located at 0xE002 C008).

PINSEL3 - Pin function select register 3 (address 0xE002 C00C)
   +--------+------------+------------+------------+------------+
   |  bits  |   = 0 0    |    = 0 1   |   = 1 0    |    = 1 1   |
   +--------+------------+------------+------------+------------+
   |  1:0   | P1.16 GPIO | ENET_MDC   |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  3:2   | P1.17 GPIO | ENET_MDIO  |  Reserved  |  Reserved  |
   +--------+------------+------------+------------+------------+
   |  5:4   | P1.18 GPIO | U1UP_LED   |   PWM1.1   |   CAP1.0   |
   +--------+------------+------------+------------+------------+
   |  7:6   | P1.19 GPIO |  Reserved  |  Reserved  |   CAP1.1   |
   +--------+------------+------------+------------+------------+
   |  9:8   | P1.20 GPIO |  Reserved  |  PWM1.2    |   SCK0     |
   +--------+------------+------------+------------+------------+
   | 11:10  | P1.21 GPIO |  Reserved  |  PWM1.3    |   SSEL0    |
   +--------+------------+------------+------------+------------+
   | 13:12  | P1.22 GPIO |  Reserved  |  Reserved  |   MAT1.0   |
   +--------+------------+------------+------------+------------+
   | 15:14  | P1.23 GPIO |  Reserved  |  PWM1.4    |   MISO0    |
   +--------+------------+------------+------------+------------+
   | 17:16  | P1.24 GPIO |  Reserved  |  PWM1.5    |   MOSI0    |
   +--------+------------+------------+------------+------------+
   | 19:18  | P1.25 GPIO |  Reserved  |  Reserved  |   MAT1.1   |
   +--------+------------+------------+------------+------------+
   | 21:20  | P1.26 GPIO |  Reserved  |  PWM1.6    |   CAP0.0   |
   +--------+------------+------------+------------+------------+
   | 23:22  | P1.27 GPIO |  Reserved  |  Reserved  |   CAP0.1   |
   +--------+------------+------------+------------+------------+
   | 25:24  | P1.28 GPIO |  Reserved  |  PCAP1.0   |   MAT0.0   |
   +--------+------------+------------+------------+------------+
   | 27:26  | P1.29 GPIO |  Reserved  |  PCAP1.1   |   MAT0.1   |
   +--------+------------+------------+------------+------------+
   | 29:28  | P1.30 GPIO |  Reserved  |  VBUS      |   AD0.4    |
   +--------+------------+------------+------------+------------+
   | 31:30  | P1.31 GPIO |  Reserved  |  SCK1      |   AD0.5    |
   +--------+------------+------------+------------+------------+


PCONP - Power Control for Peripherals register (address 0xE01F C0C4)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |   0   | Unused, always 0                                   |     0    |
   +-------+----------------------------------------------------+----------+
   |   1   | PCTIM0 Timer/Counter 0 power/clock control bit.    |     1    |
   +-------+----------------------------------------------------+----------+
   |   2   | PCTIM1 Timer/Counter 1 power/clock control bit.    |     1    |
   +-------+----------------------------------------------------+----------+
   |   3   | PCUART0 UART0 power/clock control bit.             |     1    |
   +-------+----------------------------------------------------+----------+
   |   4   | PCUART1 UART1 power/clock control bit.             |     1    |
   +-------+----------------------------------------------------+----------+
   |   5   | PCPWM0 PWM0 power/clock control bit.               |     1    |
   +-------+----------------------------------------------------+----------+
   |   6   | PCPWM1 PWM1 power/clock control bit.               |     1    |
   +-------+----------------------------------------------------+----------+
   |   7   | PCI2C0 The I2C0 interface power/clock control bit. |     1    |
   +-------+----------------------------------------------------+----------+
   |   8   | PCSPI The SPI interface power/clock control bit.   |     1    |
   +-------+----------------------------------------------------+----------+
   |   9   | PCRTC The RTC power/clock control bit.             |     1    |
   +-------+----------------------------------------------------+----------+
   |  10   | PCSSP1 The SSP 1 interface power/clock control bit.|     1    |
   +-------+----------------------------------------------------+----------+
   |  11   | PCEMC External Memory Controller                   |     1    |
   +-------+----------------------------------------------------+----------+
   |  12   | PCAD A/D converter (ADC) power/clock control bit.  |     0    |
   +-------+----------------------------------------------------+----------+
   |  13   | PCAN1 CAN Controller 1 power/clock control bit.    |     0    |
   +-------+----------------------------------------------------+----------+
   |  14   | PCAN2 CAN Controller 2 power/clock control bit.    |     0    |
   +-------+----------------------------------------------------+----------+
   | 18:15 | Reserved                                           |    NA    |
   +-------+----------------------------------------------------+----------+
   |  19   | PCI2C1 The I2C1 interface power/clock control bit. |     1    |
   +-------+----------------------------------------------------+----------+
   |  20   | Unused, always                                     |     0    |
   +-------+----------------------------------------------------+----------+
   |  21   | PCSSP0 The SSP0 interface power/clock control bit. |     1    |
   +-------+----------------------------------------------------+----------+
   |  22   | PCTIM2 Timer 2 power/clock control bit.            |     0    |
   +-------+----------------------------------------------------+----------+
   |  23   | PCTIM3 Timer 3 power/clock control bit.            |     0    |
   +-------+----------------------------------------------------+----------+
   |  24   | PCUART2 UART 2 power/clock control bit.            |     0    |
   +-------+----------------------------------------------------+----------+
   |  25   | PCUART3 UART 3 power/clock control bit.            |     0    |
   +-------+----------------------------------------------------+----------+
   |  26   | PCI2C2 I2S interface 2 power/clock control bit.    |     1    |
   +-------+----------------------------------------------------+----------+
   |  27   | PCI2S I2S interface power/clock control bit.       |     0    |
   +-------+----------------------------------------------------+----------+
   |  28   | PCSDC SD card interface power/clock control bit.   |     0    |
   +-------+----------------------------------------------------+----------+
   |  29   | PCGPDMA GP DMA function power/clock control bit.   |     0    |
   +-------+----------------------------------------------------+----------+
   |  30   | PCENET Ethernet block power/clock control bit.     |     0    |
   +-------+----------------------------------------------------+----------+
   |  31   | PCUSB USB interface power/clock control bit.       |     0    |
   +-------+----------------------------------------------------+----------+



  ****************************************************************************
  **                          EMAC LPC2378 Registers                        **
  ****************************************************************************

MAC1 - MAC Configuration register 1 (address 0xFFE0 0000)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |    0  | RECEIVE ENABLE: Set this to allow receive frames to|     0    |
   |       | be received. Internally the MAC synchronizes       |          |
   |       | this control bit to the incoming receive stream.   |          |
   +-------+----------------------------------------------------+----------+
   |    1  | PASS ALL RECEIVE FRAMES: When enabled (set to 1),|     0    |
   |       | the MAC will pass all frames regardless of type    |          |
   |       | (normal vs. Control). When disabled, the MAC does  |          |
   |       | not pass valid Control frames.                     |          |
   +-------+----------------------------------------------------+----------+
   |    2  | RX FLOW CONTROL: When enabled (set to 1), the    |     0    |
   |       | MAC acts upon received PAUSE Flow Control frames.  |          |
   |       | When disabled, received PAUSE Flow Control frames  |          |
   |       | are ignored.                                       |          |
   +-------+----------------------------------------------------+----------+
   |    3  | TX FLOW CONTROL: When enabled (set to 1),        |     0    |
   |       | PAUSE Flow Control frames are allowed to be        |          |
   |       | transmitted. When disabled, Flow Control frames are|          |
   |       | blocked.                                           |          |
   +-------+----------------------------------------------------+----------+
   |    4  | LOOPBACK: Setting this bit will cause the MAC      |     0    |
   |       | Transmit interface to be looped back to the MAC    |          |
   |       | Receive interface. Clearing this bit results in    |          |
   |       | normal operation.                                  |          |
   +-------+----------------------------------------------------+----------+
   |    8  | RESET TX: Setting this bit will put the Transmit   |     0    |
   |       | Function logic in reset.                           |          |
   +-------+----------------------------------------------------+----------+
   |    9  | RESET MCS / TX: Setting this bit resets the MAC    |     0    |
   |       | Control Sublayer / Transmit logic. The MCS logic   |          |
   |       | implements flow control.                           |          |
   +-------+----------------------------------------------------+----------+
   |   10  | RESET RX: Setting this bit will put the Ethernet   |     0    |
   |       | receive logic in reset.                            |          |
   +-------+----------------------------------------------------+----------+
   |   11  | RESET MCS / RX: Setting this bit resets the MAC    |     0    |
   |       | Control Sublayer / Receive logic. The MCS logic    |          |
   |       | implements flow control.                           |          |
   +-------+----------------------------------------------------+----------+
   |   14  | SIMULATION RESET: Setting this bit will cause a    |     0    |
   |       | reset to the random number generator within the    |          |
   |       | Transmit Function.                                 |          |
   +-------+----------------------------------------------------+----------+
   |   15  | SOFT RESET: Setting this bit will put all modules  |     0    |
   |       | within the MAC in reset except the Host Interface. |          |
   +-------+----------------------------------------------------+----------+


MAC2 - MAC Configuration register 2 (address 0xFFE0 0004)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |    0  | FULL-DUPLEX: When enabled (set to 1), the MAC    |     0    |
   |       | operates in Full-Duplex mode. When disabled,       |          |
   |       | the MAC operates in Half-Duplex mode.              |          |
   +-------+----------------------------------------------------+----------+
   |    1  | FRAME LENGTH CHECKING: When enabled (set to 1),  |     0    |
   |       | both transmit and receive frame lengths are        |          |
   |       | compared to the Length/Type field. If the          |          |
   |       | Length/Type field represents a length then the     |          |
   |       | check is performed. Mismatches are reported in the |          |
   |       | StatusInfo word for each received frame.           |          |
   +-------+----------------------------------------------------+----------+
   |    2  | HUGE FRAME ENABLE: When enabled (set to 1),      |     0    |
   |       | frames of any length are transmitted and received. |          |
   +-------+----------------------------------------------------+----------+
   |    3  | DELAYED CRC: This bit determines the number of     |     0    |
   |       | bytes, if any, of proprietary header information   |          |
   |       | that exist on the front of IEEE 802.3 frames. When |          |
   |       | 1, four bytes of header (ignored by the CRC        |          |
   |       | function) are added. When 0, there is no           |          |
   |       | proprietary header.                                |          |
   +-------+----------------------------------------------------+----------+
   |    4  | CRC ENABLE: Set this bit to append a CRC to every  |     0    |
   |       | frame whether padding was required or not.         |          |
   |       | Must be set if PAD/CRC ENABLE is set. Clear this   |          |
   |       | bit if frames presented to the MAC contain a CRC.  |          |
   +-------+----------------------------------------------------+----------+
   |    5  | PAD / CRC ENABLE: Set this bit to have the MAC pad |     0    |
   |       | all short frames. Clear this bit if frames         |          |
   |       | presented to the MAC have a valid length. This bit |          |
   |       | is used in conjunction with AUTO PAD ENABLE and    |          |
   |       | VLAN PAD ENABLE. See Table 11137 - Pad Operation  |          |
   |       | for details on the pad function.                   |          |
   +-------+----------------------------------------------------+----------+
   |    6  | VLAN PAD ENABLE: Set this bit to cause the MAC to  |     0    |
   |       | pad all short frames to 64 bytes and append a valid|          |
   |       | CRC. Consult Table 11137 - Pad Operation for more |          |
   |       | information on the various padding features.       |          |
   |       | Note: This bit is ignored if PAD / CRC ENABLE is   |          |
   |       | cleared.                                           |          |
   +-------+----------------------------------------------------+----------+
   |    7  | AUTO DETECT PAD ENABLE: Set this bit to cause the  |     0    |
   |       | MAC to automatically detect the type of frame,     |          |
   |       | either tagged or un-tagged, by comparing the two   |          |
   |       | octets following the source address with 0x8100    |          |
   |       | (VLAN Protocol ID) and pad accordingly.            |          |
   +-------+----------------------------------------------------+----------+
   |    8  | PURE PREAMBLE ENFORCEMENT: When enabled (set to    |     0    |
   |       | 1), the MAC will verify the content of the       |          |
   |       | preamble to ensure it contains 0x55 and is         |          |
   |       | error-free. A packet with an incorrect preamble    |          |
   |       | is discarded. When disabled, no preamble checking  |          |
   |       | is performed.                                      |          |
   +-------+----------------------------------------------------+----------+
   |    9  | LONG PREAMBLE ENFORCEMENT: When enabled (set to    |     0    |
   |       | 1), the MAC only allows receive packets which    |          |
   |       | contain preamble fields less than 12 bytes in      |          |
   |       | length. When disabled, the MAC allows any length   |          |
   |       | preamble as per the Standard.                      |          |
   +-------+----------------------------------------------------+----------+
   |   12  | NO BACKOFF: When enabled (set to 1), the MAC will|     0    |
   |       | immediately retransmit following a collision rather|          |
   |       | than using the Binary Exponential Backoff algorithm|          |
   |       | as specified in the Standard.                      |          |
   +-------+----------------------------------------------------+----------+
   |   13  | BACK PRESSURE / NO BACKOFF: When enabled (set to   |     0    |
   |       | 1), after the MAC incidentally causes a          |          |
   |       | collision during back pressure, it will immediately|          |
   |       | retransmit without backoff, reducing the chance of |          |
   |       | further collisions and ensuring transmit packets   |          |
   |       | get sent.                                          |          |
   +-------+----------------------------------------------------+----------+
   |   14  | EXCESS DEFER: When enabled (set to 1) the MAC    |     0    |
   |       | will defer to carrier indefinitely as per the      |          |
   |       | Standard. When disabled, the MAC will abort when   |          |
   |       | the excessive deferral limit is reached.           |          |
   +-------+----------------------------------------------------+----------+


IPGT - Back-to-back Inter-packet-gap register (address 0xFFE0 0008)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  6:0  | BACK-TO-BACK INTER-PACKET-GAP                      |    0     |
   |       | This is a programmable field representing the      |          |
   |       | nibble time offset of the minimum possible period  |          |
   |       | between the end of any transmitted packet to the   |          |
   |       | beginning of the next. In Full-Duplex mode, the    |          |
   |       | register value should be the desired period in     |          |
   |       | nibble times minus 3. In Half-Duplex mode, the     |          |
   |       | register value should be the desired period in     |          |
   |       | nibble times minus 6. In Full-Duplex the           |          |
   |       | recommended setting is 0x15, which represents the  |          |
   |       | minimum IPG of 960 ns (in 100 Mbps mode) or 9.6 s |          |
   |       | (in 10 Mbps mode). In Half-Duplex the recommended  |          |
   |       | setting is 0x12, which also represents the minimum |          |
   |       | IPG of 960 ns (in 100 Mbps mode) or 9.6 s         |          |
   |       | (in 10 Mbps mode).                                 |          |
   +-------+----------------------------------------------------+----------+
   |  31:7 | Reserved.                                          |          |
   +-------+----------------------------------------------------+----------+

IPGR - Non Back-to-back Inter-packet-gap register (address 0xFFE0 000C)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  6:0  | NON-BACK-TO-BACK INTER-PACKET-GAP PART2            |     0    |
   |       | This is a programmable field representing the      |          |
   |       | Non-Back-to-Back Inter-Packet-Gap. The recommended |          |
   |       | value is 0x12 (18d), which represents the minimum  |          |
   |       | IPG of 960 ns (in 100 Mbps mode) or 9.6 s         |          |
   |       | (in 10 Mbps mode).                                 |          |
   +-------+----------------------------------------------------+----------+
   | 14:8  | NON-BACK-TO-BACK INTER-PACKET-GAP PART1            |     0    |
   |       | This is a programmable field representing the      |          |
   |       | optional carrierSense window referenced in         |          |
   |       | IEEE 802.3/4.2.3.2.1 'Carrier Deference'. If       |          |
   |       | carrier is detected during the timing of IPGR1,    |          |
   |       | the MAC defers to carrier. If, however, carrier    |          |
   |       | becomes active after IPGR1, the MAC continues      |          |
   |       | timing IPGR2 and transmits, knowingly causing a    |          |
   |       | collision, thus ensuring fair access to medium.    |          |
   |       | Its range of values is 0x0 to IPGR2. The           |          |
   |       | recommended value is 0xC (12d)                     |          |
   +-------+----------------------------------------------------+----------+


CLRT - Collision Window/Retry register (address 0xFFE0 0010)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  3..0 | RETRANSMISSION MAXIMUM                             |    0x0F  |
   |       | This is a programmable field specifying the number |          |
   |       | of retransmission attempts following a collision   |          |
   |       | before aborting the packet due to excessive        |          |
   |       | collisions. The Standard specifies the attemptLimit|          |
   |       | to be 0xF (15d). See IEEE 802.3/4.2.3.2.5.         |          |
   +-------+----------------------------------------------------+----------+
   | 13:8  | COLLISION WINDOW                                   |    0x37  |
   |       | This is a programmable field representing the slot |          |
   |       | time or collision window during which collisions   |          |
   |       | occur in properly configured networks. The default |          |
   |       | value of 0x37 (55d) represents a 56 byte window    |          |
   |       | following the preamble and SFD.                    |          |
   +-------+----------------------------------------------------+----------+


MAXF - Maximum Frame register (address 0xFFE0 0014)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   | 15..0 | MTU                                                |   0x0600 |
   |       |                                                    |  (1536)  |
   +-------+----------------------------------------------------+----------+


SUPP - PHY Support register (address 0xFFE0 0018)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |    8  | SPEED                                              |     0    |
   |       | This bit configures the Reduced MII logic for the  |          |
   |       | current operating speed. When set, 100 Mbps mode   |          |
   |       | is selected. When cleared, 10 Mbps mode is         |          |
   |       | selected.                                          |          |
   +-------+----------------------------------------------------+----------+
   |   11  | ?????                                              |     ?    |
   +-------+----------------------------------------------------+----------+


MCMD - MII Mgmt Command register (address 0xFFE0 0024)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |   0   | READ                                               |    0     |
   |       | This bit causes the MII Management hardware to     |          |
   |       | perform a single Read cycle. The Read data is      |          |
   |       | returned in Register MRDD (MII Mgmt Read Data).    |          |
   +-------+----------------------------------------------------+----------+
   |   1   | SCAN                                               |    0     |
   |       | This bit causes the MII Management hardware to     |          |
   |       | perform Read cycles continuously. This is          |          |
   |       | useful for monitoring Link Fail for example.       |          |
   +-------+----------------------------------------------------+----------+
   |  31:2 | Unused                                             |    0     |
   +-------+----------------------------------------------------+----------+


MADR - MII Mgmt Address register (address 0xFFE0 0028)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  4:0  | REGISTER ADDRESS                                   |     0    |
   |       | This field represents the 5 bit Register Address   |          |
   |       | field of Mgmt cycles. Up to 32 registers can be    |          |
   |       | accessed.                                          |          |
   +-------+----------------------------------------------------+----------+
   |  7:5  | Unused                                             |          |
   +-------+----------------------------------------------------+----------+
   |  12:8 | PHY ADDRESS                                        |     0    |
   |       | This field represents the 5 bit PHY Address field  |          |
   |       | of Mgmt cycles. Up to 31 PHYs can be addressed.    |          |
   +-------+----------------------------------------------------+----------+
   | 31:13 | Unused                                             |          |
   +-------+----------------------------------------------------+----------+


MWTD - MII Mgmt Write Data register (address 0xFFE0 002C)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   | 15:0  | WRITE DATA                                         |     0    |
   |       | When written, an MII Mgmt write cycle is performed |          |
   |       | using the 16 bit data and the pre-configured PHY   |          |
   |       | and Register addresses from the MII Mgmt Address   |          |
   |       | register (MADR).                                   |          |
   +-------+----------------------------------------------------+----------+
   | 31:16 | Unused                                             |          |
   +-------+----------------------------------------------------+----------+


MRDD - MII Mgmt Read Data register (address 0xFFE0 0030)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   | 15:0  | READ DATA                                          |     0    |
   |       | Following an MII Mgmt Read Cycle, the 16 bit data  |          |
   |       | can be read from this location.                    |          |
   +-------+----------------------------------------------------+----------+
   | 31:16 | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+


MIND - MII Mgmt Indicators register (address 0xFFE0 0034)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |   0   | BUSY                                               |     0    |
   |       | When 1 is returned - indicates MII Mgmt is       |          |
   |       | currently performing an MII Mgmt Read or Write     |          |
   |       | cycle.                                             |          |
   +-------+----------------------------------------------------+----------+
   |   1   | SCANNING                                           |     0    |
   |       | When 1 is returned - indicates a scan operation  |          |
   |       | (continuous MII Mgmt Read cycles) is in progress.  |          |
   +-------+----------------------------------------------------+----------+
   |   2   | NOT VALID                                          |     0    |
   |       | When 1 is returned - indicates MII Mgmt Read     |          |
   |       | cycle has not completed and the Read Data is not   |          |
   |       | yet valid.                                         |          |
   +-------+----------------------------------------------------+----------+
   |   3   | MII Link Fail                                      |     0    |
   |       | When 1 is returned - indicates that an MII Mgmt  |          |
   |       | link fail has occurred.                            |          |
   +-------+----------------------------------------------------+----------+
   | 31:4  | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+


SA0 - address Station Address register (0xFFE0 0040)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  7:0  | STATION ADDRESS (2nd octet)                        |     0    |
   |       | this field holds the second octet of the station   |          |
   |       | address.                                           |          |
   +-------+----------------------------------------------------+----------+
   |  15:8 | STATION ADDRESS (1st octet)                        |     0    |
   |       | This field holds the first octet of the station    |          |
   |       | address.                                           |          |
   +-------+----------------------------------------------------+----------+
   | 31:16 | Unused                                             |          |
   +-------+----------------------------------------------------+----------+


SA1 - address Station Address register (0xFFE0 0044)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  7:0  | STATION ADDRESS (4th octet)                        |     0    |
   |       | this field holds the second octet of the station   |          |
   |       | address.                                           |          |
   +-------+----------------------------------------------------+----------+
   |  15:8 | STATION ADDRESS (3rd octet)                        |     0    |
   |       | This field holds the first octet of the station    |          |
   |       | address.                                           |          |
   +-------+----------------------------------------------------+----------+
   | 31:16 | Unused                                             |          |
   +-------+----------------------------------------------------+----------+


SA2 - address Station Address register (0xFFE0 0048)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  7:0  | STATION ADDRESS (6th octet)                        |     0    |
   |       | this field holds the second octet of the station   |          |
   |       | address.                                           |          |
   +-------+----------------------------------------------------+----------+
   |  15:8 | STATION ADDRESS (5th octet)                        |     0    |
   |       | This field holds the first octet of the station    |          |
   |       | address.                                           |          |
   +-------+----------------------------------------------------+----------+
   | 31:16 | Unused                                             |          |
   +-------+----------------------------------------------------+----------+


MAC_COMMAND - Command register (address 0xFFE0 0100)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |    0  | RxEnable Enable receive.                           |     0    |
   +-------+----------------------------------------------------+----------+
   |    1  | TxEnable Enable transmit.                          |     0    |
   +-------+----------------------------------------------------+----------+
   |    3  | RegReset When a 1 is written, all datapaths and  |     0    |
   |       | the host registers are reset. The MAC needs to be  |          |
   |       | reset separately.                                  |          |
   +-------+----------------------------------------------------+----------+
   |    4  | TxReset When a 1 is written, the transmit        |     0    |
   |       | datapath is reset.                                 |          |
   +-------+----------------------------------------------------+----------+
   |    5  | RxReset When a 1 is written, the receive         |     0    |
   |       | datapath is reset.                                 |          |
   +-------+----------------------------------------------------+----------+
   |    6  | PassRuntFrame When set to 1, passes runt frames  |     0    |
   |       | smaller than 64 bytes to memory unless they have a |          |
   |       | CRC error. If 0 runt frames are filtered out.    |          |
   +-------+----------------------------------------------------+----------+
   |    7  | PassRxFilter When set to 1, disables receive     |     0    |
   |       | filtering i.e. all frames received are written to  |          |
   |       | memory.                                            |          |
   +-------+----------------------------------------------------+----------+
   |    8  | TxFlowControl Enable IEEE 802.3 / clause 31 flow   |     0    |
   |       | control sending pause frames in full duplex and    |          |
   |       | continuous preamble in half duplex.                |          |
   +-------+----------------------------------------------------+----------+
   |    9  | RMII When set to 1, RMII mode is selected;       |     0    |
   |       | if 0, MII mode is selected.                      |          |
   +-------+----------------------------------------------------+----------+
   |   10  | FullDuplex When set to 1, indicates full duplex  |     0    |
   |       | operation.                                         |          |
   +-------+----------------------------------------------------+----------+


Status - Status register (address 0xFFE0 0104)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |    0  | RxStatus                                           |     0    |
   |       | If 1, the receive channel is active. If 0, the     |          |
   |       | receive channel is inactive. 0                     |          |
   +-------+----------------------------------------------------+----------+
   |    1  | TxStatus                                           |     0    |
   |       | If 1, the transmit channel is active. If 0, the    |          |
   |       | transmit channel is inactive.                      |          |
   +-------+----------------------------------------------------+----------+
   |  31:2 | Unused                                             |          |
   +-------+----------------------------------------------------+----------+


RxDescriptor - Receive Descriptor Base Address register (address 0xFFE0 0108)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  1:0  | Fixed to 00                                      |     ?    |
   +-------+----------------------------------------------------+----------+
   | 31:2  | RxDescriptor MSBs of receive descriptor base       |     0    |
   |       | address.                                           |          |
   +-------+----------------------------------------------------+----------+


RxStatus - receive Status Base Address register (address 0xFFE0 010C)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  2:0  | Fixed to 000                                     |     ?    |
   +-------+----------------------------------------------------+----------+
   | 31:3  | RxStatus MSBs of receive status base address       |     0    |
   +-------+----------------------------------------------------+----------+


RxDescriptorNum - Receive Number of Descriptors register (address 0xFFE0 0110)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  15:0 | RxDescriptorNumber Number of descriptors in the    |     0    |
   |       | descriptor array for which RxDescriptor is the base|          |
   |       | address. The number of descriptors is minus one    |          |
   |       | encoded.                                           |          |
   +-------+----------------------------------------------------+----------+
   | 31:16 | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+


RXConsumeIndex - Receive Consume Index register (address 0xFFE0 0118)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   | 15:0  | RxConsumeIndex Index of the descriptor that is     |     0    |
   |       | going to be processed next by the receive          |          |
   +-------+----------------------------------------------------+----------+
   | 31:16 | Unused                                             |          |
   +-------+----------------------------------------------------+----------+


TxDescriptor - Transmit Descriptor Base Address register (address 0xFFE0 011C)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  1:0  | Fixed to 00                                      |     ?    |
   +-------+----------------------------------------------------+----------+
   | 31:2  | TxDescriptor MSBs of transmit descriptor base      |     0    |
   |       | address.                                           |          |
   +-------+----------------------------------------------------+----------+


TxStatus - Transmit Status Base Address register (address 0xFFE0 0120)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  1:0  | Fixed to 00                                      |     ?    |
   +-------+----------------------------------------------------+----------+
   | 31:2  | TxStatus MSBs of transmit status base address      |     0    |
   +-------+----------------------------------------------------+----------+


TxDescriptorNumber - Transmit Number of Descriptors register (address 0xFFE0 0124)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  15:0 | TxDescriptorNumber Number of descriptors in the    |     0    |
   |       | descriptor array for which TxDescriptor is the base|          |
   |       | address. The number of descriptors is minus one    |          |
   |       | encoded.                                           |          |
   +-------+----------------------------------------------------+----------+
   | 31:16 | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+


TxProduceIndex - Transmit Produce Index register (address 0xFFE0 0128)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   | 15:0  | TxProduceIndex Index of the descriptor that is     |     0    |
   |       | going to be processed next by the receive          |          |
   +-------+----------------------------------------------------+----------+
   | 31:16 | Unused                                             |          |
   +-------+----------------------------------------------------+----------+


TxConsumeIndex - Transmit Consume Index register (address 0xFFE0 012C)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  15:0 | TxConsumeIndex Index                               |          |
   |       | of the descriptor that is going to be transmitted  |
   |       | next by the transmit datapath.                     |          |
   +-------+----------------------------------------------------+----------+
   | 31:16 | Unused                                             |          |
   +-------+----------------------------------------------------+----------+


TSV0 - Transmit Status Vector 0 register (address 0xFFE0 0158)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |    0  | CRC error                                          |     0    |
   |       | The attached CRC in the packet did not match the   |          |
   |       | internally generated CRC.                          |          |
   +-------+----------------------------------------------------+----------+
   |    1  | Length check error                                 |     0    |
   |       | Indicates the frame length field does not match the|          |
   |       | actual number of data items and is not a type field|          |
   +-------+----------------------------------------------------+----------+
   |    2  | Length out of range                                |     0    |
   |       | Indicates that frame type/length field was larger  |          |
   |       | than 1500 bytes.                                   |          |
   +-------+----------------------------------------------------+----------+
   |    3  | Done                                               |     0    |
   |       | Transmission of packet was completed.              |          |
   +-------+----------------------------------------------------+----------+
   |    4  | Multicast                                          |     0    |
   |       | Packets destination was a multicast address.      |          |
   +-------+----------------------------------------------------+----------+
   |    5  | Broadcast                                          |     0    |
   |       | Packets destination was a broadcast address.      |          |
   +-------+----------------------------------------------------+----------+
   |    6  | Packet Defer                                       |     0    |
   |       | Packet was deferred for at least one attempt, but  |          |
   |       | less than an excessive defer.                      |          |
   +-------+----------------------------------------------------+----------+
   |    7  | Excessive Defer                                    |     0    |
   |       | Packet was deferred in excess of 6071 nibble times |          |
   |       | in 100 Mbps or 24287 bit times in 10 Mbps mode.    |          |
   +-------+----------------------------------------------------+----------+
   |    8  | Excessive Collision                                |     0    |
   |       | Packet was aborted due to exceeding of maximum     |          |
   |       | allowed number of collisions.                      |          |
   +-------+----------------------------------------------------+----------+
   |    9  | Late Collision                                     |     0    |
   |       | Collision occurred beyond collision window,        |          |
   |       | 512 bit times.                                     |          |
   +-------+----------------------------------------------------+----------+
   |   10  | Giant                                              |     0    |
   |       | Byte count in frame was greater than can be        |          |
   |       | represented in the transmit byte count field in    |          |
   |       | TSV1.                                              |          |
   +-------+----------------------------------------------------+----------+
   |   11  | Underrun                                           |     0    |
   |       | Host side caused buffer underrun.                  |          |
   +-------+----------------------------------------------------+----------+
   | 27:12 | Total bytes                                        |     0    |
   |       | The total number of bytes transferred including    |          |
   |       | collided attempts.                                 |          |
   +-------+----------------------------------------------------+----------+
   |   28  | Control frame                                      |     0    |
   |       | The frame was a control frame.                     |          |
   +-------+----------------------------------------------------+----------+
   |   29  | Pause                                              |     0    |
   |       | The frame was a control frame with a valid PAUSE   |          |
   |       | opcode.                                            |          |
   +-------+----------------------------------------------------+----------+
   |   30  | Backpressure                                       |     0    |
   |       | Carrier-sense method backpressure was previously   |          |
   |       | applied.                                           |          |
   +-------+----------------------------------------------------+----------+
   |   31  | VLAN                                               |     0    |
   |       | Frames length/type field contained 0x8100 which   |          |
   |       | is the VLAN protocol identifier.                   |          |
   +-------+----------------------------------------------------+----------+


TSV1 - Transmit Status Vector 1 register (address 0xFFE0 015C)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  15:0 | Transmit byte count                                |     0    |
   |       | The total number of bytes in the frame, not        |          |
   |       | counting the collided bytes.                       |          |
   +-------+----------------------------------------------------+----------+
   | 19:16 | Transmit collision count                           |     0    |
   |       | Number of collisions the current packet incurred   |          |
   |       | during transmission attempts. The maximum number   |          |
   |       | of collisions (16) cannot be represented.          |          |
   +-------+----------------------------------------------------+----------+
   | 31:20 | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+


RSV - Receive Status Vector register (address 0xFFE0 0160)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  15:0 | Received byte count                                |     0    |
   |       | Indicates length of received frame.                |          |
   +-------+----------------------------------------------------+----------+
   |   16  | Packet previously ignored                          |     0    |
   |       | Indicates that a packet was dropped.               |          |
   +-------+----------------------------------------------------+----------+
   |   17  | RXDV event previously seen                         |     0    |
   |       | Indicates that the last receive event seen was not |          |
   |       | long enough to be a valid packet.                  |          |
   +-------+----------------------------------------------------+----------+
   |   18  | Carrier event previously seen                      |     0    |
   |       | Indicates that at some time since the last receive |          |
   |       | statistics, a carrier event was detected.          |          |
   +-------+----------------------------------------------------+----------+
   |   19  | Receive code violation                             |     0    |
   |       | Indicates that MII data does not represent a valid |          |
   |       | receive code.                                      |          |
   +-------+----------------------------------------------------+----------+
   |   20  | CRC error                                          |     0    |
   |       | The attached CRC in the packet did not match the   |          |
   |       | internally generated CRC.                          |          |
   +-------+----------------------------------------------------+----------+
   |   21  | Length check error                                 |     0    |
   |       | Indicates the frame length field does not match the|          |
   |       | actual number of data items and is not a type field|          |
   +-------+----------------------------------------------------+----------+
   |   22  | Length out of range                                |     0    |
   |       | Indicates that frame type/length field was larger  |          |
   |       | than 1518 bytes.                                   |          |
   +-------+----------------------------------------------------+----------+
   |   23  | Receive OK                                         |     0    |
   |       | The packet had valid CRC and no symbol errors.     |          |
   +-------+----------------------------------------------------+----------+
   |   24  | Multicast                                          |     0    |
   |       | The packet destination was a multicast address.    |          |
   +-------+----------------------------------------------------+----------+
   |   25  | Broadcast                                          |     0    |
   |       | The packet destination was a broadcast address.    |          |
   +-------+----------------------------------------------------+----------+
   |   26  | Dribble Nibble                                     |     0    |
   |       | Indicates that after the end of packet another 1-7 |          |
   |       | bits were received. A single nibble, called dribble|          |
   |       | nibble, is formed but not sent out.                |          |
   +-------+----------------------------------------------------+----------+
   |   27  | Control frame                                      |     0    |
   |       | The frame was a control frame.                     |          |
   +-------+----------------------------------------------------+----------+
   |   28  | PAUSE                                              |     0    |
   |       | The frame was a control frame with a valid PAUSE   |          |
   |       | opcode.                                            |          |
   +-------+----------------------------------------------------+----------+
   |   29  | Unsupported Opcode                                 |     0    |
   |       | The current frame was recognized as a Control      |          |
   |       | Frame but contains an unknown opcode.              |          |
   +-------+----------------------------------------------------+----------+
   |   30  | VLAN                                               |     0    |
   |       | Frames length/type field contained 0x8100 which   |          |
   |       | is the VLAN protocol identifier.                   |          |
   +-------+----------------------------------------------------+----------+
   |   31  | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+


FlowControlCounter - Flow Control Counter register (address 0xFFE0 0170)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  15:0 | MirrorCounter                                      |     0    |
   |       | In full duplex mode the MirrorCounter specifies the|          |
   |       | number of cycles before re-issuing the Pause       |          |
   |       | control frame.                                     |          |
   +-------+----------------------------------------------------+----------+
   | 31:16 | PauseTimer                                         |     0    |
   |       | In full-duplex mode the PauseTimer specifies the   |          |
   |       | value that is inserted into the pause timer field  |          |
   |       | of a pause flow control frame. In half duplex mode |          |
   |       | the PauseTimer specifies the number of             |          |
   |       | backpressure cycles.                               |          |
   +-------+----------------------------------------------------+----------+


FlowControlStatus - Flow Control Status register (address 0xFFE0 8174)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  15:0 | MirrorCounterCurrent                               |     0    |
   |       | In full duplex mode this register represents the   |          |
   |       | current value of the datapaths mirror counter     |          |
   |       | which counts up to the value specified by the      |          |
   |       | MirrorCounter field in the FlowControlCounter      |          |
   |       | register. In half duplex mode the register counts  |          |
   |       | until it reaches the value of the PauseTimer bits  |          |
   |       | in the FlowControlCounter register.                |          |
   +-------+----------------------------------------------------+----------+
   | 31:16 | Unused                                             |          |
   +-------+----------------------------------------------------+----------+


RxFilterCtrl - Receive Filter Control register (address 0xFFE0 0200)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |   0   | AcceptUnicastEn                                    |     0    |
   |       | When set to 1, all unicast frames are accepted.  |          |
   +-------+----------------------------------------------------+----------+
   |   1   | AcceptBroadcastEn                                  |     0    |
   |       | When set to 1, all broadcast frames are accepted.|          |
   +-------+----------------------------------------------------+----------+
   |   2   | AcceptMulticastEn                                  |     0    |
   |       | When set to 1, all multicast frames are accepted.|          |
   +-------+----------------------------------------------------+----------+
   |   3   | AcceptUnicastHashEn                                |     0    |
   |       | When set to 1, unicast frames that pass the      |          |
   |       | imperfect hash filter are accepted.                |          |
   +-------+----------------------------------------------------+----------+
   |   4   | AcceptMulticastHashEn                              |     0    |
   |       | When set to 1, multicast frames that pass the    |          |
   |       | imperfect hash filter are accepted.                |          |
   +-------+----------------------------------------------------+----------+
   |   5   | AcceptPerfectEn                                    |     0    |
   |       | When set to 1, the frames with a destination     |          |
   |       | address identical to the station address are       |          |
   |       | accepted.                                          |          |
   +-------+----------------------------------------------------+----------+
   | 11:6  | Reserved                                           |          |
   +-------+----------------------------------------------------+----------+
   |  12   | MagicPacketEnWoL                                   |     0    |
   |       | When set to 1, the result of the magic packet    |          |
   |       | filter will generate a WoL interrupt when there    |          |
   |       | is a match.                                        |          |
   +-------+----------------------------------------------------+----------+
   |  13   | RxFilterEnWoL                                      |     0    |
   |       | When set to 1, the result of the perfect address |          |
   |       | matching filter and the imperfect hash filter will |          |
   |       | generate a WoL interrupt when there is a match.    |          |
   +-------+----------------------------------------------------+----------+
   | 31:14 | Unused                                             |          |
   +-------+----------------------------------------------------+----------+


RxFilterWoLStatus - Receive Filter WoL Status register (address 0xFFE0 0204)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |   0   | AcceptUnicastWoL                                   |     0    |
   |       | When the value is 1, a unicast frames caused WoL.|          |
   +-------+----------------------------------------------------+----------+
   |   1   | AcceptBroadcastWoL                                 |     0    |
   |       | When the value is 1, a broadcast frame caused WoL|          |
   +-------+----------------------------------------------------+----------+
   |   2   | AcceptMulticastWoL                                 |     0    |
   |       | When the value is 1, a multicast frame caused WoL|          |
   +-------+----------------------------------------------------+----------+
   |   3   | AcceptUnicastHashWoL                               |     0    |
   |       | When the value is 1, a unicast frame that passes |          |
   |       | the imperfect hash filter caused WoL.              |          |
   +-------+----------------------------------------------------+----------+
   |   4   | AcceptMulticastHashWoL                             |     0    |
   |       | When the value is 1, a multicast frame that      |          |
   |       | passes the imperfect hash filter caused WoL.       |          |
   +-------+----------------------------------------------------+----------+
   |   5   | AcceptPerfectWoL                                   |     0    |
   |       | When the value is 1, the perfect address         |          |
   |       | matching filter caused WoL.                        |          |
   +-------+----------------------------------------------------+----------+
   |   6   | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+
   |   7   | RxFilterWoL                                        |     0    |
   |       | When the value is 1, the receive filter caused   |          |
   |       | WoL                                                |          |
   +-------+----------------------------------------------------+----------+
   |   8   | MagicPacketWoL                                     |     0    |
   |       | When the value is 1, the magic packet filter     |          |
   |       | caused WoL.                                        |          |
   +-------+----------------------------------------------------+----------+
   |  31:9 | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+


RxFilterWoLClear - Receive Filter WoL Clear register (address 0xFFE0 0208)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |   0   | AcceptUnicastWoLClr                                |     0    |
   |       | When a 1 is written to one of these bits         |          |
   |       | (0 to 5), the corresponding status bit in the      |          |
   |       | RxFilterWoLStatus register is cleared.             |          |
   +-------+----------------------------------------------------+----------+
   |   1   | AcceptBroadcastWoLClr                              |     0    |
   +-------+----------------------------------------------------+----------+
   |   2   | AcceptMulticastWoLClr                              |     0    |
   +-------+----------------------------------------------------+----------+
   |   3   | AcceptUnicastHashWoLClr 0                          |     0    |
   +-------+----------------------------------------------------+----------+
   |   4   | AcceptMulticastHashWoLClr 0                        |     0    |
   +-------+----------------------------------------------------+----------+
   |   5   | AcceptPerfectWoLClr 0                              |     0    |
   +-------+----------------------------------------------------+----------+
   |   6   | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+
   |   7   | RxFilterWoLClr                                     |     0    |
   |       | When a 1 is written to one of these bits         |          |
   |       | (7 and/or 8), the corresponding status bit in the  |          |
   |       | RxFilterWoLStatus register is cleared.             |          |
   +-------+----------------------------------------------------+----------+
   |   8   | MagicPacketWoLClr                                  |     0    |
   +-------+----------------------------------------------------+----------+
   |  31:9 | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+


HashFilterL - Hash Filter Table LSBs register (address 0xFFE0 0210)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  31:0 | HashFilterL                                        |     0    |
   |       | Bit 31:0 of the imperfect filter hash table for    |          |
   |       | receive filtering.                                 |          |
   +-------+----------------------------------------------------+----------+


HashFilterH - Hash Filter MSBs register (address 0xFFE0 0214)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |  31:0 | HashFilterH                                        |     0    |
   |       | Bit 63:32 of the imperfect filter hash table for   |          |
   |       | receive filtering.                                 |          |
   +-------+----------------------------------------------------+----------+


IntStatus - Interrupt Status register (address 0xFFE0 0FE0)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |   0   | RxOverrunInt                                       |     0    |
   |       | Interrupt set on a fatal overrun error in the      |          |
   |       | receive queue. The fatal interrupt should be       |          |
   |       | resolved by a Rx soft-reset. The bit is not set    |          |
   |       | when there is a nonfatal overrun error.            |          |
   +-------+----------------------------------------------------+----------+
   |   1   | RxErrorInt                                         |     0    |
   |       | Interrupt trigger on receive errors:               |          |
   |       | AlignmentError, RangeError, LengthError,           |          |
   |       | SymbolError, CRCError or NoDescriptor or Overrun.  |          |
   +-------+----------------------------------------------------+----------+
   |   2   | RxFinishedInt                                      |     0    |
   |       | Interrupt triggered when all receive descriptors   |          |
   |       | have been processed i.e. on the transition to the  |          |
   |       | situation where ProduceIndex == ConsumeIndex.      |          |
   +-------+----------------------------------------------------+----------+
   |   3   | RxDoneInt                                          |     0    |
   |       | Interrupt triggered when a receive descriptor has  |          |
   |       | been processed while the Interrupt bit in the      |          |
   |       | Control field of the descriptor was set.           |          |
   +-------+----------------------------------------------------+----------+
   |   4   | TxUnderrunInt                                      |     0    |
   |       | Interrupt set on a fatal underrun error in the     |          |
   |       | transmit queue. The fatal interrupt should be      |          |
   |       | resolved by a Tx soft-reset. The bit is not set    |          |
   |       | when there is a nonfatal underrun error.           |          |
   +-------+----------------------------------------------------+----------+
   |   5   | TxErrorInt                                         |     0    |
   |       | Interrupt trigger on transmit errors:              |          |
   |       | LateCollision, ExcessiveCollision and              |          |
   |       | ExcessiveDefer, NoDescriptor or Underrun.          |          |
   +-------+----------------------------------------------------+----------+
   |   6   | TxFinishedInt                                      |     0    |
   |       | Interrupt triggered when all transmit descriptors  |          |
   |       | have been processed i.e. on the transition to the  |          |
   |       | situation where ProduceIndex == ConsumeIndex.      |          |
   +-------+----------------------------------------------------+----------+
   |   7   | TxDoneInt                                          |     0    |
   |       | Interrupt triggered when a descriptor has been     |          |
   |       | transmitted while the Interrupt bit in the Control |          |
   |       | field of the descriptor was set.                   |          |
   +-------+----------------------------------------------------+----------+
   |  11:8 | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+
   |   12  | SoftInt                                            |     0    |
   |       | Interrupt triggered by software writing a 1 to the |          |
   |       | SoftintSet bit in the IntSet register.             |          |
   +-------+----------------------------------------------------+----------+
   |  13   | WakeupInt                                          |     0    |
   |       | Interrupt triggered by a Wakeup event detected by  |          |
   |       | the receive filter.                                |          |
   +-------+----------------------------------------------------+----------+
   | 31:14 | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+


IntEnable - Interrupt Enable register (address 0xFFE0 0FE4)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |   0   | RxOverrunIntEn Enable                              |     0    |
   |       | for interrupt trigger on receive buffer overrun or |          |
   |       | descriptor underrun situations.                    |          |
   +-------+----------------------------------------------------+----------+
   |   1   | RxErrorIntEn Enable                                |     0    |
   |       | for interrupt trigger on receive errors.           |          |
   +-------+----------------------------------------------------+----------+
   |   2   | RxFinishedIntEn Enable                             |     0    |
   |       | for interrupt triggered when all receive           |          |
   |       | descriptors have been processed i.e. on the        |          |
   |       | transition to the situation where                  |          |
   |       | ProduceIndex == ConsumeIndex.                      |          |
   +-------+----------------------------------------------------+----------+
   |   3   | RxDoneIntEn Enable                                 |     0    |
   |       | for interrupt triggered when a receive descriptor  |          |
   |       | has been processed while the Interrupt bit in the  |          |
   |       | Control field of the descriptor was set.           |          |
   +-------+----------------------------------------------------+----------+
   |   4   | TxUnderrunIntEn Enable                             |     0    |
   |       | for interrupt trigger on transmit buffer or        |          |
   |       | descriptor underrun situations.                    |          |
   +-------+----------------------------------------------------+----------+
   |   5   | TxErrorIntEn Enable                                |     0    |
   |       | for interrupt trigger on transmit errors.          |          |
   +-------+----------------------------------------------------+----------+
   |   6   | TxFinishedIntEn Enable                             |     0    |
   |       | for interrupt triggered when all transmit          |          |
   |       | descriptors have been processed i.e. on the        |          |
   |       | transition to the situation where                  |          |
   |       | ProduceIndex == ConsumeIndex.                      |          |
   +-------+----------------------------------------------------+----------+
   |   7   | TxDoneIntEn Enable                                 |     0    |
   |       | for interrupt triggered when a descriptor has been |          |
   |       | transmitted while the Interrupt bit in the Control |          |
   |       | field of the descriptor was set.                   |          |
   +-------+----------------------------------------------------+----------+
   |  11:8 | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+
   |  12   | SoftIntEn Enable                                   |     0    |
   |       | for interrupt triggered by the SoftInt bit in the  |          |
   |       | IntStatus register, caused by software writing a 1 |          |
   |       | to the SoftIntSet bit in the IntSet register.      |          |
   +-------+----------------------------------------------------+----------+
   |  13   | WakeupIntEn Enable                                 |     0    |
   |       | for interrupt triggered by a Wakeup event detected |          |
   |       | by the receive filter.                             |          |
   +-------+----------------------------------------------------+----------+
   | 31:14 | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+


IntClear - Interrupt Clear register (address 0xFFE0 0FE8)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |   0   | RxOverrunIntClr                                    |     0    |
   |       | Writing a 1 to one of these bits clears (0 to 7) |          |
   |       | the corresponding status bit in interrupt status   |          |
   |       | register IntStatus.                                |          |
   +-------+----------------------------------------------------+----------+
   |   1   | RxErrorIntClr                                      |     0    |
   +-------+----------------------------------------------------+----------+
   |   2   | RxFinishedIntClr                                   |     0    |
   +-------+----------------------------------------------------+----------+
   |   3   | RxDoneIntClr                                       |     0    |
   +-------+----------------------------------------------------+----------+
   |   4   | TxUnderrunIntClr                                   |     0    |
   +-------+----------------------------------------------------+----------+
   |   5   | TxErrorIntClr                                      |     0    |
   +-------+----------------------------------------------------+----------+
   |   6   | TxFinishedIntClr                                   |     0    |
   +-------+----------------------------------------------------+----------+
   |   7   | TxDoneIntClr                                       |     0    |
   +-------+----------------------------------------------------+----------+
   |  11:8 | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+
   |  12   | SoftIntClr                                         |     0    |
   |       | Writing a 1 to one of these bits (12 and/or 13)  |          |
   |       | clears the corresponding status bit in interrupt   |          |
   |       | status register IntStatus.                         |          |
   +-------+----------------------------------------------------+----------+
   |  13   | WakeupIntClr                                       |     0    |
   +-------+----------------------------------------------------+----------+
   | 31:14 | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+


IntSet - Interrupt Set register (address 0xFFE0 0FEC)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   |   0   | RxOverrunIntSet                                    |     0    |
   |       | Writing a 1 to one of these bits (0 to 7) sets   |          |
   |       | the corresponding status bit in interrupt status   |          |
   |       | register IntStatus.                                |          |
   +-------+----------------------------------------------------+----------+
   |   1   | RxErrorIntSet                                      |     0    |
   +-------+----------------------------------------------------+----------+
   |   2   | RxFinishedIntSet                                   |     0    |
   +-------+----------------------------------------------------+----------+
   |   3   | RxDoneIntSet                                       |     0    |
   +-------+----------------------------------------------------+----------+
   |   4   | TxUnderrunIntSet                                   |     0    |
   +-------+----------------------------------------------------+----------+
   |   5   | TxErrorIntSet                                      |     0    |
   +-------+----------------------------------------------------+----------+
   |   6   | TxFinishedIntSet                                   |     0    |
   +-------+----------------------------------------------------+----------+
   |   7   | TxDoneIntSet                                       |     0    |
   +-------+----------------------------------------------------+----------+
   |  11:8 | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+
   |  12   | SoftIntSet                                         |     0    |
   |       | Writing a 1 to one of these bits (12 and/or 13)  |          |
   |       | sets the corresponding status bit in interrupt     |          |
   |       | status register IntStatus.                         |          |
   +-------+----------------------------------------------------+----------+
   |  13   | WakeupIntSet                                       |     0    |
   +-------+----------------------------------------------------+----------+
   | 31:14 | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+


PowerDown - Power Down register (address 0xFFE0 0FF4)
   +-------+----------------------------------------------------+----------+
   |  bit  |                                                    |Reset val.|
   +-------+----------------------------------------------------+----------+
   | 30:0  | Unused                                             |     0    |
   +-------+----------------------------------------------------+----------+
   |   31  | PowerDownMACAHB                                    |     0    |
   |       | If true, all AHB accesses will return a read/write |          |
   |       | error, except accesses to the PowerDown register.  |          |
   +-------+----------------------------------------------------+----------+


ERRATA
Ethernet.2: Ethernet SRAM disabled
Introduction: The LPC2378 has an Ethernet interface, which has a dedicated 16kB SRAM.
Problem: When the Ethernet block is disabled (in the PCONP register located at 0xE01F C0C4), the Ethernet
SRAM is also disabled.
Workaround: Enable the Ethernet block by setting the PCENET bit (bit no 30) in the PCONP register. The
Ethernet SRAM is now enabled.

*/


typedef struct {              /* Struktura deskryptowa odbiornika: musi odtwarzac rysunek 11-20 z dokumentacji: UM10211 */
                 ULONG   Packet ;
                 ULONG   Control ;
               } RxDescriptor_RecT ;
typedef struct {              /* Struktura statusow odbiornika: musi odtwarzac rysunek 11-20 z dokumentacji: UM10211 */
                 ULONG   StatusInfo ;
                 ULONG   StatusHashCRC ;
               } RxStatus_RecT ;
typedef struct {              /* Struktura deskryptowa nadajnika: musi odtwarzac rysunek 11-21 z dokumentacji: UM10211 */
                 ULONG   Packet ;
                 ULONG   Control ;
               } TxDescriptor_RecT ;
typedef struct {              /* Struktura statusow nadajnika: musi odtwarzac rysunek 11-21 z dokumentacji: UM10211 */
                 ULONG   StatusInfo ;
               } TxStatus_RecT ;

#define RxDescrBaseAdr   EthernetRAMBase
#define RxStatusBaseAdr  ( RxDescrBaseAdr + RxFragmentsPcs * sizeof ( RxDescriptor_RecT ) )
#define TxDescrBaseAdr   ( RxStatusBaseAdr + RxFragmentsPcs * sizeof ( RxStatus_RecT ) )
#define TxStatusBaseAdr  ( TxDescrBaseAdr + TxFragmentsPcs * sizeof ( TxDescriptor_RecT ) )
#define RxBufferBaseAdr  ( TxStatusBaseAdr + TxFragmentsPcs * sizeof ( TxStatus_RecT ) )
#define TxBufferBaseAdr  ( RxBufferBaseAdr + RxFragmentsPcs * EthMaxFrameSzie )


EMACInstanceRecT EMACInstanceRec ;


static void PoolEthInterface ( void )
{
  ULONG RxDataLength ;
  ULONG Index ;
  ULONG FrameStatus ;
  ULONG * SourcePointer ;
  ULONG * DestinationPointer ;
  PtrOSFrameRecT Frame ;
  RxDescriptor_RecT * WorkRxDescr ;
  RxStatus_RecT * WorkRxStatus ;
  /*-------------------------------------------------------------------------*/
  if ( MAC_RXPRODUCEINDEX != MAC_RXCONSUMEINDEX )
  {
    Index = MAC_RXCONSUMEINDEX ;
    WorkRxDescr = ( RxDescriptor_RecT * ) RxDescrBaseAdr ;
    WorkRxDescr += Index ;
    WorkRxStatus = ( RxStatus_RecT * ) RxStatusBaseAdr ;
    WorkRxStatus += Index ;
    FrameStatus = WorkRxStatus -> StatusInfo ;
    RxDataLength = ( FrameStatus & RINFO_SIZE ) - 3 ;
    if ( RxDataLength > EtherMTU )
    {
      Frame = NIL ;
    } /* if ... */
    else
    {
      Frame = EMACInstanceRec . InterfaceAllocFrameMemory ( RxDataLength ) ;
      if ( Frame )
      {
        DestinationPointer = ( ULONG * ) ( & Frame -> Data [ 0 ] ) ;
        Frame -> Length = RxDataLength ;
        Frame -> Index = 0 ;
        SourcePointer = ( ULONG * ) WorkRxDescr -> Packet ;
        RxDataLength = ( RxDataLength + 3 ) >> 2 ;
        for ( ; RxDataLength ; RxDataLength -- )
          * DestinationPointer ++ = * SourcePointer ++ ;
      } /*if */ ;
    } /* if ... else */ ;
    Index = MAC_RXCONSUMEINDEX ;
    if ( ++ Index == RxFragmentsPcs )
      Index = 0 ;
    MAC_RXCONSUMEINDEX = Index ;
		if ( Frame )
      EMACInstanceRec . InterfaceProcessFrame ( Frame ) ;
  } /* if */ ;
} /* PoolEthInterface */


static void IgnoreFrame ( PtrOSFrameRecT Frame )
{
  /*-------------------------------------------------------------------------*/
  EMACInstanceRec . InterfaceDeallocFrameMemory ( Frame ) ;
} /* IgnoreFrame */


static PtrOSFrameRecT DefaultAllocation ( ULONG Size )
{
  /*-------------------------------------------------------------------------*/
  return ( ( OSFrameRecT * ) 0 ) ;
} /* DefaultAllocation */


static void DefaultDeallocation ( PtrOSFrameRecT Frame )
{
  /*-------------------------------------------------------------------------*/
} /* DefaultDeallocation */


static ULONG EMACSendFrame ( PtrOSFrameRecT Frame )
{
  ULONG Index ;
  ULONG Length ;
  ULONG * SourcePointer ;
  ULONG * DestinationPointer ;
  TxDescriptor_RecT * TxDescr ;
  /*-------------------------------------------------------------------------*/
  while ( Frame -> Length < EthMinFrameSize )
  {
    Frame -> Data [ Frame -> Length ++ ] = 0x00 ;
  } /* while */ ;
  Index = MAC_TXPRODUCEINDEX ;
  SourcePointer  = ( ULONG * ) ( & Frame -> Data [ 0 ] ) ;
  TxDescr = ( TxDescriptor_RecT * ) TxDescrBaseAdr ;
  TxDescr += Index ;
  DestinationPointer  = ( ULONG * ) TxDescr -> Packet ;
  Length = ( Frame -> Length + 3 ) >> 2 ;
  for ( ; Length ; Length -- )
    * DestinationPointer ++ = * SourcePointer ++ ;
  TxDescr -> Control = ( Frame -> Length - 1 ) | TCTRL_LAST ;
                                           /* Ustawienie dlugosci pakietu +            */
                                           /* Last Descriptor for TX Frame +           */
  Index = MAC_TXPRODUCEINDEX ;
  if ( ++ Index == TxFragmentsPcs )
    Index = 0 ;
  MAC_TXPRODUCEINDEX = Index ;
  return ( SendFrame_OK ) ;
} /* EMACSendFrame */


static void InitEMACDescr ( void )
{
  ULONG Loop ;
  ULONG BufferAddress ;
  RxDescriptor_RecT * WorkRxDescr ;
  RxStatus_RecT * WorkRxStatus ;
  TxDescriptor_RecT * WorkTxDescr ;
  TxStatus_RecT * WorkTxStatus ;
  /*-------------------------------------------------------------------------*/
  WorkRxDescr = ( RxDescriptor_RecT * ) RxDescrBaseAdr ;
  WorkRxStatus = ( RxStatus_RecT * ) RxStatusBaseAdr ;
  WorkTxDescr = ( TxDescriptor_RecT * ) TxDescrBaseAdr ;
  WorkTxStatus = ( TxStatus_RecT * ) TxStatusBaseAdr ;
  BufferAddress = RxBufferBaseAdr ;
  for ( Loop = 0 ; Loop < RxFragmentsPcs ; Loop ++ )
  {                             /* Odtworzenie powiazan: musi odtwarzac rysunek 11-20 z dokumentacji: UM10211 */
    WorkRxDescr -> Packet = BufferAddress ;
    WorkRxDescr -> Control = ( ULONG ) RCTRL_INT | ( EthMaxFrameSzie - 1 ) ;
    WorkRxStatus -> StatusInfo = 0 ;
    WorkRxStatus -> StatusHashCRC = 0 ;
    BufferAddress += EthMaxFrameSzie ;
    WorkRxDescr ++ ;
    WorkRxStatus ++ ;
  } /* for */ ;
  BufferAddress = TxBufferBaseAdr ;
  for ( Loop = 0 ; Loop < TxFragmentsPcs ; Loop ++ )
  {                             /* Odtworzenie powiazan: musi odtwarzac rysunek 11-20 z dokumentacji: UM10211 */
    WorkTxDescr -> Packet = BufferAddress ;
    WorkTxDescr -> Control = 0 ;
    WorkTxStatus -> StatusInfo = 0 ;
    BufferAddress += EthMaxFrameSzie ;
    WorkTxDescr ++ ;
    WorkTxStatus ++ ;
  } /* for */ ;
  MAC_RXDESCRIPTOR = RxDescrBaseAdr ;           /* Ustawienie rejestru deksryptorw: EMAC Receive Descriptor Registers. */
  MAC_RXSTATUS = RxStatusBaseAdr ;              /* Ustawienie rejestru statusw */
  MAC_RXDESCRIPTORNUM = RxFragmentsPcs - 1 ;    /* Liczba dostepnych dla DMA deskryptorw */
  MAC_RXCONSUMEINDEX = 0 ;                      /* Wskazanie na pierwszy wolny deskryptor */
  MAC_TXDESCRIPTOR = TxDescrBaseAdr ;           /* Ustawienie rejestru deksryptorw: EMAC Transmit Descriptor Registers. */
  MAC_TXSTATUS = TxStatusBaseAdr ;              /* Ustawienie rejestru statusw */
  MAC_TXDESCRIPTORNUM = TxFragmentsPcs - 1 ;    /* Liczba dostepnych dla DMA deskryptorw */
  MAC_TXPRODUCEINDEX  = 0 ;                     /* Wskazanie na pierwszy wolny deskryptor */
} /* InitEMACDescr */


static void WriteToPHY ( ULONG  PhyRegisterSelector ,
                         USHORT Value )
{
  ULONG TimeOutLoop ;
  /*-------------------------------------------------------------------------*/
  MAC_MADR = DP83848C_DEF_ADR | PhyRegisterSelector ;   /* MII Mgmt Address register */
  MAC_MWTD = Value ;                                    /* MII Mgmt Write Data register */
  for ( TimeOutLoop = 0 ; TimeOutLoop < MII_WR_TOUT ; TimeOutLoop ++ )
  {
    if ( ( MAC_MIND & MIND_BUSY ) == 0 )
     break ;
  } /* for */ ;
} /* WriteToPHY */


static USHORT ReadFromPHY ( ULONG PhyRegisterSelector )
{
  ULONG TimeOutLoop ;
  /*-------------------------------------------------------------------------*/
  MAC_MADR = DP83848C_DEF_ADR | PhyRegisterSelector ;   /* MII Mgmt Address register */
  MAC_MCMD = MCMD_READ ;                                /* MII Mgmt Command -> read  */
  for ( TimeOutLoop = 0 ; TimeOutLoop < MII_RD_TOUT ; TimeOutLoop ++ )
  {
    if ( ( MAC_MIND & MIND_BUSY ) == 0 )
      break ;
  } /* for */ ;
  MAC_MCMD = 0 ;
  return ( MAC_MRDD ) ;                                /* MII Mgmt Read Data register */
} /* ReadFromPHY */


EMACInstanceRecT * InitPHYLayerInstance ( UCHAR *        MACAddress ,
                                          EtherSpeedMode SpeedMode )
{
  ULONG Loop ;
  ULONG RegisterValue ;
  ULONG Ident1 ;
  ULONG Ident2 ;
  /*-------------------------------------------------------------------------*/
  EMACInstanceRec . CheckEthInterface = PoolEthInterface ;
  EMACInstanceRec . InterfaceProcessFrame = IgnoreFrame ;
  EMACInstanceRec . InterfaceAllocFrameMemory = DefaultAllocation ;
  EMACInstanceRec . InterfaceDeallocFrameMemory = DefaultDeallocation ;
  EMACInstanceRec . InterfaceSendFrame = EMACSendFrame ;
  for ( Loop = 0 ; Loop < 6 ; Loop ++ )
    EMACInstanceRec . InterfaceMACAddress [ Loop ] = * MACAddress ++ ;
  PCONP |= PCENET ;                                     /* wlaczenie zasilania bloku ethernetowego */
                                /* Port P1 jako interfejs do ukdadu PHY */
                                /* P1.0  ENET_TXD0  Ethernet        => PINSEL2 1:0   = 01 */
                                /* P1.1  ENET_TXD1  Ethernet        => PINSEL2 3:2   = 01 */
                                /* P1.2  RESERVED                    => PINSEL2 5:4   = -- */
                                /* P1.3  RESERVED                    => PINSEL2 7:6   = -- */
                                /* P1.4  ENET_TX_EN  Ethernet       => PINSEL2 9:8   = 01 */
                                /* P1.5  RESERVED                    => PINSEL2 11:10 = -- */
                                /* P1.6  RESERVED                    => PINSEL2 13:12 = -- */
                                /* P1.7  RESERVED                    => PINSEL2 15:14 = -- */
                                /* P1.8  ENET_CRS  Ethernet         => PINSEL2 17:16 = 01 */
                                /* P1.9  ENET_RXD0  Ethernet        => PINSEL2 19:18 = 01 */
                                /* P1.10 ENET_RXD1  Ethernet        => PINSEL2 21:20 = 01 */
                                /* P1.11 RESERVED                    => PINSEL2 23:22 = -- */
                                /* P1.12 RESERVED                    => PINSEL2 25:24 = -- */
                                /* P1.13 RESERVED                    => PINSEL2 27:26 = -- */
                                /* P1.14 ENET_RX_ER  Ethernet       => PINSEL2 29:28 = 01 */
                                /* P1.15 ENET_REF_CLK  Ethernet     => PINSEL2 31:30 = 01 */
  if ( MAC_MODULEID == OLD_EMAC_MODULE_ID )
	{ 
    /* Dla starych wersji procesorow P1.6 musi byc ustawiony. */
    PINSEL2 = 0x50151105 ;
  } /* if ... */
  else
	{
    /* Dla nowych wersji procesorow P1.6 NIE moze byc ustawiony. */
    PINSEL2 = 0x50150105 ;
  } /* if ... else */ ;
  PINSEL3 = ( PINSEL3 & ( ~ 0x0000000F ) ) | 0x00000005 ;
                                /* P1.16 ENET_MDC  Ethernet         => PINSEL3 1:0   = 01 */
                                /* P1.17 ENET_MDIO  Ethernet        => PINSEL3 3:2   = 01 */
                                /* P1.18 COS TAM                     => PINSEL3 5:4   = -- */
                                /* P1.19 COS TAM                     => PINSEL3 7:6   = -- */
                                /* P1.20 COS TAM                     => PINSEL3 9:8   = -- */
                                /* P1.21 COS TAM                     => PINSEL3 11:10 = -- */
                                /* P1.22 COS TAM                     => PINSEL3 13:12 = -- */
                                /* P1.23 COS TAM                     => PINSEL3 15:14 = -- */
                                /* P1.24 COS TAM                     => PINSEL3 17:16 = -- */
                                /* P1.25 COS TAM                     => PINSEL3 19:18 = -- */
                                /* P1.26 COS TAM                     => PINSEL3 21:20 = -- */
                                /* P1.27 COS TAM                     => PINSEL3 23:22 = -- */
                                /* P1.28 COS TAM                     => PINSEL3 25:24 = -- */
                                /* P1.29 COS TAM                     => PINSEL3 27:26 = -- */
                                /* P1.30 COS TAM                     => PINSEL3 29:28 = -- */
                                /* P1.31 COS TAM                     => PINSEL3 31:30 = -- */
  MAC_MAC1 = MAC1_RES_TX | MAC1_RES_MCS_TX | MAC1_RES_RX | MAC1_RES_MCS_RX |
             MAC1_SIM_RES | MAC1_SOFT_RES ;             /* Wpis do rejestru konfiguracyjnego 1: */
  MAC_COMMAND = CR_REG_RES | CR_TX_RES | CR_RX_RES ;    /* Reset interfejsu ethernetowego jako : */
                                                        /* Reset Host Registers */
                                                        /* Reset Transmit Datapath */
                                                        /* Reset Receive Datapath */
  for ( Loop = 200 ; Loop ; Loop -- )                   /* Opoznienie po operacji reset */
  {
    nop ( ) ;
  } /* for */ ;
  MAC_MAC1 = MAC1_PASS_ALL ;                            /* Po wyzerowaniu wpis: */
  MAC_MAC2 = MAC2_CRC_EN | MAC2_PAD_EN ;                /* Rejestr konfiguracyjny: */
  MAC_MAXF = EtherMTU ;                                 /* MTU, czyli 600hex */
  MAC_CLRT = CLRT_DEF ;
  MAC_IPGR = IPGR_DEF ;
  MAC_COMMAND = CR_RMII | CR_PASS_RUNT_FRM ;            /* Wlaczenie interfejsu RMII */
  MAC_SUPP = SUPP_RES_RMII ;                            /* Reset Reduced MII Logic */
  for ( Loop = 100 ; Loop ; Loop -- )                   /* Opoznienie po operacji reset */
  {
    nop ( ) ;
  } /* for */ ;
  MAC_SUPP = 0 ;                                        /* RMII interface: 10Mbps */
  WriteToPHY ( DP83848_BMCR , DP83848_BMCR_Reset ) ;    /* Reset dla DP83848C */
  for ( Loop = 0 ; Loop < 0x100000 ; Loop ++ )          /* Czekamy */
  {
    RegisterValue = ReadFromPHY ( DP83848_BMCR ) ;
    if ( ! ( RegisterValue & 0x8000 ) )                 /* 8000 oznacza Reset trwa. */
    {
       break;                                           /* Reset kompletny */
    } /* if */ ;
  } /* for */ ;
  Ident1 = ReadFromPHY ( DP83848_IDR1 ) ;               /* Sprawdzenie czy to DP83848C PHY. */
  Ident2 = ReadFromPHY ( DP83848_IDR2 ) ;
  if ( ( Ident1 == DP83848C_ID1 ) && ( ( Ident2 & 0xFFF0 ) == DP83848C_ID2 ) )
  {
    switch ( SpeedMode )
    {
      case Mode_10MBIT :
        WriteToPHY ( DP83848_BMCR , DP83848_FULLD_10M ) ;  /* Polaczenie na 10MBit */
        break ;
      case Mode_100MBIT :
        WriteToPHY ( DP83848_BMCR , DP83848_FULLD_100M ) ; /* Polaczenie na 100MBit */
        break ;
      case Mode_AutoNeg :
        WriteToPHY ( DP83848_BMCR , DP83848_AUTO_NEG ) ;   /* Polecenie autonegocjacji dla DP83848 */
        for ( Loop = 0 ; Loop < 0x100000 ; Loop ++ )       /* Czekamy */
        {
          RegisterValue = ReadFromPHY ( DP83848_BMSR ) ;
          if ( RegisterValue & DP83848_Auto_Neg_Compl )
          {
            break ;                                        /* Autonegotiation Complete. */
          } /* if */ ;
        } /* for */ ;
        break ;
      default :
        WriteToPHY ( DP83848_BMCR , DP83848_FULLD_10M ) ;  /* Polaczenie na 10MBit */
    } /* switch */ ;
  } /* if */ ;
  for ( Loop = 0 ; Loop < 0x100000 ; Loop ++ )             /* Sprawdzenie statusu LINK */
  {
    RegisterValue = ReadFromPHY ( DP83848_STS ) ;
    if ( RegisterValue & 0x0001 )
    {
       break ;                                             /* LINK jest -> dioda swieci */
    } /* if */ ;
  } /* for */ ;
  if ( RegisterValue & DP83848_Duplex_Status )
  {
    MAC_MAC2 |= MAC2_FULL_DUP ;
    MAC_COMMAND |= CR_FULL_DUP ;
    MAC_IPGT = IPGT_FULL_DUP ;
  } /* if ... */
  else
  {
    MAC_IPGT = IPGT_HALF_DUP ;
  } /* if ... else */ ;
  if ( RegisterValue & DP83848_Speed_Stat_10M )
  {
    MAC_SUPP = 0 ;                                         /* 10MBit mode. */
  } /* if ... */
  else
  {
    MAC_SUPP = SUPP_SPEED ;                                /* 100MBit mode. */
  } /* if ... else */ ;
                                                           /* Ustawienie adresu MAC */
  MAC_SA0 = ( ( ULONG ) EMACInstanceRec . InterfaceMACAddress [ 1 ] << 8 ) |
              ( ULONG ) EMACInstanceRec . InterfaceMACAddress [ 0 ] ;
  MAC_SA1 = ( ( ULONG ) EMACInstanceRec . InterfaceMACAddress [ 3 ] << 8 ) |
              ( ULONG ) EMACInstanceRec . InterfaceMACAddress [ 2 ] ;
  MAC_SA2 = ( ( ULONG ) EMACInstanceRec . InterfaceMACAddress [ 5 ] << 8 ) |
              ( ULONG ) EMACInstanceRec . InterfaceMACAddress [ 4 ] ;
  InitEMACDescr ( ) ;
  MAC_RXFILTERCTRL = RFC_UCAST_EN | RFC_BCAST_EN | RFC_PERFECT_EN ;
                                                      /* Zapis do: Receive Filter Control Register */
  MAC_INTENABLE = 0 ;                                 /* Blokada przerwan dla EMAC */
//  MAC_INTCLEAR  = 0xFFFF ;                          /* Reset przerwan od EMAC */
  MAC_COMMAND |= ( CR_RX_EN | CR_TX_EN ) ;            /* Dodano do rejestru Command Register: */
                                                      /* Zezwolenie na odbiow */
                                                      /* Zezwolenie na nadawanie */
  MAC_MAC1 |= MAC1_REC_EN ;                           /* Dodano do rejestru MAC Configuration Register 1: */
  return ( & EMACInstanceRec ) ;
} /* InitPHYLayerInstance */


