/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2007                               **
**                                                                        **
**                          MODULE EMACDriver                             **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2007.03.22                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#ifndef _EMAC_Service_
#define _EMAC_Service_

#include "types.h"

#define IPV4                4
#define EthMinFrameSize     64
#define EthMaxFrameSzie     1536
#define EtherMTU            EthMaxFrameSzie

#define EthMACAddressSize    6
typedef struct {
                 UCHAR                DestinMACAddress [ EthMACAddressSize ] ;  /* =  6 */
                 UCHAR                SourceMACAddress [ EthMACAddressSize ] ;  /* =  6 */
                 USHORT               LengthOrType ;                            /* =  2 */
               } EtherPackedHeaderRecT ;                                        /* = 14 */
#define EtherPackedHeaderRecTSize     14

#define DestinMACAddressOffset        0
#define SourceMACAddressOffset        DestinMACAddressOffset + EthMACAddressSize
#define LengthOrTypeOffset            SourceMACAddressOffset + EthMACAddressSize

#define FRAME_ARP                0x0806
#define FRAME_IP                 0x0800

#define EthHardwType             1
#define IP_HLen_PLen             0x0604

typedef struct {
                 ULONG         Length ;
                 ULONG         Index ;
//                 UCHAR         Data [ 3 ] ;
                 UCHAR         Data [ ] ;
               } OSFrameRecT ;
typedef OSFrameRecT * PtrOSFrameRecT ;

typedef enum { Mode_10MBIT , Mode_100MBIT , Mode_AutoNeg } EtherSpeedMode ;

#define SendFrame_OK        0           /* OK */
#define SendFrame_DescrOvl  1           /* brak wolnego deskryptora nadajnika */

typedef ULONG ( * SendFrameServiceProcT ) ( OSFrameRecT * /* Frame */ ) ;
                                        /* RETURN: SendFrame_OK                 */
                                        /*         SendFrame_DescrOvl           */
typedef void ( * FrameServiceProcT ) ( OSFrameRecT * /* Frame */ ) ;

typedef PtrOSFrameRecT ( * AllocFrameMemoryProcT ) ( ULONG /* Size */ ) ;
                                        /* RETURN: wskaznik do przydzielonego obszaru lub NULL */

typedef void ( * DeallocFrameMemoryProcT ) ( PtrOSFrameRecT /* Frame */ ) ;

typedef void ( * VoidServiceProcT ) ( void ) ;

typedef struct {
                 UCHAR                   InterfaceMACAddress [ 6 ] ;
                 VoidServiceProcT        CheckEthInterface ;                /* pobranie ramki ethernetowej */
                 FrameServiceProcT       InterfaceProcessFrame ;            /* obrobka odebranej ramki ethernetowej */
                 AllocFrameMemoryProcT   InterfaceAllocFrameMemory ;        /* przydzial pamieci na odebrana ramke */
                 DeallocFrameMemoryProcT InterfaceDeallocFrameMemory ;      /* zwolnienie pamieci przydzielonej na ramke */
                 SendFrameServiceProcT   InterfaceSendFrame ;               /* wyslanie ramki */
               } EMACInstanceRecT ;
typedef EMACInstanceRecT * PtrEMACInstanceRecT ;
							 
extern EMACInstanceRecT EMACInstanceRec ;

extern EMACInstanceRecT * InitPHYLayerInstance ( UCHAR *        /* MACAddress */ ,
                                                 EtherSpeedMode /* SpeedMode  */ ) ;

#endif
