/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                            BIALYSTOK 2018                              **
**                                                                        **
**                          MODULE HTTPService                            **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2018.06.12                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.H>
#include "httpservice.h"
#include "ipservice.h"
#include "tcpservice.h"
#include "strings.h"
#include "systime.h"
#include "dataservice.h"
#include "leddisplay.h"

#define NetBufferSize                1024
#define InputCommandSize             32
#define CloseDelayInterval           150

static UCHAR NetBuffer [ NetBufferSize ] ;
static USHORT NetBufferIndex ;
static UCHAR InputCommand [ InputCommandSize ] ;


static const UCHAR PageNotFound [ ] = 
	       "HTTP/1.1 404 Not Found\r\n\r\n" ;
static const UCHAR EpmtyPage [ ] = 
	       "HTTP/1.1 200 OK\r\n\r\n<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\r\n"
         "<title>LPC2378 Domowy system automatyki</title></head>\r\n"
         "<body bgcolor=\"#00BF00\" text=\"#FFFF00\">"
         "<b><font color=\"#FFFFFF\" size=\"8\">Strona nie istnieje<br><br>"
         "</font></body></html>\r\n\r\n" ;
static const UCHAR IndexPage [ ] = 
         "HTTP/1.1 200 OK\r\n\r\n<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\r\n"
         "<title>LPC2378 Domowy system automatyki</title></head>\r\n"
         "<body bgcolor=\"#00BF00\" text=\"#FFFF00\">"
         "<p align=\"center\"><b><font color=\"#FFFFFF\" size=\"8\">"
         "<i>Domowa automatyka: sterowanie i pomiary.<br>Witaj Kole\xC5\x9B przy kompie.</i>"
//                                                           ^^^ \xC5\x9B - litera s' w UTF-8
         "</font></b></p><p align=\"center\"><b><font size=\"4\">System pomiarowy: INFINITY</b></font></p><br><br>"
         "<b><font color=\"#0000FF\" size=\"6\">Sterowanie: </b></font><font size=\"6\"><a href=\"control\">KLIK</a></font>"
         "<br><br><br><br>"
         "<b><font color=\"#0000FF\" size=\"6\">Pomiary: </b></font><font size=\"6\"><a href=\"means\">KLIK</a></font><br>"
         "</body></html>\r\n\r\n" ;
static const UCHAR ControlOpenPage [ ] = 
         "HTTP/1.1 200 OK\r\n\r\n<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\r\n"
         "<meta http-equiv=\"refresh\" content=\"5\">\n"
         "<title>LPC2378: sterowanie</title></head>\r\n"
         "<body bgcolor=\"#00BF00\" text=\"#FFFF00\">"
         "<p align=\"center\"><b><font color=\"#FFFFFF\" size=\"8\">"
         "<i>Domowa automatyka: sterowanie.</i>"
         "</font></b></p><p align=\"center\"><b><font size=\"4\">System pomiarowy: INFINITY</b></font></p><br><br>" ;
static const UCHAR ControlDataPage [ ] = 
         "<b><font color=\"#0000FF\" size=\"6\">` : <a href=\"@\" >W\xC5\x81\xC4\x84\x43Z</a> <a href=\"$\">WY\xC5\x81\xC4\x84\x43Z</a></font></b><br>"
//                                                         ^^^ \xC5\x81 - litera L/ w UTF-8               ^^^ \xC5\x81 - litera L/ w UTF-8
//                                                                  ^^^ \xC4\x84 - litera A, w UTF-8          ^^^ \xC4\x84 - litera A, w UTF-8
         "<table bgcolor=\"%\" border=\"5\" cellpadding=\"0\" cellspacing=\"0\" width=\"8\"><tbody><tr><td>&nbsp;&nbsp;&nbsp;&nbsp;</td></tr></tbody></table>" ;
static const UCHAR ControlClosePage [ ] = 
         "<br><br><br><br><font size=\"6\"><a href=\"index\">WR\xC3\x93\xC4\x86</a></font></body></html>\r\n\r\n" ;
//                                                       ^^^ \xC3\x93 - litera O' w UTF-8
//                                                               ^^^ \xC4\x86 - litera C' w UTF-8
static const UCHAR MeansOpenPage [ ] = 
         "HTTP/1.1 200 OK\r\n\r\n<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">"
         "<meta http-equiv=\"refresh\" content=\"5\">"
         "<title>LPC2378: pomiary</title></head>"
         "<body bgcolor=\"#00BF00\" text=\"#FFFF00\">"
         "<p align=\"center\"><b><font color=\"#FFFFFF\" size=\"8\">"
         "<i>Domowa automatyka: pomiary.</i>"
         "</font></b></p><p align=\"center\"><b><font size=\"4\">System pomiarowy: INFINITY</b></font></p><br><br>" ;
static const UCHAR MeansDataPage [ ] = 
         "<b><font color=\"#0000FF\" size=\"6\">@ = $</font></b><br>" ;
static const UCHAR MeansClosePage [ ] = 
         "<br><br><br><br><font size=\"6\"><a href=\"index\">WR\xC3\x93\xC4\x86</a></font></body></html>\r\n\r\n" ;
//                                                        ^^^ \xC3\x93 - litera O' w UTF-8
//                                                               ^^^ \xC4\x86 - litera C' w UTF-8
static const UCHAR SensorDataPage [ ] = 
         "<b><font color=\"#0000FF\" size=\"6\">@</font></b><br>"
         "<table bgcolor=\"%\" border=\"5\" cellpadding=\"0\" cellspacing=\"0\" width=\"8\"><tbody><tr><td>&nbsp;&nbsp;&nbsp;&nbsp;</td></tr></tbody></table>" ;
static const UCHAR RedicrectPage [ ] =
         "HTTP/1.1 200 OK\r\n\r\n<html><head><meta http-equiv=\"refresh\" content=\"0; url=control\"></head></html>\r\n" ;

static const UCHAR GetStartPagePattern [ ] = "GET / HTTP/1" ;
static const UCHAR GetIndexPagePattern [ ] = "GET /index HTTP/1" ;
static const UCHAR GetFaviconPattern [ ]   = "GET /favicon.ico HTTP/1" ;
static const UCHAR GetMeansPattern [ ]     = "GET /means HTTP/1" ;
static const UCHAR GetControlPattern [ ]   = "GET /control HTTP/1" ;
static const UCHAR FanOnPattern [ ]        = "GET /fon HTTP/1" ;
static const UCHAR FanOffPattern [ ]       = "GET /fof HTTP/1" ;
static const UCHAR LightOnPattern [ ]      = "GET /lon HTTP/1" ;
static const UCHAR LightOffPattern [ ]     = "GET /lof HTTP/1" ;

static const UCHAR Device00Name [ ]        = "Czujnik CO" ;
static const UCHAR Device01Name [ ]        = "Wentylator" ;
static const UCHAR Device02Name [ ]        = "O\xC5\x9Bwietlenie" ;
//                                            ^^^ \xC5\x9B - litera s' w UTF-8
static const UCHAR Device03Name [ ]        = "Pompa obiegowa" ;
static const UCHAR Device01OnControl [ ]   = "fon" ;
static const UCHAR Device01OffControl [ ]  = "fof" ;
static const UCHAR Device02OnControl [ ]   = "lon" ;
static const UCHAR Device02OffControl [ ]  = "lof" ;

static const UCHAR ColorOn [ ]             = "ff0000" ;
static const UCHAR ColorOff [ ]            = "0000ff" ;

static USHORT SendHTTPSegment ( PtrTCPSocketDescriptionRecT TCPSocket ,
	                              UCHAR *                     TCPBuffer ,
                                USHORT                      TCPBufferSize )
{
  PtrTCPLayerInstanceRecT TCPLayerInstance ;
  PtrIPLayerInstanceRecT  IPLayerInstance ;
	USHORT SendStatus ;
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  TCPLayerInstance = GetTCPInstance ( TCPSocket ) ;
  IPLayerInstance = GetIPInstanceTCP ( TCPLayerInstance ) ;
  for ( Loop = 0 ; Loop < 20 ; Loop ++ )
  {
    SendStatus = TCPSendData ( TCPSocket , TCPBuffer , TCPBufferSize ) ;
		switch ( SendStatus )
		{
      case TCPSend_OK         :
				return 1 ;
      case TCPSend_NoConnect  :
				return 0 ;
      case TCPSend_Overflow   :
        IPLayerInstance -> IPLayerPoolService ( IPLayerInstance ) ;
        SysTimerPool ( ) ;
				break ;
			default :
				return 0 ;
		} /* switch */ ;
  } /* for */ ;
	return 0 ;
} /* SendHTTPSegment */


static void SendConstData ( PtrTCPSocketDescriptionRecT TCPSocket ,
	                          const UCHAR *               PageContent ,
														USHORT                      PageSize )
{
  const UCHAR * ChPtr ;
  UCHAR Ch ;
  /*-------------------------------------------------------------------------*/
  ChPtr = PageContent ;
	for ( ; ; )
	{
    NetBufferIndex = 0 ;
		for ( ; ; )
		{
		  Ch = * ChPtr ++ ;
      NetBuffer [ NetBufferIndex ] = Ch ;
			NetBufferIndex ++ ;
      PageSize -- ;
      if ( ! PageSize )
			{
        if ( NetBufferIndex )
		    {
          SendHTTPSegment ( TCPSocket , NetBuffer , NetBufferIndex ) ;
          return ;
		    } /* if */ ;
			} /* if */ ;
			if ( NetBufferIndex >= NetBufferSize )
				break ;
		} /* for */ ;
		if ( NetBufferIndex )
		{
      if ( ! SendHTTPSegment ( TCPSocket , NetBuffer , NetBufferIndex ) )
			{
				return ;
			} /* if */ ;
		} /* if */ ;
	} /* for */ ;
} /* SendConstData */


static USHORT ComparePageName ( const UCHAR * Pattern )
{
	const UCHAR * PatternCh ;
	UCHAR Ch ;
	USHORT Loop ;
  /*-------------------------------------------------------------------------*/
	PatternCh = Pattern ;
  for ( Loop = 0 ; Loop < InputCommandSize ; Loop ++ )
  {
	  Ch = * PatternCh ++ ;
		if ( ! Ch )
  		return 1 ;
		if ( InputCommand [ Loop ] != Ch )
			return 0 ;
	} /* for */ ;
	return 1 ;
} /* ComparePageName */


static void ExtractHTTPCommand (  UCHAR * CommandBuffer ) 
{
  USHORT Loop ;
  UCHAR * ChPtr ;
	UCHAR Ch ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < InputCommandSize ; Loop ++ )
    InputCommand [ Loop ] = 0 ;
  ChPtr = CommandBuffer ;
	for ( Loop = 0 ; Loop < InputCommandSize ; Loop ++ )
	{
    Ch = * ChPtr ++ ;
		if ( Ch < 32 )
		  break ;
    InputCommand [ Loop ] = Ch ;
	} /* for */ ;
} /* ExtractHTTPCommand */


static void DelayedSocketClose ( void * CallParam )
{
	PtrTCPSocketDescriptionRecT TCPSocket ;
  /*-------------------------------------------------------------------------*/
	TCPSocket = ( PtrTCPSocketDescriptionRecT ) CallParam ;
  TCPClose ( TCPSocket ) ;
} /* DelayedSocketClose */ ;


static USHORT AddHTTPCh ( PtrTCPSocketDescriptionRecT TCPSocket ,
	                        UCHAR                       Ch )
{
  /*-------------------------------------------------------------------------*/
  NetBuffer [ NetBufferIndex ++ ] = Ch ;
  if ( NetBufferIndex >= NetBufferSize )
  {
    if ( ! SendHTTPSegment ( TCPSocket , NetBuffer , NetBufferIndex ) )
		  return 0 ;
    NetBufferIndex = 0 ;
	} /* if */ ;
	return 1 ;
} /* AddHTTPCh */


static void CopyHTTPSegment ( PtrTCPSocketDescriptionRecT TCPSocket ,
	                            const UCHAR *               Page )
{
	UCHAR Ch ;
	const UCHAR * ChPtr ;
  /*-------------------------------------------------------------------------*/
	ChPtr = Page ;
	for ( ; ; )
	{
		Ch = * ChPtr ++ ;
		if ( ! Ch )
			return ;
		if ( ! AddHTTPCh ( TCPSocket , Ch ) )
			return ;
	} /* for */ ;
} /* CopyHTTPSegment */


static void SendStatusValue ( PtrTCPSocketDescriptionRecT TCPSocket ,
                              const UCHAR *               DeviceName ,
                              const UCHAR *               OnStateVariant ,
                              const UCHAR *               OffStateWariant ,
                              UCHAR *                     Color )
{
	UCHAR Ch ;
	const UCHAR * ChPtr ;
  /*-------------------------------------------------------------------------*/
	ChPtr = ControlDataPage ;
	for ( ; ; )
	{
		Ch = * ChPtr ++ ;
		if ( ! Ch )
			return ;
		if ( Ch == '`' )
		{
      CopyHTTPSegment ( TCPSocket , DeviceName ) ;
			continue ;
		} /* if */ ;
		if ( Ch == '@' )
		{
      CopyHTTPSegment ( TCPSocket , OnStateVariant ) ;
			continue ;
		} /* if */ ;
		if ( Ch == '$' )
		{
      CopyHTTPSegment ( TCPSocket , OffStateWariant ) ;
			continue ;
		} /* if */ ;
		if ( Ch == '%' )
		{
      CopyHTTPSegment ( TCPSocket , Color ) ;
			continue ;
		} /* if */ ;
    if ( ! AddHTTPCh ( TCPSocket , Ch ) )
			return ;
	} /* for */ ;
} /* SendStatusValue */


static void Assign ( UCHAR *       OutStr ,
	                   const UCHAR * InStr )
{
	UCHAR Ch ;
  /*-------------------------------------------------------------------------*/
	for ( ; ; )
  {
		Ch = * InStr ++ ;
		* OutStr ++ = Ch ;
		if ( ! Ch )
			break ;
	} /* for */ ;
} /* Assign */


static void SetColor ( UCHAR * Color , 
	                     UCHAR OnStatus )
{
  /*-------------------------------------------------------------------------*/
  if ( OnStatus )
    Assign ( Color , ColorOn ) ;
	else
    Assign ( Color , ColorOff ) ;
} /* SetColor */


static void SendDataValue ( PtrTCPSocketDescriptionRecT TCPSocket ,
	                          UCHAR *                     SymbolID ,
	                          UCHAR *                     Value )
{
	UCHAR Ch ;
	const UCHAR * ChPtr ;
  /*-------------------------------------------------------------------------*/
	ChPtr = MeansDataPage ;
	for ( ; ; )
	{
		Ch = * ChPtr ++ ;
		if ( ! Ch )
			return ;
		if ( Ch == '@' )
		{
      CopyHTTPSegment ( TCPSocket , SymbolID ) ;
			continue ;
		} /* if */ ;
		if ( Ch == '$' )
		{
      CopyHTTPSegment ( TCPSocket , Value ) ;
			continue ;
		} /* if */ ;
    if ( ! AddHTTPCh ( TCPSocket , Ch ) )
			return ;
	} /* for */ ;
} /* SendDataValue */


static void SendSensorStatus ( PtrTCPSocketDescriptionRecT TCPSocket ,
                               const UCHAR *               DeviceName ,
                               UCHAR *                     Color )
{
	UCHAR Ch ;
	const UCHAR * ChPtr ;
  /*-------------------------------------------------------------------------*/
	ChPtr = SensorDataPage ;
	for ( ; ; )
	{
		Ch = * ChPtr ++ ;
		if ( ! Ch )
			return ;
		if ( Ch == '@' )
		{
      CopyHTTPSegment ( TCPSocket , DeviceName ) ;
			continue ;
		} /* if */ ;
		if ( Ch == '%' )
		{
      CopyHTTPSegment ( TCPSocket , Color ) ;
			continue ;
		} /* if */ ;
    if ( ! AddHTTPCh ( TCPSocket , Ch ) )
			return ;
	} /* for */ ;
} /* SendSensorStatus */


static void HTTPSendControl ( PtrTCPSocketDescriptionRecT TCPSocket )
{
  UCHAR Color [ 8 ] ;
  /*-------------------------------------------------------------------------*/
  NetBufferIndex = 0 ;
	CopyHTTPSegment ( TCPSocket , ControlOpenPage ) ;
	SetColor ( Color , On1Status ) ;
	SendStatusValue ( TCPSocket , Device01Name , Device01OnControl , Device01OffControl , Color ) ;
	SetColor ( Color , On2Status ) ;
	SendStatusValue ( TCPSocket , Device02Name , Device02OnControl , Device02OffControl , Color ) ;
	SetColor ( Color , On0Status ) ;
	SendSensorStatus ( TCPSocket , Device00Name , Color ) ;
	SetColor ( Color , On3Status ) ;
	SendSensorStatus ( TCPSocket , Device03Name , Color ) ;
	CopyHTTPSegment ( TCPSocket , ControlClosePage ) ;
	if ( NetBufferIndex )
	{
    SendHTTPSegment ( TCPSocket , NetBuffer , NetBufferIndex ) ;
	} /* if */ ;
  AttachTimer ( CloseDelayInterval , DelayedSocketClose , ( void * ) TCPSocket ) ;
} /* HTTPSendControl */


static void HTTPSendMeans ( PtrTCPSocketDescriptionRecT TCPSocket )
{
  UCHAR Color [ 8 ] ;
  /*-------------------------------------------------------------------------*/
  NetBufferIndex = 0 ;
	CopyHTTPSegment ( TCPSocket , MeansOpenPage ) ;
	SendDataValue ( TCPSocket , ( UCHAR * ) "Temp pieca" , Analog1Value ) ;
	SendDataValue ( TCPSocket , ( UCHAR * ) "Temp spalin" , Analog3Value ) ;
	SendDataValue ( TCPSocket , ( UCHAR * ) "Temp wewn\xC4\x99trzna" , Analog2Value ) ;
//                                           ^^^ \xC4\x99 - litera e, w UTF-8
	SendDataValue ( TCPSocket , ( UCHAR * ) "Temp zewn\xC4\x99trzna" , Analog5Value ) ;
//                                           ^^^ \xC4\x99 - litera e, w UTF-8
	SendDataValue ( TCPSocket , ( UCHAR * ) "Wilgotno\xC5\x9B\xC4\x87" , Analog4Value ) ;
//                                                 ^^^ \xC5\x9B - litera s' w UTF-8
//                                                          ^^^  \xC4 \x87 - litera c' w UTF-8
	SendDataValue ( TCPSocket , ( UCHAR * ) "Ci\xC5\x9Bnienie" , Analog6Value ) ;
//                                           ^^^ \xC5\x9B - litera s' w UTF-8
	SetColor ( Color , On0Status ) ;
	SendSensorStatus ( TCPSocket , Device00Name , Color ) ;
	SetColor ( Color , On3Status ) ;
	SendSensorStatus ( TCPSocket , Device03Name , Color ) ;
	CopyHTTPSegment ( TCPSocket , MeansClosePage ) ;
	if ( NetBufferIndex )
	{
    SendHTTPSegment ( TCPSocket , NetBuffer , NetBufferIndex ) ;
	} /* if */ ;
  AttachTimer ( CloseDelayInterval , DelayedSocketClose , ( void * ) TCPSocket ) ;
} /* HTTPSendMeans */ ;


static void HTTPRedirect ( PtrTCPSocketDescriptionRecT TCPSocket )
{
  /*-------------------------------------------------------------------------*/
  NetBufferIndex = 0 ;
	CopyHTTPSegment ( TCPSocket , RedicrectPage ) ;
	if ( NetBufferIndex )
	{
    SendHTTPSegment ( TCPSocket , NetBuffer , NetBufferIndex ) ;
	} /* if */ ;
  AttachTimer ( CloseDelayInterval , DelayedSocketClose , ( void * ) TCPSocket ) ;
} /* HTTPRedirect */ ;


void TCPRecvOperation ( PtrTCPSocketDescriptionRecT TCPSocket ,
                        ULONG                       SenderIPAddress ,
                        USHORT                      SenderPortNo ,
                        ULONG                       DestIPAddress ,
                        USHORT                      DestPortNo ,
                        void *                      DataBuffer ,
                        USHORT                      DataBufferSize )
{
  /*-------------------------------------------------------------------------*/
  if ( DataBufferSize )
  {
 		ExtractHTTPCommand ( ( UCHAR * ) DataBuffer ) ;
		if ( ComparePageName ( GetStartPagePattern ) || ComparePageName ( GetIndexPagePattern ) )
		{
			SendConstData ( TCPSocket , IndexPage , sizeof ( IndexPage ) - 1 ) ;
      AttachTimer ( CloseDelayInterval , DelayedSocketClose , ( void * ) TCPSocket ) ;
			return ;
		} /* if */ ;
		if ( ComparePageName ( GetFaviconPattern ) )
		{
			SendConstData ( TCPSocket , PageNotFound , sizeof ( PageNotFound ) - 1 ) ;
      AttachTimer ( CloseDelayInterval , DelayedSocketClose , ( void * ) TCPSocket ) ;
			return ;
		} /* if */ ;
		if ( ComparePageName ( GetMeansPattern ) )
		{
			HTTPSendMeans ( TCPSocket ) ;
			return ;
		} /* if */ ;
		if ( ComparePageName ( GetControlPattern ) )
		{
			HTTPSendControl ( TCPSocket ) ;
			return ;
		} /* if */ ;
		if ( ComparePageName ( FanOnPattern ) )
		{
      Device01On ( ) ;
      HTTPRedirect ( TCPSocket ) ;
			return ;
		} /* if */ ;
		if ( ComparePageName ( FanOffPattern ) )
		{
      Device01Off ( ) ;
      HTTPRedirect ( TCPSocket ) ;
			return ;
		} /* if */ ;
		if ( ComparePageName ( LightOnPattern ) )
		{
      Device02On ( ) ;
      HTTPRedirect ( TCPSocket ) ;
			return ;
		} /* if */ ;
		if ( ComparePageName ( LightOffPattern ) )
		{
      Device02Off ( ) ;
      HTTPRedirect ( TCPSocket ) ;
			return ;
		} /* if */ ;
  } /* if */ ;
  SendConstData ( TCPSocket , EpmtyPage , sizeof ( EpmtyPage ) - 1 ) ;
  TCPClose ( TCPSocket ) ;
} /* TCPRecvOperation */


USHORT TCPConnectOperation ( PtrTCPSocketDescriptionRecT TCPSocket ,
                             ULONG                       RemoteAddress ,
                             USHORT                      RemotePort )
{
  /*-------------------------------------------------------------------------*/
  return ( TRUE ) ;
} /* TCPConnectOperation */


void TCPEstablishOperation ( PtrTCPSocketDescriptionRecT TCPSocket )
{
  /*-------------------------------------------------------------------------*/
  LEDOn ( HTTPConnectLED ) ;
} /* TCPEstablishOperation */

                                                                                
void TCPCloseOperation ( PtrTCPSocketDescriptionRecT TCPSocket )
{
  /*-------------------------------------------------------------------------*/
  LEDOff ( HTTPConnectLED ) ;
} /* TCPCloseOperation */


void InitWebService ( void )
{
  /*-------------------------------------------------------------------------*/
  NetBufferIndex = 0 ;
} /* InitBridge */

							 
