/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2018                               **
**                                                                        **
**                         MODULE I2C Service                             **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2018.06.12                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/


#include "i2cservice.h"
#include <LPC23xx.H>

#define NOP() __asm ( "nop" )

// SDA - P0.27
// SCL - P0.28

#define EEPROM_Address                0xA0

#define SCLPin                        28
#define SDAPin                        27

#define SetSDA    IOSET0 = ( ( ULONG ) 1 << SDAPin )
#define SetSCL    IOSET0 = ( ( ULONG ) 1 << SCLPin )
#define ClrSDA    IOCLR0 = ( ( ULONG ) 1 << SDAPin )
#define ClrSCL    IOCLR0 = ( ( ULONG ) 1 << SCLPin )


static void I2CDelay ( void )
{
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < 100 /*200*/ ; Loop ++ )
  {
    NOP ( ) ;
  } /* for */ ;
} /* I2CDelay */


static UCHAR Clock ( void )
{
  ULONG PortData ;
  /*-------------------------------------------------------------------------*/
  I2CDelay ( ) ;
  SetSCL ;
  I2CDelay ( ) ;
  PortData = IOPIN0 & ( ( ULONG ) 1 << SDAPin ) ;
  I2CDelay ( ) ;
  ClrSCL ;
  I2CDelay ( ) ;
  if ( PortData )
    return ( 1 ) ;
  else
    return ( 0 ) ;
} /* Clock */


static void OutByte ( UCHAR Data )
{
  /*-------------------------------------------------------------------------*/
  I2CDelay ( ) ;
  if ( Data & 0x80 )
    SetSDA ;
  else
    ClrSDA ;
  Clock ( ) ;
  if ( Data & 0x40 )
    SetSDA ;
  else
    ClrSDA ;
  Clock ( ) ;
  if ( Data & 0x20 )
    SetSDA ;
  else
    ClrSDA ;
  Clock ( ) ;
  if ( Data & 0x10 )
    SetSDA ;
  else
    ClrSDA ;
  Clock ( ) ;
  if ( Data & 0x08 )
    SetSDA ;
  else
    ClrSDA ;
  Clock ( ) ;
  if ( Data & 0x04 )
    SetSDA ;
  else
    ClrSDA ;
  Clock ( ) ;
  if ( Data & 0x02 )
    SetSDA ;
  else
    ClrSDA ;
  Clock ( ) ;
  if ( Data & 0x01 )
    SetSDA ;
  else
    ClrSDA ;
  Clock ( ) ;
} /* OutByte */


static UCHAR InByte ( void )
{
  UCHAR RetV ;
  /*-------------------------------------------------------------------------*/
  RetV = 0 ;
  SetSDA ;
  I2CDelay ( ) ;
  if ( Clock ( ) )
    RetV |= 0x80 ;
  I2CDelay ( ) ;
  if ( Clock ( ) )
    RetV |= 0x40 ;
  I2CDelay ( ) ;
  if ( Clock ( ) )
    RetV |= 0x20 ;
  I2CDelay ( ) ;
  if ( Clock ( ) )
    RetV |= 0x10 ;
  I2CDelay ( ) ;
  if ( Clock ( ) )
    RetV |= 0x08 ;
  I2CDelay ( ) ;
  if ( Clock ( ) )
    RetV |= 0x04 ;
  I2CDelay ( ) ;
  if ( Clock ( ) )
    RetV |= 0x02 ;
  I2CDelay ( ) ;
  if ( Clock ( ) )
    RetV |= 0x01 ;
  I2CDelay ( ) ;
  return ( RetV ) ;
} /* InByte */


static void NotACK ( void )
{
  /*-------------------------------------------------------------------------*/
  SetSDA ;
  I2CDelay ( ) ;
  Clock ( ) ;
} /* NotACK */


static UCHAR GetACK ( void )
{
  /*-------------------------------------------------------------------------*/
  SetSDA ;
  I2CDelay ( ) ;
  return ( Clock ( ) ) ;
} /* GetACK */


static void StartI2C ( void )
{
  /*-------------------------------------------------------------------------*/
  SetSDA ;
  I2CDelay ( ) ;
  SetSCL ;
  I2CDelay ( ) ;
  ClrSDA ;
  I2CDelay ( ) ;
  ClrSCL ;
  I2CDelay ( ) ;
} /* StartI2C */


static void StopI2C ( void )
{
  /*-------------------------------------------------------------------------*/
  ClrSDA ;
  I2CDelay ( ) ;
  SetSCL ;
  I2CDelay ( ) ;
  SetSDA ;
  I2CDelay ( ) ;
} /* StopI2C */


UCHAR GetEEPROMData ( USHORT Ofs )
{
  UCHAR EEData ;
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < 250 ; Loop ++ )
  {
    I2CDelay ( ) ;
  } /* for */ ;
  StartI2C ( ) ;
  I2CDelay ( ) ;
  OutByte ( EEPROM_Address ) ;
  I2CDelay ( ) ;
  GetACK ( ) ;
  OutByte ( Ofs ) ;
  I2CDelay ( ) ;
  GetACK ( ) ;
  I2CDelay ( ) ;
  StartI2C ( ) ;
  I2CDelay ( ) ;
  OutByte ( EEPROM_Address + 1 ) ;
  I2CDelay ( ) ;
  GetACK ( ) ;
  I2CDelay ( ) ;
  EEData = InByte ( ) ;
  NotACK ( ) ;
  I2CDelay ( ) ;
  StopI2C ( ) ;
  return ( EEData ) ;
} /* GetEEPROMData */


void SetEEPROMData ( USHORT Ofs ,
                     UCHAR Val )
{
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < 250 ; Loop ++ )
  {
    I2CDelay ( ) ;
  } /* for */ ;
  StartI2C ( ) ;
  I2CDelay ( ) ;
  OutByte ( EEPROM_Address ) ;
  I2CDelay ( ) ;
  GetACK ( ) ;
  OutByte ( Ofs ) ;
  I2CDelay ( ) ;
  GetACK ( ) ;
  OutByte ( Val ) ;
  I2CDelay ( ) ;
  NotACK ( ) ;
  I2CDelay ( ) ;
  StopI2C ( ) ;
} /* SetEEPROMData */


void I2CInitEnvir ( void )
{
  /*-------------------------------------------------------------------------*/
  IODIR0 |= ( ( ULONG ) 1 << SCLPin ) |
            ( ( ULONG ) 1 << SDAPin ) ;
  IOSET0 = ( ( ULONG ) 1 << SCLPin ) |
           ( ( ULONG ) 1 << SDAPin ) ;
} /* I2CInitEnvir */

