/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2007                               **
**                                                                        **
**                       MODULE IP memory services                        **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2007.03.22                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include "ipmemservice.h"
#include "types.h"
#include "emacdriver.h"


//#define PoolSize 16
#define PoolSize 8

typedef struct {
                 ULONG         Length ;
                 ULONG         Index ;
                 UCHAR         Data [ EthMaxFrameSzie ] ;
               } MemFrameRecT ;
typedef struct {
                 UCHAR                   BusyFlag [ PoolSize ] ;
                 PtrOSFrameRecT          FrameBuffer [ PoolSize ] ;
                 MemFrameRecT            BufferBody [ PoolSize ] ;
               } FramePoolRecT ;


static FramePoolRecT FramePool ;


PtrOSFrameRecT FrameMemoryAllocate ( ULONG Size )
{
  ULONG Loop ;
  PtrOSFrameRecT Frame ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < PoolSize ; Loop ++ )
  {
    if ( ! FramePool . BusyFlag [ Loop ] )
    {
      FramePool . BusyFlag [ Loop ] = 1 ;
      Frame =  FramePool . FrameBuffer [ Loop ] ;
      Frame -> Length = 0 ;
      Frame -> Index = 0 ;
      return ( Frame ) ;
    } /* if */ ;
  } /* for */ ;
  return NIL ;
} /* FrameMemoryAllocate  */


void FrameMemoryDeallocate ( PtrOSFrameRecT Frame )
{
  ULONG Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < PoolSize ; Loop ++ )
  {
    if ( FramePool . FrameBuffer [ Loop ] == Frame )
    {
      FramePool . BusyFlag [ Loop ] = 0 ;
      return ;
    } /* if */ ;
  } /* for */ ;
} /* FrameMemoryDeallocate  */


void InitIPLayerMemoryService ( void )
{
  ULONG Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < PoolSize ; Loop ++ )
  {
    FramePool . BusyFlag [ Loop ] = 0 ;
    FramePool . FrameBuffer [ Loop ] = ( PtrOSFrameRecT ) & ( FramePool . BufferBody [ Loop ] ) ;
  } /* for */ ;
} /* InitIPLayerMemoryService */

