/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2007                               **
**                                                                        **
**                             MODULE IP                                  **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2007.03.22                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/


#ifndef _ip_service_
#define _ip_service_

#include "emacdriver.h"
#include "types.h"

#define Prot_ICMP                1               // Internet Control Message Protocol
#define Prot_TCP                 6               // Transmission Control Protocol
#define Prot_UDP                 17              // User Datagram Protocol

#define StandardServiceType      0
#define StandardTTL              64

#define ARPTableSize             8

#define IPVersOffset                  EtherPackedHeaderRecTSize + 0
#define IPServiceTypeOffset           EtherPackedHeaderRecTSize + 1
#define IPTotalLengthOffset           EtherPackedHeaderRecTSize + 2
#define IPIdentificationOffset        EtherPackedHeaderRecTSize + 4
#define IPFragmentOffsOffset          EtherPackedHeaderRecTSize + 6
#define IPTTLOffset                   EtherPackedHeaderRecTSize + 8
#define IPProtocolOffset              EtherPackedHeaderRecTSize + 9
#define IPHeaderChecksumOffset        EtherPackedHeaderRecTSize + 10
#define IPSourceIPAddressOffset       EtherPackedHeaderRecTSize + 12
#define IPDestinIPAddressOffset       EtherPackedHeaderRecTSize + 16
#define IPDataOffset                  EtherPackedHeaderRecTSize + 20

typedef struct {
                 USHORT          ValidElement ;
                 USHORT          LifeTime ;
                 UCHAR           MACAddress [ EthMACAddressSize ] ;
                 ULONG           IPAddress ;
               } ARPTableElementRecT ;
typedef ARPTableElementRecT * PtrARPTableElementRecT ;

#define IPHeaderSize                  20

typedef struct {
                 UCHAR             Vers ;
                 UCHAR             HLen ;
                 UCHAR             ServiceType ;
                 UCHAR             Flags ;
                 UCHAR             TTL ;
                 UCHAR             Protocol ;
                 USHORT            TotalLength ;
                 USHORT            Identification ;
                 USHORT            FragmentOffs ;
                 USHORT            HeaderChecksum ;
                 ULONG             SourceIPAddress ;
                 ULONG             DestinationIPAddress ;
               } IPHeaderRecT ;

struct _IPLayerInstanceRec ;

typedef struct _IPLayerInstanceRec * PtrIPLayerInstanceRecT ;

typedef void ( * TCPTimerPoolServiceProcT ) ( void * /* TCPInstanceEntry */ ) ;
typedef void ( * VoidIPLayerServiceProcT ) ( PtrIPLayerInstanceRecT /* IPLayerInstance */ ) ;
typedef void ( * IPPackedServiceProcT ) ( PtrIPLayerInstanceRecT /* IPLayerInstance */ ,
                                          PtrOSFrameRecT         /* Frame           */ ,
                                          ULONG                  /* IPAddress       */ ) ;
typedef void ( * ProcessPackedServiceProcT ) ( PtrIPLayerInstanceRecT /* IPLayerInstance */ ,
                                               PtrOSFrameRecT         /* Frame           */ ) ;
typedef void ( * ProcessProtPServiceProcT ) ( PtrIPLayerInstanceRecT /* IPLayerInstance */ ,
                                              IPHeaderRecT *         /* IPHeader        */ ,
                                              PtrOSFrameRecT         /* Frame           */ ) ;

typedef struct _IPLayerInstanceRec {
                 void *                    UDPInstanceEntry ;
                 void *                    TCPInstanceEntry ;
                 ARPTableElementRecT       ARPTable [ ARPTableSize ] ;
                 ULONG                     MyLocalIPAddress ;
                 ULONG                     MyLocalNetMask ;
                 ULONG                     MyDefaultGWIPAddress ;
                 EMACInstanceRecT *        InterfaceInstance ;
                 ProcessPackedServiceProcT OthFrameProcessSerevice ;
                 ProcessPackedServiceProcT ARPProcessService ;
                 ProcessProtPServiceProcT  ICMPProcessService ;
                 ProcessProtPServiceProcT  ICMPEchoProcessService ;
                 ProcessProtPServiceProcT  UDPProcessService ;
                 ProcessProtPServiceProcT  TCPProcessService ;
                 ProcessPackedServiceProcT IPProcessService ;
                 TCPTimerPoolServiceProcT  TCPTimerPoolService ;
                 VoidIPLayerServiceProcT   DiagARPTableService ;
                 VoidIPLayerServiceProcT   IPLayerPoolService ;
                 VoidIPLayerServiceProcT   StandardTimeService ;
                 VoidIPLayerServiceProcT   ExtraTimeService ;
                 IPPackedServiceProcT      SendIPPackedService ;
                 IPPackedServiceProcT      AddToSendQueueService ;
               } IPLayerInstanceRecT ;

extern void SetIPAdresses ( PtrIPLayerInstanceRecT /* IPLayerInstance */ ,
                            ULONG                  /* IPAddress       */ ,
                            ULONG                  /* NetMask         */ ,
                            ULONG                  /* GateWayIP       */ ) ;

extern PtrEMACInstanceRecT GetEMACInstance ( PtrIPLayerInstanceRecT /* IPLayerInstance */ ) ;

extern PtrIPLayerInstanceRecT InitIPLayerInstance ( EMACInstanceRecT * /* EMACInstance */ ,
                                                    ULONG              /* IPAddress    */ ,
                                                    ULONG              /* NetMask      */ ) ;

#endif
