/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK, 2018                              **
**                                                                        **
**                          MODULE LEDService                             **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2018.06.12                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include "leddisplay.h"
#include <LPC23xx.H>


/*
PISNSEL3
Bity    PIN          00          01          10          11
1:0    P1.16 GPIO Port 1.16  ENET_MDC    Reserved      Reserved
3:2    P1.17 GPIO Port 1.17  ENET_MDIO   Reserved      Reserved
5:4    P1.18 GPIO Port 1.18  USB_UP_LED1 PWM1.1        CAP1.0
7:6    P1.19 GPIO Port 1.19 USB_TX_E1    USB_PPWR1     CAP1.1
9:8    P1.20 GPIO Port 1.20 USB_TX_DP1   PWM1.2        SCK0
11:10  P1.21 GPIO Port 1.21 USB_TX_DM1   PWM1.3        SSEL0
13:12  P1.22 GPIO Port 1.22 USB_RCV1     USB_PWRD1     MAT1.0        ---> LED
15:14  P1.23 GPIO Port 1.23 USB_RX_DP1   PWM1.4        MISO0         ---> LED
17:16  P1.24 GPIO Port 1.24 USB_RX_DM1   PWM1.5        MOSI0         ---> LED
19:18  P1.25 GPIO Port 1.25 USB_LS1      USB_HSTEN 1   MAT1.1        ---> LED
21:20  P1.26 GPIO Port 1.26 USB_SSPND1   PWM1.6        CAP0.0        ---> LED
23:22  P1.27 GPIO Port 1.27 USB_INT1     USB_OVRCR 1   CAP0.1        ---> LED
25:24  P1.28 GPIO Port 1.28 USB_SCL1     PCAP1.0       MAT0.0        ---> LED
27:26  P1.29 GPIO Port 1.29 USB_SDA1     PCAP1.1       MAT0.1        ---> LED
29:28  P1.30 GPIO Port 1.30 Reserved     VBUS          AD0.4
31:30  P1.31 GPIO Port 1.31 Reserved     SCK1          AD0.5
00001111111111111111000000000000
10987654321098765432109876543210
 3         2         1         0
*/

#define LED0Pin                        29
#define LED1Pin                        28
#define LED2Pin                        27
#define LED3Pin                        26
#define LED4Pin                        25
#define LED5Pin                        24
#define LED6Pin                        23
#define LED7Pin                        22


void LEDOn ( LEDSymbolType LEDSymbol )
{
  /*-------------------------------------------------------------------------*/
  switch ( LEDSymbol )
  {
    case StatusLED       :
      IOCLR1 = ( ULONG ) 1 << LED0Pin ;
      break ;
    case UART0TxDLED     :
      IOCLR1 = ( ULONG ) 1 << LED1Pin ;
      break ;
    case UART0RxDLED     :
      IOCLR1 = ( ULONG ) 1 << LED2Pin ;
      break ;
    case UART1TxDLED     :
      IOCLR1 = ( ULONG ) 1 << LED3Pin ;
      break ;
    case UART1RxDLED     :
      IOCLR1 = ( ULONG ) 1 << LED4Pin ;
      break ;
    case HTTPConnectLED  :
      IOCLR1 = ( ULONG ) 1 << LED5Pin ;
      break ;
    case DataErrorLED    :
      IOCLR1 = ( ULONG ) 1 << LED6Pin ;
      break ;
  } /* switch */ ;
} /* LEDOn */


void LEDOff ( LEDSymbolType LEDSymbol )
{
  /*-------------------------------------------------------------------------*/
  switch ( LEDSymbol )
  {
    case StatusLED       :
      IOSET1 = ( ULONG ) 1 << LED0Pin ;
      break ;
    case UART0TxDLED     :
      IOSET1 = ( ULONG ) 1 << LED1Pin ;
      break ;
    case UART0RxDLED     :
      IOSET1 = ( ULONG ) 1 << LED2Pin ;
      break ;
    case UART1TxDLED     :
      IOSET1 = ( ULONG ) 1 << LED3Pin ;
      break ;
    case UART1RxDLED     :
      IOSET1 = ( ULONG ) 1 << LED4Pin ;
      break ;
    case HTTPConnectLED  :
      IOSET1 = ( ULONG ) 1 << LED5Pin ;
      break ;
    case DataErrorLED    :
      IOSET1 = ( ULONG ) 1 << LED6Pin ;
      break ;
  } /* switch */ ;
} /* LEDOff */

               
void InitLEDEnvir ( void )
{
  /*-------------------------------------------------------------------------*/
  PINSEL3 = PINSEL3 & ( ~ 0x0FFFF000 ) ;
  IODIR1 |= ( ( ULONG ) 1 << LED0Pin |
              ( ULONG ) 1 << LED1Pin |
              ( ULONG ) 1 << LED2Pin |
              ( ULONG ) 1 << LED3Pin |
              ( ULONG ) 1 << LED4Pin |
              ( ULONG ) 1 << LED5Pin |
              ( ULONG ) 1 << LED6Pin |
              ( ULONG ) 1 << LED7Pin ) ;
  IOSET1 |= ( ( ULONG ) 1 << LED0Pin |
              ( ULONG ) 1 << LED1Pin |
              ( ULONG ) 1 << LED2Pin |
              ( ULONG ) 1 << LED3Pin |
              ( ULONG ) 1 << LED4Pin |
              ( ULONG ) 1 << LED5Pin |
              ( ULONG ) 1 << LED6Pin |
              ( ULONG ) 1 << LED7Pin ) ;
} /* InitLEDEnvir */
