/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2018                               **
**                                                                        **
**                         MODULE I2C Service                             **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2018.06.12                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include "setupservice.h"
#include "i2cservice.h"
#include "uart0.h"
#include "strings.h"


#define DipSwitchPin0             26
#define DipSwitchPin1             25
#define DipSwitchPin2             24
#define DipSwitchPin3             23

UCHAR SetupStart [ ]              = "\r\nKonfiguracja\r\n" ;
UCHAR EnterMACPrompt [ ]          = "\r\nPodaj nowy MAC:\r\n" ;
UCHAR EnterIPPrompt [ ]           = "\r\nPodaj nowy adres IP:\r\n" ;
UCHAR EnterMaskPrompt [ ]         = "\r\nPodaj nowa maske:\r\n" ;
UCHAR EnterGWPrompt [ ]           = "\r\nPodaj nowy adres bramy domyslnej:\r\n" ;

ConfigEEPROMBlockRecType ConfigEEPROMBlock ;
               
void ReadConfigBlock ( ConfigEEPROMBlockRecPtrType ConfigBlock )
{
  USHORT Loop ;
  UCHAR * ConfigBlockByte ;
  /*-------------------------------------------------------------------------*/
  ConfigBlockByte = ( UCHAR * ) ConfigBlock ;
  for ( Loop = 0 ; Loop < sizeof ( ConfigEEPROMBlockRecType ) ; Loop ++ )
  {
    * ConfigBlockByte = GetEEPROMData ( Loop ) ;
    ConfigBlockByte ++ ;
  } /* for */ ;
} /* ReadConfigBlock */


void WriteConfigBlock ( ConfigEEPROMBlockRecPtrType ConfigBlock )
{
  USHORT Loop ;
  UCHAR * ConfigBlockByte ;
  /*-------------------------------------------------------------------------*/
  ConfigBlockByte = ( UCHAR * ) ConfigBlock ;
  for ( Loop = 0 ; Loop < sizeof ( ConfigEEPROMBlockRecType ) ; Loop ++ )
  {
    SetEEPROMData ( Loop , * ConfigBlockByte ) ;
    ConfigBlockByte ++ ;
  } /* for */ ;
} /* WriteConfigBlock */


UCHAR GetDISPSwitch ( void )
{
  ULONG PortData ;
  UCHAR RetValue ;
  /*-------------------------------------------------------------------------*/
  PortData = FIO3PIN ;
  RetValue = 0 ;
  if ( PortData & ( ( ULONG ) 1 << DipSwitchPin0 ) )
    RetValue |= 0x08 ;
  if ( PortData & ( ( ULONG ) 1 << DipSwitchPin1 ) )
    RetValue |= 0x04 ;
  if ( PortData & ( ( ULONG ) 1 << DipSwitchPin2 ) )
    RetValue |= 0x02 ;
  if ( PortData & ( ( ULONG ) 1 << DipSwitchPin3 ) )
    RetValue |= 0x01 ;
  return ( RetValue ) ;
} /* GetDISPSwitch */

UCHAR ChToHex ( UCHAR Byte )
{
  /*-------------------------------------------------------------------------*/
  if ( Byte == '0' )
    return 0 ;
  if ( Byte == '1' )
    return 1 ;
  if ( Byte == '2' )
    return 2 ;
  if ( Byte == '3' )
    return 3 ;
  if ( Byte == '4' )
    return 4 ;
  if ( Byte == '5' )
    return 5 ;
  if ( Byte == '6' )
    return 6 ;
  if ( Byte == '7' )
    return 7 ;
  if ( Byte == '8' )
    return 8 ;
  if ( Byte == '9' )
    return 9 ;
  if ( Byte == 'A' )
    return 10 ;
  if ( Byte == 'B' )
    return 11 ;
  if ( Byte == 'C' )
    return 12 ;
  if ( Byte == 'D' )
    return 13 ;
  if ( Byte == 'E' )
    return 14 ;
  return 15 ;
} /* ChToHex */


UCHAR Conv ( UCHAR HiByte ,
             UCHAR LoByte )
{
  UCHAR Res ;
  /*-------------------------------------------------------------------------*/
  Res = 16 * ChToHex ( HiByte ) + ChToHex ( LoByte ) ;
  return Res ;
} /* Conv */


void ReadNewMACAddress ( ConfigEEPROMBlockRecPtrType ConfigBlock )
{
  UCHAR MACAddrText [ 16 ] ;
  USHORT CharCounter ;
  UCHAR Ch ;
  /*-------------------------------------------------------------------------*/
  UART0SendString ( SetupStart ) ;
  UART0SendString ( EnterMACPrompt ) ;
  CharCounter = 0 ;
  for ( ; ; )
  {
    if ( UART0DataPresent ( ) )
    {
      Ch = UART0GetData ( ) ;
      if ( ( ( Ch >= '0' ) && ( Ch <= '9' ) ) || ( ( Ch >= 'A' ) && ( Ch <= 'F' ) ) )
      {
        UART0Send ( Ch ) ;
        MACAddrText [ CharCounter ] = Ch ;
        CharCounter ++ ;
        if ( CharCounter >= 12 )
          break ;
      } /* if */ ;
    } /* if */ ;
  } /* for */ ;
  ConfigBlock -> MyMACAddress [ 0 ] = Conv ( MACAddrText [ 0 ] , MACAddrText [ 1 ] ) ;
  ConfigBlock -> MyMACAddress [ 1 ] = Conv ( MACAddrText [ 2 ] , MACAddrText [ 3 ] ) ;
  ConfigBlock -> MyMACAddress [ 2 ] = Conv ( MACAddrText [ 4 ] , MACAddrText [ 5 ] ) ;
  ConfigBlock -> MyMACAddress [ 3 ] = Conv ( MACAddrText [ 6 ] , MACAddrText [ 7 ] ) ;
  ConfigBlock -> MyMACAddress [ 4 ] = Conv ( MACAddrText [ 8 ] , MACAddrText [ 9 ] ) ;
  ConfigBlock -> MyMACAddress [ 5 ] = Conv ( MACAddrText [ 10 ] , MACAddrText [ 11 ] ) ;
} /* ReadNewMACAddress */


ULONG InputIPAddress ( void )
{
  ULONG TmpIP ;
  UCHAR IPString [ 20 ] ;
  USHORT CharCounter ;
  UCHAR Ch ;
  /*-------------------------------------------------------------------------*/
  CharCounter = 0 ;
  for ( ; ; )
  {
    if ( UART0DataPresent ( ) )
    {
      Ch = UART0GetData ( ) ;
      if ( Ch == '\r' )
        break ;
      if ( ( ( Ch >= '0' ) && ( Ch <= '9' ) ) || ( Ch >= '.' ) )
      {
        UART0Send ( Ch ) ;
        IPString [ CharCounter ] = Ch ;
        CharCounter ++ ;
        IPString [ CharCounter ] = 0 ;
        if ( CharCounter >= 16 )
          break ;
      } /* if */ ;
    } /* if */ ;
  } /* for */ ;
  StrToIP ( IPString , & TmpIP ) ;
  return TmpIP ;
} /* InputIPAddress */


void ReadNewIPAddress ( ConfigEEPROMBlockRecPtrType ConfigBlock )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( SetupStart ) ;
  UART0SendString ( EnterIPPrompt ) ;
  ConfigBlock -> MyIPAddress = InputIPAddress ( ) ;
  UART0SendString ( EnterMaskPrompt ) ;
  ConfigBlock -> MyNetMask = InputIPAddress ( ) ;
  UART0SendString ( EnterGWPrompt ) ;
  ConfigBlock -> MyGWAddress = InputIPAddress ( ) ;
} /* ReadNewIPAddress */


void DIPSwitchInitEnvir ( void )
{
  /*-------------------------------------------------------------------------*/
} /* DIPSwitchInitEnvir */


