/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2007                               **
**                                                                        **
**                           MODULE Strings                               **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2007.03.22                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include "strings.h"


static const UCHAR CnvTable [ 16 ] = { '0','1','2','3','4','5','6','7',
                                       '8','9','A','B','C','D','E','F'} ;


void memclear ( void * Buff ,
                USHORT Size )
{
  UCHAR * Byte = ( UCHAR * ) Buff ;
  /*-------------------------------------------------------------------------*/
  while ( Size )
  {
    * Byte ++ = 0 ;
    Size -- ;
  } /* while */ ;
} /* memclear */


void memcopy ( void * DestAdr ,
               void * SourAdr ,
               USHORT Size )
{
  UCHAR * IByte = ( UCHAR * ) SourAdr ;
  UCHAR * OByte = ( UCHAR * ) DestAdr ;
  /*-------------------------------------------------------------------------*/
  while ( Size )
  {
    * OByte ++ = * IByte ++ ;
    Size -- ;
  } /* while */ ;
} /* memcopy */


UCHAR Cap ( UCHAR Ch )
{
  /*-------------------------------------------------------------------------*/
  if ( ( Ch >= 'a' ) && ( Ch <= 'z' ) )
    return ( Ch - 32 ) ;
  else
    return ( Ch ) ;
} /* Cap */


void StrCpy ( UCHAR * DestStr ,
        const UCHAR * SourStr )
{
  /*-------------------------------------------------------------------------*/
  while ( * DestStr ++ = * SourStr ++ )
    ;
} /* StrCpy */


USHORT StrLen ( UCHAR * Str )
{
  UCHAR * SaveStr = Str ;
  /*-------------------------------------------------------------------------*/
  while ( * SaveStr ++ )
    ;
  return ( SaveStr - Str - 1 ) ;
} /* StrLen */


void StrDelete ( UCHAR * String ,
                 USHORT  ChPos ,
                 USHORT  Lgt )
{
  UCHAR * InStr = String + ChPos + Lgt ;
  UCHAR * OutStr = String + ChPos ;
  /*-------------------------------------------------------------------------*/
  while ( * OutStr ++ = * InStr ++ )
    ;
} /* StrDelete */


void StrNorm ( UCHAR * String )
{
  USHORT SpaceCt = 0 ;
  UCHAR * Ch = String ;
  /*-------------------------------------------------------------------------*/
  while ( * Ch ++ == ' ' )
    SpaceCt ++ ;
  if ( SpaceCt )
    StrDelete ( String , 0 , SpaceCt ) ;
  if ( * String )
  {
    while ( * String ++ )
      ;
    String -- ;
    String -- ;
    while ( * String == ' ' )
    {
      * String -- = '\0' ;
    } /* while */ ;
  } /* if */ ;
} /* StrNorm */


void StrExp ( USHORT  MaxSize ,
              UCHAR * String  )
{
  UCHAR * Str = String ;
  /*-------------------------------------------------------------------------*/
  while ( * Str )
  {
    if ( ! MaxSize -- )
      return ;
    Str ++ ;
  } /* while */ ;
  if ( MaxSize == 1 )
  {
    * Str = '\0' ;
  } /* if ... */
  else
  {
    MaxSize -- ;
    while ( MaxSize )
    {
      * Str ++ = ' ' ;
      MaxSize -- ;
    } /* while */ ;
    * Str = '\0' ;
  } /* if ... else */ ;
} /* StrExp */


void AppendCh ( UCHAR * String ,
                UCHAR   AppCh )
{
  /*-------------------------------------------------------------------------*/
  while ( * String )
  {
    String ++ ;
  } /* while */ ;
  * String ++ = AppCh ;
  * String = ( UCHAR ) 0 ;
} /* AppendCh */


void AppendStr ( UCHAR * Str ,
           const UCHAR * AppStr )
{
  UCHAR Ch ;
  /*-------------------------------------------------------------------------*/
  while ( * Str )
  {
    Str ++ ;
  } /* while */ ;
  for ( ; ; )
  {
    Ch = * ( AppStr ++ ) ;
    * ( Str ++ ) = Ch ;
    if ( ! Ch )
      return ;
  } /* loop */ ;
} /* AppendStr */


void Append ( USHORT  MaxSize ,
              UCHAR * String ,
        const UCHAR * SubStr )
{
  /*-------------------------------------------------------------------------*/
  while ( * String )
  {
    if ( ! MaxSize -- )
      return ;
    String ++ ;
  } /* while */ ;
  if ( MaxSize )
  {
    for ( ; ; )
    {
      if ( MaxSize -- == 1 )
      {
        * String = '\0' ;
        return ;
      } /* if */ ;
      if ( ! ( * String ++ = * SubStr ++ ) )
        return ;
    } /* loop */ ;
  } /* if */ ;
} /* Append */


UCHAR * BegOfString ( UCHAR * String )
{
  /*-------------------------------------------------------------------------*/
  while ( ( * String ) == ' ' )
    String ++ ;
  return ( String ) ;
} /* BegOfString */


void WordToHexStr ( UCHAR * Number ,
                    USHORT  ConvWidth ,
                    UCHAR   EmptyCh ,
                    USHORT  CardV )
{
  USHORT Loop ;
  UCHAR LocalBuffer [ 12 ] ;
  /*-------------------------------------------------------------------------*/
  if ( ConvWidth > 7 )
  {
    * Number ++ = '!' ;
    * Number = ( UCHAR ) 0 ;
    return ;
  } /* if */ ;
  for ( Loop = 0 ; Loop < ConvWidth ; Loop ++ )
    LocalBuffer [ Loop ] = EmptyCh ;
  LocalBuffer [ ConvWidth ] = ( UCHAR ) 0 ;
  Loop = ConvWidth - 1 ;
  for ( ; ; )
  {
    LocalBuffer [ Loop ] = CnvTable [ CardV & 0x0F ] ;
    CardV = CardV >> 4 ;
    if ( ( ! Loop ) || ( ! CardV ) )
      break ;
    Loop -- ;
  } /* for */ ;
  for ( Loop = 0 ; Loop <= ConvWidth ; Loop ++ )
    * Number ++ = LocalBuffer [ Loop ] ;
} /* WordToHexStr */


void WordToStr ( UCHAR * Number ,
                 USHORT  ConvWidth ,
                 UCHAR   EmptyCh ,
                 USHORT  Value )
{
  USHORT Loop ;
  UCHAR LocalBuffer [ 12 ] ;
  UCHAR LastDigit ;
  /*-------------------------------------------------------------------------*/
  if ( ConvWidth > 7 )
  {
    * Number ++ = '!' ;
    * Number = ( UCHAR ) 0 ;
    return ;
  } /* if */ ;
  for ( Loop = 0 ; Loop < ConvWidth ; Loop ++ )
    LocalBuffer [ Loop ] = EmptyCh ;
  LocalBuffer [ ConvWidth ] = ( UCHAR ) 0 ;
  Loop = ConvWidth - 1 ;
  for ( ; ; )
  {
    LastDigit = ( UCHAR ) ( Value % 10 ) ;
    LocalBuffer [ Loop ] = LastDigit + ( UCHAR ) 0x30 ;
    Value = Value / 10 ;
    if ( ( ! Loop ) || ( ! Value ) )
      break ;
    Loop -- ;
  } /* for */ ;
  for ( Loop = 0 ; Loop <= ConvWidth ; Loop ++ )
    * Number ++ = LocalBuffer [ Loop ] ;
} /* WordToStr */


void LongToStr ( UCHAR * Number ,
                 USHORT  ConvWidth ,
                 UCHAR   EmptyCh ,
                 ULONG   Value )
{
  USHORT Loop ;
  UCHAR LocalBuffer [ 20 ] ;
  UCHAR LastDigit ;
  /*-------------------------------------------------------------------------*/
  if ( ConvWidth > 16 )
  {
    * Number ++ = ( UCHAR ) '!' ;
    * Number = ( UCHAR ) 0 ;
    return ;
  } /* if */ ;
  for ( Loop = 0 ; Loop < ConvWidth ; Loop ++ )
    LocalBuffer [ Loop ] = EmptyCh ;
  LocalBuffer [ ConvWidth ] = ( UCHAR ) 0 ;
  Loop = ConvWidth - 1 ;
  for ( ; ; )
  {
    LastDigit = ( UCHAR ) ( Value % ( ULONG ) 10 ) ;
    LocalBuffer [ Loop ] = LastDigit + ( UCHAR ) 0x30 ;
    Value = Value / ( ULONG ) 10 ;
    if ( ( ! Loop ) || ( ! Value ) )
      break ;
    Loop -- ;
  } /* for */ ;
  for ( Loop = 0 ; Loop <= ConvWidth ; Loop ++ )
    * Number ++ = LocalBuffer [ Loop ] ;
} /* LongToStr */


void IPToString ( UCHAR * ConvBuff ,
                  ULONG   IPAddress )
{
  UCHAR CnvPart [ 6 ] ;
  UCHAR B0 ;
  UCHAR B1 ;
  UCHAR B2 ;
  UCHAR B3 ;
  /*-------------------------------------------------------------------------*/
  * ConvBuff = 0 ;
  B0 = ( UCHAR ) ( ( IPAddress >> 24 ) & 0xFF ) ;
  B1 = ( UCHAR ) ( ( IPAddress >> 16 ) & 0xFF ) ;
  B2 = ( UCHAR ) ( ( IPAddress >> 8 ) & 0xFF ) ;
  B3 = ( UCHAR ) ( IPAddress & 0xFF ) ;
  WordToStr ( CnvPart , 3 , ( UCHAR ) ' ' , ( USHORT ) B0 ) ;
  AppendStr ( ConvBuff , BegOfString ( CnvPart ) ) ;
  AppendCh ( ConvBuff , '.' ) ;
  WordToStr ( CnvPart , 3 , ( UCHAR ) ' ' , ( USHORT ) B1 ) ;
  AppendStr ( ConvBuff , BegOfString ( CnvPart ) ) ;
  AppendCh ( ConvBuff , '.' ) ;
  WordToStr ( CnvPart , 3 , ( UCHAR ) ' ' , ( USHORT ) B2 ) ;
  AppendStr ( ConvBuff , BegOfString ( CnvPart ) ) ;
  AppendCh ( ConvBuff , '.' ) ;
  WordToStr ( CnvPart , 3 , ( UCHAR ) ' ' , ( USHORT ) B3 ) ;
  AppendStr ( ConvBuff , BegOfString ( CnvPart ) ) ;
} /* IPToString */


UCHAR Digit ( UCHAR Character )
{
  /*-------------------------------------------------------------------------*/
  return ( ( Character >= '0' ) && ( Character <= '9' ) ) ;
} /* Digit */


UCHAR StrToCard ( UCHAR *  String ,
                  USHORT * Result )
{
  USHORT WorkResult ;
  UCHAR Ch ;
  /*-------------------------------------------------------------------------*/
  * Result = 0 ;
  WorkResult = 0 ;
  Ch = * ( String ++ ) ;
  while ( Ch )
  {
    if ( Digit ( Ch ) )
      WorkResult = WorkResult * 10 + ( USHORT ) ( Ch - '0' ) ;
    else
      return ( FALSE ) ;
    Ch = * ( String ++ ) ;
  } /* while */ ;
  * Result = WorkResult ;
  return ( TRUE ) ;
} /* StrToCard */


UCHAR StrToLCard ( UCHAR * String ,
                   ULONG * Result )
{
  ULONG WorkResult ;
  UCHAR Ch ;
  /*-------------------------------------------------------------------------*/
  * Result = 0 ;
  WorkResult = 0 ;
  Ch = * ( String ++ ) ;
  while ( Ch )
  {
    if ( Digit ( Ch ) )
      WorkResult = WorkResult * 10 + ( ULONG ) ( Ch - '0' ) ;
    else
      return ( FALSE ) ;
    Ch = * ( String ++ ) ;
  } /* while */ ;
  * Result = WorkResult ;
  return ( TRUE ) ;
} /* StrToLCard */


UCHAR CompareString ( UCHAR * String1 ,
                      UCHAR * String2 ,
                      USHORT  Size )
{
  UCHAR Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < Size ; Loop ++ )
  {
    if ( ( * String1 ) != ( * String2 ) )
      return ( FALSE ) ;
    String1 ++ ;
    String2 ++ ;
  } /* for */ ;
  return ( TRUE ) ;
} /* CompareString */


UCHAR StrToIP ( UCHAR * IPString ,
                ULONG * IPBinary )
{
  UCHAR WorkNum [ 8 ] ;
  ULONG IPAdr ;
  USHORT AdrPart ;
  UCHAR Loop ;
  UCHAR Cycle ;
  UCHAR Done ;
  UCHAR TermCh ;
  UCHAR Ch = 0 ;
  /*-------------------------------------------------------------------------*/
  IPAdr = 0 ;
  for ( Cycle = 0 ; Cycle < 4 ; Cycle ++ )
  {
    for ( Loop = 0 ; Loop < 8 ; Loop ++ )
      WorkNum [ Loop ] = 0 ;
    for ( Loop = 0 ; Loop < 7 ; Loop ++ )
    {
      Ch = * ( IPString ++ ) ;
      if ( Digit ( Ch ) )
        WorkNum [ Loop ] = Ch ;
      else
        break ;
    } /* for */ ;
    if ( Cycle == 3 )
      TermCh = 0 ;
    else
      TermCh = '.' ;
    if ( Ch != TermCh )
      return ( FALSE ) ;
    Done = StrToCard ( WorkNum , & AdrPart ) ;
    if ( ! Done )
      return ( FALSE ) ;
    if ( AdrPart >= 256 )
      return ( FALSE ) ;
    IPAdr = ( IPAdr << 8 ) + ( ULONG ) AdrPart ;
  } /* for */ ;
  * IPBinary = IPAdr ;
  return ( TRUE ) ;
} /* StrToIP */


