/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2007                               **
**                                                                        **
**                           MODULE SysTime                               **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2007.03.22                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/
#include "systime.h"
#include "types.h"

#define TimerQueueTableSize 16

typedef struct {
                  volatile ULONG    GeneralTimerConter ;
                  TimeQueueRecT     TimeQueue [ TimerQueueTableSize ] ;
                  UCHAR             TimeQueueStatus [ TimerQueueTableSize ] ;
               } SysTimeInstanceRecType ;

static SysTimeInstanceRecType SysTimeInstance ;
               

void SysTimeEnevtService ( void )
{
  /*-------------------------------------------------------------------------*/
  SysTimeInstance . GeneralTimerConter ++ ;
} /* SysTimeEnevtService */


PtrTimeQueueRecT AttachTimer ( ULONG             RelativeAlarmTime ,
                               TimeServiceProcT  CallService ,
                               void *            CallParam )
{
  USHORT Loop ;
  ULONG AlarmTime ;
  PtrTimeQueueRecT QueueElementPtr ;
  /*-------------------------------------------------------------------------*/
  QueueElementPtr = NIL ;
  AlarmTime = SysTimeInstance . GeneralTimerConter + RelativeAlarmTime ;
  if ( ! AlarmTime )
    AlarmTime ++ ;
  for ( Loop = 0 ; Loop < TimerQueueTableSize ; Loop ++ )
  {
    if ( ! SysTimeInstance . TimeQueueStatus [ Loop ] )
    {
      SysTimeInstance . TimeQueueStatus [ Loop ] = 1 ;
      QueueElementPtr = & SysTimeInstance . TimeQueue [ Loop ] ;
      QueueElementPtr -> BellTime = AlarmTime ;
      QueueElementPtr -> CallParam = CallParam ;
      QueueElementPtr -> CallService = CallService ;
      return ( QueueElementPtr ) ;
    } /* if */ ;
  } /* for */ ;
  return ( NIL ) ;
} /* AttachTimer */


void DetachTimer ( PtrTimeQueueRecT TimeQueueElement )
{
  USHORT Loop ;
  PtrTimeQueueRecT QueueElementPtr ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < TimerQueueTableSize ; Loop ++ )
  {
    QueueElementPtr = & SysTimeInstance . TimeQueue [ Loop ] ;
    if ( TimeQueueElement == QueueElementPtr )
    {
      SysTimeInstance . TimeQueueStatus [ Loop ] = 0 ;
      break ;
    } /* if */ ;
  } /* for */ ;
} /* DetachTimer */


void SysTimerPool ( void )
{
  USHORT Loop ;
  PtrTimeQueueRecT QueueElementPtr ;
  ULONG TimeDif ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < TimerQueueTableSize ; Loop ++ )
  {
    if ( SysTimeInstance . TimeQueueStatus [ Loop ] )
    {
      QueueElementPtr = & SysTimeInstance . TimeQueue [ Loop ] ;
      if ( QueueElementPtr -> BellTime )
      {
        TimeDif = QueueElementPtr -> BellTime - SysTimeInstance . GeneralTimerConter ;
        if ( ( TimeDif == 0 ) || ( TimeDif & 0x80000000 ) )
        {
          SysTimeInstance . TimeQueueStatus [ Loop ] = 0 ;
          QueueElementPtr -> BellTime = 0 ;
          QueueElementPtr -> CallService ( QueueElementPtr -> CallParam ) ;
        } /* if */ ;
      } /* if */ ;
    } /* if */ ;
  } /* for */ ;
} /* SysTimerPool */


void InitSysTime ( void )
{
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  SysTimeInstance . GeneralTimerConter = 0 ;
  for ( Loop = 0 ; Loop < TimerQueueTableSize ; Loop ++ )
  {
    SysTimeInstance . TimeQueueStatus [ Loop ] = 0 ;
  } /* for */ ;
} /* InitSysTime */

