/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2007                               **
**                                                                        **
**                       MODULE TCP Protocol Service                      **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2007.03.22                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#ifndef _tcp_service_
#define _tcp_service_

#include "types.h"
#include "ipservice.h"

#define AutoIncrementTCPPortNoLoBound 2048
#define AutoIncrementTCPPortNoUpBound 4095

struct _TCPLayerInstanceRec ;

typedef struct _TCPLayerInstanceRec * PtrTCPLayerInstanceRecT ;

typedef enum { CLOSED        ,
               LISTENING     ,
               SYN_SENT      ,
               SYN_RECEIVED  ,
               ESTABLISHED   ,
               FIN_WAIT_1    ,
               FIN_WAIT_2    ,
               CLOSE_WAIT    ,
               CLOSING       ,
               LAST_ACK      ,
               TIME_WAIT     } TCPStateMachineType ;

struct _TCPSocketDescriptionRec ;

typedef struct _TCPSocketDescriptionRec * PtrTCPSocketDescriptionRecT ;

typedef USHORT ( * TCPConnetOperationProcT ) ( PtrTCPSocketDescriptionRecT /* TCPSocket     */ ,
                                               ULONG                       /* RemoteAddress */ ,
                                               USHORT                      /* RemotePort    */ ) ;

typedef void ( * TCPEstablishOperationProcT ) ( PtrTCPSocketDescriptionRecT /* TCPSocket */ ) ;

typedef void ( * TCPCloseOperationProcT ) ( PtrTCPSocketDescriptionRecT /* TCPSocket */ ) ;

typedef void ( * TCPRecvOperationProcT ) ( PtrTCPSocketDescriptionRecT /* TCPSocket       */ ,
                                           ULONG                       /* SenderIPAddress */ ,
                                           USHORT                      /* SenderPortNo    */ ,
                                           ULONG                       /* DestIPAddress   */ ,
                                           USHORT                      /* DestPortNo      */ ,
                                           void *                      /* DataBuffer      */ ,
                                           USHORT                      /* DataBufferSize  */ ) ;


#define MaxWindowSzie     1024

struct _TCPWindowDescr ;

typedef struct _TCPWindowDescr * PtrTCPWindowDescrRecT ;

typedef struct _TCPWindowDescr {
                 USHORT                      WritePtr ;
                 USHORT                      ReadPtr ;
                 USHORT                      DataLength ;
                 UCHAR                       DataBuffer [ MaxWindowSzie ] ;
                 PtrTCPWindowDescrRecT       FwdLink ;
               } TCPWindowDescrRecT ;

typedef enum { None            ,
               LastSend_FIN    ,
               LastSend_SYN    ,
               LastSend_SYNACC ,
               LastSend_PSH    ,
               LastSend_RST    } TCPLastSendType ;

typedef struct {
                 TCPLastSendType             TCPLastSend ;
                 USHORT                      DelayTime ;
                 USHORT                      Counter ;
               } RetransmRecT ;
typedef RetransmRecT * PtrRetransmRecT ;

typedef struct _TCPSocketDescriptionRec {
                 PtrTCPSocketDescriptionRecT NextSocketLink ;
                 PtrTCPLayerInstanceRecT     TCPLayerInstanceLink ;
                 PtrTCPWindowDescrRecT       WindowLink ;
                 RetransmRecT                RetransmData ;
                 TCPStateMachineType         TCPStateMachine ;              /* Socket state (machine) code */
                 ULONG                       RemoteAddr ;                   /* Remote IP address */
                 ULONG                       LocalAddress ;                 /* Local IP address */
                 USHORT                      RemotePort ;                   /* Remote port number */
                 USHORT                      LocalPort ;                    /* Local port number */
                 ULONG                       SND_UNA ;                      /* SND.UNA - send unacknowledged */
                 ULONG                       SND_NXT ;                      /* SND.NXT - send next */
                 USHORT                      SND_WND ;                      /* SND.WND - send window */
                 USHORT                      SND_UP ;                       /* SND.UP  - send urgent pointer */
                 ULONG                       SND_WL1 ;                      /* SND.WL1 - segment sequence number used for last window update */
                 ULONG                       SND_WL2 ;                      /* SND.WL2 - segment acknowledgment number used for last window update */
                 ULONG                       ISS ;                          /* ISS - initial send sequence number */
                 ULONG                       RCV_NXT ;                      /* RCV.NXT - receive next */
                 USHORT                      RCV_WND ;                      /* RCV.WND - receive window */
                 USHORT                      RCV_UP ;                       /* RCV.UP  - receive urgent pointer */
                 ULONG                       IRS ;                          /* IRS - initial receive sequence number */
                 TCPConnetOperationProcT     ConnectNotify ;
                 TCPEstablishOperationProcT  EstablishNotify ;
                 TCPRecvOperationProcT       ReceiveNotify ;
                 TCPCloseOperationProcT      CloseNotify ;
               } TCPSocketDescriptionRecT ;


typedef struct _TCPLayerInstanceRec
               { PtrIPLayerInstanceRecT       IPLayerInstance ;
                 PtrTCPSocketDescriptionRecT  ListenSocketTopEntry ;
                 PtrTCPSocketDescriptionRecT  TransmSocketTopEntry ;
                 USHORT                       LastLocalPort ;
                 ULONG                        InitSequenceNumber ;
               } TCPLayerInstanceRecT ;

#define TCP_OpenOK                  0
#define TCP_OpenBusy                1

extern USHORT TCPOpen ( PtrTCPSocketDescriptionRecT  /* Socket                   */ ,
                        ULONG                        /* RemoteIP                 */ ,
                        USHORT                       /* RemotePort               */ ,
                        TCPEstablishOperationProcT   /* EstablishCallBackService */ ,
                        TCPRecvOperationProcT        /* RecvCallBackService      */ ,
                        TCPCloseOperationProcT       /* CloseCallBackService     */ ) ;

#define TCP_CloseOK         0
#define TCP_CloseNotConn    1

extern USHORT TCPClose ( PtrTCPSocketDescriptionRecT /* Socket */ ) ;

#define TCPSend_OK         0
#define TCPSend_NoConnect  1
#define TCPSend_Overflow   2

extern USHORT TCPSendData ( PtrTCPSocketDescriptionRecT /* Socket        */ ,
                            void *                      /* DataBufferPtr */ ,
                            USHORT                      /* DataSize      */ ) ;

extern void DeallocateSocket ( PtrTCPSocketDescriptionRecT /* TCPSocket */ ) ;

extern PtrTCPSocketDescriptionRecT CreateTCPSocket ( PtrTCPLayerInstanceRecT /* TCPLayerInstance */ ) ;

#define TCP_ListenPortOK              0
#define TCP_ListenPortBusy            1
#define TCP_ListenPortTableFull       2

extern USHORT TCPListen ( PtrTCPSocketDescriptionRecT  /* TCPSocket                */ ,
                          TCPConnetOperationProcT      /* ConnetCallBackService    */ ,
                          TCPEstablishOperationProcT   /* EstablishCallBackService */ ,
                          TCPRecvOperationProcT        /* RecvCallBackService      */ ,
                          TCPCloseOperationProcT       /* CloseCallBackService     */ ) ;

extern PtrTCPLayerInstanceRecT GetTCPInstance ( PtrTCPSocketDescriptionRecT /* TCPSocket */ ) ;

#define TCP_BindPortOK              0
#define TCP_BindPortDupl            1
#define TCP_BindPortIsOpened        2

extern USHORT BindTCPSocket ( PtrTCPSocketDescriptionRecT /* TCPSocket */ ,
                              USHORT                      /* TCPPortNo */ ) ;

extern void TCPInitInstance ( PtrTCPLayerInstanceRecT /* TCPLayerInstance */ ,
                              PtrIPLayerInstanceRecT  /* IPLayerInstance  */ ) ;

extern PtrIPLayerInstanceRecT GetIPInstanceTCP ( PtrTCPLayerInstanceRecT /* TCPLayerInstance */ ) ;

extern PtrTCPLayerInstanceRecT StandardAddTCPInstance ( PtrIPLayerInstanceRecT /* IPLayerInstance */ ) ;

#endif
