/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2018                               **
**                                                                        **
**                          MODULE WEBServer                              **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2018.06.12                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.H>
#include "system.h"
#include "types.h"
#include "systime.h"
#include "irqvectors.h"
#include "serialconst.h"
#include "uart0.h"
#include "uart1.h"
#include "strings.h"
#include "emacdriver.h"
#include "ipservice.h"
#include "tcpservice.h"
#include "httpservice.h"
#include "dataservice.h"
#include "leddisplay.h"
#include "xramservice.h"
#include "i2cservice.h"
#include "setupservice.h"

#define IRQCallCntLimit                 100

#define MagicSygnature                  0xAA55

typedef struct {
                 EMACInstanceRecT *          EMACLayer ;
                 IPLayerInstanceRecT *       IPLayer ;
                 TCPLayerInstanceRecT *      TCPLayer ;
                 USHORT                      LEDState ;
                 PtrTCPSocketDescriptionRecT TCPSocket ;
                 ConfigEEPROMBlockRecType    ConfigBlock ;
               } WebSeverInstanceRecT ;
typedef WebSeverInstanceRecT * PtrWebSeverInstanceRecT ;

static WebSeverInstanceRecT WebSeverInstance ;
static ULONG IRQCallCounter ;

#define MachineIPAddress        0xC0A8002C
////                              ^^^ 192.168.0.44
#define MachineIPSubnet         0xFFFFFF00
//                              ^^^ 255.255.255.0
#define MachineGWIPAddress      0xC0A800FE
//                              ^^^ 192.168.0.254
#define TCPPortNumber           80


const UCHAR MachineMACAddr [ 6 ]     = { 0x00 , 0x03 , 0x64 , 0xA1 , 0x00 , 0x01 } ;


static __irq void T1_IRQHandler ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter ++ ;
  if ( IRQCallCounter >= IRQCallCntLimit )
  {
    IRQCallCounter = 0 ;
    if ( WebSeverInstance . LEDState )
    {
      LEDOn ( StatusLED ) ;
      WebSeverInstance . LEDState = 0 ;
    } /* if ... */
    else
    {
      LEDOff ( StatusLED ) ;
      WebSeverInstance . LEDState = 1 ;
    } /* if ... else */
  } /* if */ ;
  if ( UART0TxdLEDDelayCounter )
  {
    UART0TxdLEDDelayCounter -- ;
    if ( ! UART0TxdLEDDelayCounter )
      LEDOff ( UART0TxDLED ) ;
  } /* if */ ;
  if ( UART0RxDLEDDelayCounter )
  {
    UART0RxDLEDDelayCounter -- ;
    if ( ! UART0RxDLEDDelayCounter )
      LEDOff ( UART0RxDLED ) ;
  } /* if */ ;
  if ( UART1TxdLEDDelayCounter )
  {
    UART1TxdLEDDelayCounter -- ;
    if ( ! UART1TxdLEDDelayCounter )
      LEDOff ( UART1TxDLED ) ;
  } /* if */ ;
  if ( UART1RxDLEDDelayCounter )
  {
    UART1RxDLEDDelayCounter -- ;
    if ( ! UART1RxDLEDDelayCounter )
      LEDOff ( UART1RxDLED ) ;
  } /* if */ ;
  T1IR = 1 ;
  VICVectAddr = 0 ;
} /* T1_IRQHandler */


static void HardwareInit ( void ) 
{
  /*-------------------------------------------------------------------------*/
  InitXRAM ( ) ;
  PINSEL10 = 0 ;
  T1MR0 = ( PCLK / 1000 ) - 1 ;
  T1MCR = 3 ;
  T1TCR = 1 ;
  VICVectAddr5 = ( unsigned long ) T1_IRQHandler ;
  VICVectCntl5 = 15 ;
  VICIntEnable = ( 1 << TIMER1_Interrupt ) ;
  UART0InitEnvir ( ) ;
  UART1InitEnvir ( ) ;
  InitLEDEnvir ( ) ;
  I2CInitEnvir ( ) ;
} /* HardwareInit */


static void StartNet ( PtrWebSeverInstanceRecT Instance )
{
  EtherSpeedMode EthMode ;
  UCHAR LocalMyMACAddr [ 6 ] ;
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < 6 ; Loop ++ )
    LocalMyMACAddr [ Loop ] = WebSeverInstance . ConfigBlock . MyMACAddress [ Loop ] ;
  EthMode = Mode_AutoNeg ;
//  EthMode = Mode_10MBIT ;
//  EthMode = Mode_100MBIT ;
  Instance -> EMACLayer = InitPHYLayerInstance ( LocalMyMACAddr , EthMode ) ;
  Instance -> IPLayer = InitIPLayerInstance ( Instance -> EMACLayer ,
                                              WebSeverInstance . ConfigBlock . MyIPAddress ,
                                              WebSeverInstance . ConfigBlock . MyNetMask ) ;
  Instance -> IPLayer -> MyDefaultGWIPAddress = WebSeverInstance . ConfigBlock . MyGWAddress ;
  Instance -> TCPLayer = StandardAddTCPInstance ( Instance -> IPLayer ) ;
} /* StartNet */


UCHAR Hello1 [ ]      = "\r\nHello: Serwer stron www\r\n" ;
UCHAR Hello2 [ ]      = "\r\nMoj adres IP=" ;
UCHAR Hello3 [ ]      = "\r\nMaska podsieci=" ;
UCHAR Hello4 [ ]      = "\r\nAdres bramy domyslnej=" ;
UCHAR Hello6 [ ]      = "\r\nAdres MAC=" ;
UCHAR Hello7 [ ]      = "\r\n************************\r\n" ;
UCHAR Error1 [ ]      = "\r\nProblem z gniazdkiem\r\n" ;
UCHAR Stopped [ ]     = "\r\nProgram zatrzymany po rekonfiguracji\r\n" ;

static void HelloMessage ( void )
{
  UCHAR NumberStr [ 32 ] ;
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Hello1 ) ;
  UART0SendString ( Hello2 ) ;
  IPToString ( NumberStr , WebSeverInstance . ConfigBlock . MyIPAddress ) ;
  UART0SendString ( NumberStr ) ;
  UART0SendString ( Hello3 ) ;
  IPToString ( NumberStr , WebSeverInstance . ConfigBlock . MyNetMask ) ;
  UART0SendString ( NumberStr ) ;
  UART0SendString ( Hello4 ) ;
  IPToString ( NumberStr , WebSeverInstance . ConfigBlock . MyGWAddress ) ;
  UART0SendString ( NumberStr ) ;
  UART0SendString ( Hello6 ) ;
  WordToHexStr ( NumberStr , 2 , '0' , ( USHORT ) WebSeverInstance . ConfigBlock . MyMACAddress [ 0 ] ) ;
  UART0SendString ( NumberStr ) ;
  UART0Send ( '-' ) ;
  WordToHexStr ( NumberStr , 2 , '0' , ( USHORT ) WebSeverInstance . ConfigBlock . MyMACAddress [ 1 ] ) ;
  UART0SendString ( NumberStr ) ;
  UART0Send ( '-' ) ;
  WordToHexStr ( NumberStr , 2 , '0' , ( USHORT ) WebSeverInstance . ConfigBlock . MyMACAddress [ 2 ] ) ;
  UART0SendString ( NumberStr ) ;
  UART0Send ( '-' ) ;
  WordToHexStr ( NumberStr , 2 , '0' , ( USHORT ) WebSeverInstance . ConfigBlock . MyMACAddress [ 3 ] ) ;
  UART0SendString ( NumberStr ) ;
  UART0Send ( '-' ) ;
  WordToHexStr ( NumberStr , 2 , '0' , ( USHORT ) WebSeverInstance . ConfigBlock . MyMACAddress [ 4 ] ) ;
  UART0SendString ( NumberStr ) ;
  UART0Send ( '-' ) ;
  WordToHexStr ( NumberStr , 2 , '0' , ( USHORT ) WebSeverInstance . ConfigBlock . MyMACAddress [ 5 ] ) ;
  UART0SendString ( NumberStr ) ;
  UART0SendString ( Hello7 ) ;
} /* HelloMessage */


static void SoftwareInit ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter = 0 ;
  WebSeverInstance . EMACLayer = NIL ;
  WebSeverInstance . IPLayer = NIL ;
  WebSeverInstance . TCPLayer = NIL ;
  WebSeverInstance . LEDState = 0 ;
  WebSeverInstance . TCPSocket = NIL ;
  InitDataService ( ) ;
} /* SoftwareInit */


static void SetDefaultEEPROMData ( void )
{
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  WebSeverInstance . ConfigBlock . Sygnature = MagicSygnature ;
  WebSeverInstance . ConfigBlock . MyIPAddress = MachineIPAddress ;
  WebSeverInstance . ConfigBlock . MyNetMask = MachineIPSubnet ;
  WebSeverInstance . ConfigBlock . MyGWAddress = MachineGWIPAddress ;
  for ( Loop = 0 ; Loop < 6 ; Loop ++ )
  {
    WebSeverInstance . ConfigBlock . MyMACAddress [ Loop ] = MachineMACAddr [ Loop ] ;
  } /* for */ ;
  WriteConfigBlock ( & WebSeverInstance . ConfigBlock ) ;
} /* SetDefaultEEPROMData */

    
static void GetConfigData ( void )
{
  /*-------------------------------------------------------------------------*/
  ReadConfigBlock ( & WebSeverInstance . ConfigBlock ) ;
  if ( WebSeverInstance . ConfigBlock . Sygnature != MagicSygnature )
  {
    SetDefaultEEPROMData ( ) ;
  } /* if */ ;
} /* GetConfigData */  


static void Halt ( UCHAR * Message )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Message ) ;
  for ( ; ; )
  {
    NOP ( ) ;
  } /* loop */ ;
} /* Halt */


static void DIPSwitchSerwice ( void )
{
  UCHAR DIPSwitchState ;
  /*-------------------------------------------------------------------------*/
  DIPSwitchState = GetDISPSwitch ( ) ;
  if ( DIPSwitchState & 0x08 )
  {
    SetDefaultEEPROMData ( ) ;
    Halt ( Stopped ) ;
  } /* if */ ;
  if ( DIPSwitchState & 0x04 )
  {
    ReadNewMACAddress ( & WebSeverInstance . ConfigBlock ) ;
    WebSeverInstance . ConfigBlock . Sygnature = MagicSygnature ;
    WriteConfigBlock ( & WebSeverInstance . ConfigBlock ) ;
    Halt ( Stopped ) ;
  } /* if */ ;
  if ( DIPSwitchState & 0x02 )
  {
    ReadNewIPAddress ( & WebSeverInstance . ConfigBlock ) ;
    WebSeverInstance . ConfigBlock . Sygnature = MagicSygnature ;
    WriteConfigBlock ( & WebSeverInstance . ConfigBlock ) ;
    Halt ( Stopped ) ;
  } /* if */ ;
} /* DIPSwitchSerwice */


int main ( void )
{
  /*-------------------------------------------------------------------------*/
  SysInit ( ) ;
  InitSysTime ( ) ;
  HardwareInit ( )  ;
  SoftwareInit ( ) ;
  UART0HardwInit( FOSC , N81Mode , 9600 ) ;
  UART1HardwInit( FOSC , N81Mode , 9600 ) ;
  GetConfigData ( ) ;
  DIPSwitchSerwice ( ) ;
  HelloMessage ( ) ;
  StartNet ( & WebSeverInstance ) ;
  WebSeverInstance . TCPSocket = CreateTCPSocket ( WebSeverInstance . TCPLayer ) ;
  if ( BindTCPSocket ( WebSeverInstance . TCPSocket , TCPPortNumber ) != TCP_BindPortOK )
  {
    Halt ( Error1 ) ;
  } /* if */ ;
  if ( TCPListen ( WebSeverInstance . TCPSocket , TCPConnectOperation , TCPEstablishOperation ,
                   TCPRecvOperation , TCPCloseOperation ) != TCP_ListenPortOK )
  {
    Halt ( Error1 ) ;
  } /* if */ ;
  InitWebService ( ) ;
  for ( ; ; )
  {
    SysTimerPool ( ) ;
    WebSeverInstance . IPLayer -> IPLayerPoolService ( WebSeverInstance . IPLayer ) ;
    SerialPool ( ) ;
  } /* loop */ ;
} /* main */

