/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                           BIALYSTOK 2007                               **
**                                                                        **
**                         MODULE XRAMService                             **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2018.12.09                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/
 
#include <LPC23xx.H>
#include "xramservice.h"
#include "types.h"

#define ExternalRAMBase                0x80000000

#define ElementsQnt                    32

typedef struct {
                 ULONG                  StatusArray [ ElementsQnt ] ;
                 TCPWindowDescrRecT     WindowStack [ ElementsQnt ] ;
               } ExternalRAMStructureType ;
typedef ExternalRAMStructureType * ExternalRAMStructurePtrType ;

static ExternalRAMStructurePtrType ExternalRAMPtr ;

PtrTCPWindowDescrRecT AllocateWindowXRAM ( void )
{
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < ElementsQnt ; Loop ++ )
  {
    if ( ! ExternalRAMPtr -> StatusArray [ Loop ] )
    {
      ExternalRAMPtr -> StatusArray [ Loop ] = TRUE ;
      return ( & ExternalRAMPtr -> WindowStack [ Loop ] ) ;
    } /* if */ ;
  } /* for */ ;
  return ( NIL ) ;
} /* AllocateWindowXRAM */


void DeallocateWindowXRAM ( PtrTCPWindowDescrRecT Window )
{
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < ElementsQnt ; Loop ++ )
  {
    if ( & ExternalRAMPtr -> WindowStack [ Loop ] == Window )
    {
      ExternalRAMPtr -> StatusArray [ Loop ] = 0 ;
      break ;
    } /* if */ ;
  } /* for */ ;
} /* DeallocateWindowXRAM */


void InitXRAM ( void )
{
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  PINSEL6 = 0x00005555 ; /* D0 .. D7 */
  PINSEL8 = 0x55555555 ; /* A0 .. A15 */
  PINSEL9 = 0x50090000 ; /* OE, BLS0, CS0, CS1 */
  ExternalRAMPtr = ( ExternalRAMStructurePtrType ) ExternalRAMBase ;
  for ( Loop = 0 ; Loop < ElementsQnt ; Loop ++ )
  {
    ExternalRAMPtr -> StatusArray [ Loop ] = FALSE ;
  } /* for */ ;
} /* InitXRAM */

